<?php

// Obtiene el contenedor por su identificador
function get_container($container_id) {
    global $db;
    $query = '
        SELECT * FROM contenedores WHERE contenedor_id = :container_id';    
    try {
        $statement = $db->prepare($query);
        $statement->bindValue(':container_id', $container_id);
        $statement->execute();
        $result = $statement->fetch();
        $statement->closeCursor();
        return $result;
    } catch (PDOException $e) {
        $error_message = $e->getMessage();
        display_db_error($error_message);
    }
}

// Obtiene los contenedores situados en una ubicación determinada
function get_containers_by_ubicacion($ubicacion_id) {
    global $db;
    $query = 'SELECT * FROM contenedores WHERE ubicacion_id = :ubicacion_id';   
    try {
        $statement = $db->prepare($query);
        $statement->bindValue(':ubicacion_id', $ubicacion_id);
        $statement->execute();
        $result = $statement->fetchAll();
        $statement->closeCursor();
        return $result;
    } catch (PDOException $e) {
        $error_message = $e->getMessage();
        display_db_error($error_message);
    }
}

// Obtiene un modelo de contenedor a través de su identificador
function get_modelo($modelo_id) {
    global $db;
    $query = '
        SELECT * FROM modelos WHERE modelo_id = :modelo_id';    
    try {
        $statement = $db->prepare($query);
        $statement->bindValue(':modelo_id', $modelo_id);
        $statement->execute();
        $result = $statement->fetch();
        $statement->closeCursor();
        return $result;
    } catch (PDOException $e) {
        $error_message = $e->getMessage();
        display_db_error($error_message);
    }
}

// Obtiene el modelo de contenedor al que
// pertenece el contenedor definido por su 'contenedor_id
function get_modelo_by_contenedor($contenedor_id) {
    global $db;
    $query = 'SELECT * 
             FROM modelos M
             INNER JOIN contenedores C ON M.modelo_id = C.modelo_id
             WHERE C.contenedor_id = :contenedor_id';          
    try {
        $statement = $db->prepare($query);
        $statement->bindValue('contenedor_id', $contenedor_id);
        $statement->execute();
        $result = $statement->fetch();
        $statement->closeCursor();
        return $result;
    } catch (PDOException $e) {
        $error_message = $e->getMessage();
        display_db_error($error_message);
    }
}


// Obtiene los modelos de contenedor existentes
function get_modelos_contenedor() {
    global $db;
    $query = 'SELECT * FROM modelos m ORDER BY m.denominacion';         
    try {
        $statement = $db->prepare($query);
        $statement->execute();
        $result = $statement->fetchAll();
        $statement->closeCursor();
        return $result;
    } catch (PDOException $e) {
        $error_message = $e->getMessage();
        display_db_error($error_message);
    }
}

// Actualiza algunos campos del 'modelo' definido 
// por su 'modelo_id' en la tabla 'modelos'
function update_modelo($modelo_id, $periodo_act_datos, $periodo_act_nivel, 
                       $alarma_nivel, $alarma_manto) {
    global $db;
    $query = 'UPDATE modelos
              SET periodo_actualizacion_datos = :periodo_act_datos,
                  periodo_medida_nivel = :periodo_act_nivel,
                  porcentaje_alarma_nivel = :alarma_nivel,
                  alarma_mantenimiento = :alarma_manto
              WHERE modelo_id = :modelo_id';   
    try {
        $statement = $db->prepare($query);
        $statement->bindValue(':periodo_act_datos', $periodo_act_datos);
        $statement->bindValue(':periodo_act_nivel', $periodo_act_nivel);
        $statement->bindValue(':alarma_nivel', $alarma_nivel);
        $statement->bindValue(':alarma_manto', $alarma_manto);
        $statement->bindValue(':modelo_id', $modelo_id);
        $statement->execute();
        $statement->closeCursor();
    } catch (PDOException $e) {
        $error_message = $e->getMessage();
        display_db_error($error_message);
    }
}

// Obtiene el contenedor por su dirección IP
function get_contenedor_by_ip($container_ip) {
    global $db;
    $query = '
        SELECT * FROM contenedores WHERE ip = :container_ip';    
    try {
        $statement = $db->prepare($query);
        $statement->bindValue(':container_ip', $container_ip);
        $statement->execute();
        $result = $statement->fetch();
        $statement->closeCursor();
        return $result;
    } catch (PDOException $e) {
        $error_message = $e->getMessage();
        display_db_error($error_message);
    }
}

// Actualiza la fecha del último mantenimiento en el 
// contenedor pasado por su id
function update_contenedor_manto($contenedor_id)
{
    global $db;
    $query = 'UPDATE contenedores
              SET ultimo_mantenimiento = :fecha_hora
              WHERE contenedor_id = :contenedor_id'; 
    
    // la fecha_hora del registro, se puede añadir directamente desde mysql
    $mysqldate = date('Y-m-d H:i:s', time());
    try {
        $statement = $db->prepare($query);
        $statement->bindValue(':contenedor_id', $contenedor_id);
        $statement->bindValue(':fecha_hora', $mysqldate);
        $statement->execute();
        $statement->closeCursor();
    } catch (PDOException $e) {
        $error_message = $e->getMessage();
        display_db_error($error_message);
    }    
}

// Devuelve el número de recogidas acumuladas desde el último mantenimiento
// del contenedor identificado por 'contenedor_id'
function get_recogidas_acumuladas($contenedor_id) {       
    global $db;
    $query = 'SELECT COUNT(*) AS recogidasCount
             FROM niveles N
             INNER JOIN contenedores C
             ON N.fecha_hora > C.ultimo_mantenimiento
             WHERE N.es_recogida = 1 AND
                   N.contenedor_id = :contenedor_id';  
    try {
        $statement = $db->prepare($query);
        $statement->bindValue(':contenedor_id', $contenedor_id);
        $statement->execute();
        $recogidas = $statement->fetch();
        $recogidas_count = $recogidas['recogidasCount'];
        $statement->closeCursor();
        return $recogidas_count;
    } catch (PDOException $e) {
        $error_message = $e->getMessage();
        display_db_error($error_message);
    }           
}

// Alarma de mantenimiento = (descargas realizadas desde 
// la ultima fecha de mantenimiento > descargas programadas).
// Devuelve 'true' si el contenedor 'contenedor_id' está en alarma 
// false en caso contrario
function is_activa_alarma_manto($contenedor_id) {  
    
    // recogidas acumuladas por el contenedor desde el último mantenimiento
    $recogidas_acumuladas = get_recogidas_acumuladas($contenedor_id);
    
    // valor prefijado de alarma de mantenimiento para el modelo 
    // al que pertenece el contenedor
    $modelo_id = get_container($contenedor_id)['modelo_id'];
    $modelo_contenedor = get_modelo($modelo_id);
    
    $alarma_manto = ($recogidas_acumuladas > $modelo_contenedor['alarma_mantenimiento']);   
    
    return $alarma_manto;
}

// Actualiza la última temperatura y su estado de alarma recibidas desde 
// la unidad sensora para el contenedor 'contenedor_id'
function update_temper($contenedor_id, $temper, $alarm) 
{
    global $db;
    $query = 'UPDATE contenedores
              SET temperatura_actual = :temper, alarma_temper = :alarm,
                  ultima_actualizacion = :fecha_hora
              WHERE contenedor_id = :contenedor_id';      
    
    // la fecha_hora del registro, se puede añadir directamente desde mysql
    $mysqldate = date('Y-m-d H:i:s', time());
    try {
        $statement = $db->prepare($query);
        $statement->bindValue(':contenedor_id', $contenedor_id);
        $statement->bindValue(':temper', $temper);
        $statement->bindValue(':alarm', $alarm);
        $statement->bindValue(':fecha_hora', $mysqldate);
        $statement->execute();
        $statement->closeCursor();
    } catch (PDOException $e) {
        $error_message = $e->getMessage();
        display_db_error($error_message);
    }    
}

// Actualiza el último dato de verticalidad recibido desde 
// la unidad sensora para el contenedor 'contenedor_id'
function update_vertic($contenedor_id, $vertic, $alarm) 
{
    global $db;
    $query = 'UPDATE contenedores
              SET verticalidad_actual = :vert, alarma_vertical = :alarm,
                  ultima_actualizacion = :fecha_hora
              WHERE contenedor_id = :contenedor_id';      
    
    // la fecha_hora del registro, se puede añadir directamente desde mysql
    $mysqldate = date('Y-m-d H:i:s', time());
    try {
        $statement = $db->prepare($query);
        $statement->bindValue(':contenedor_id', $contenedor_id);
        $statement->bindValue(':vert', $vertic);
        $statement->bindValue(':alarm', $alarm);
        $statement->bindValue(':fecha_hora', $mysqldate);
        $statement->execute();
        $statement->closeCursor();
    } catch (PDOException $e) {
        $error_message = $e->getMessage();
        display_db_error($error_message);
    }    
}
