<?php

    require_once('../util/main.php');
    require_once('main.php');
                
    /***************************************************************
     * Descripción del protocolo.
     * La comunicación la inicia siempre la unidad sensora.
     * 
     * Para los CRCs se utiliza el Polinomio CRC 16 XModem. 
     * Ecuación: x^16 + x^12 + x^5 + 1. 
     * Valor inicial = 0x0000. 
     **************************************************************/

    /*** Envio de datos de proceso desde la unidad sensora al host ***/
    //
    // action = send
    // 
    // Datos:
    // 
    // dir_ip   IP de la unidad sensora codificado en unsigned 32 bits.               
    //                       
    // nivel    Opcional, (coma fija signed: -3276.8 a 3276.7 cms) 
    //          
    // temper   Opcional, (coma fija signed: -50.0 a 150.0 ºC) 
    //           
    // vertic   Opcional, (coma fija signed: -180.0 a 180.0 º)  
    //                   
    // alarm    Opcional, (bits: 7=6=5=4=3=2=NA, 1:verticalidad, 0:temperatura)
    //                    bit correspondiente = 1 -> alarma.   
    //            
    // es_recog Opcional, (bits: 7=6=5=4=3=2=1=NA, 0:contenedor descargado) 
    //                    bit0 = 1 -> contenedor descargado.
    //                       
    // ack      Opcional.
    //          Si clave presente y valor = 1, el envío requiere respuesta 
    //          de reconocimiento desde el host. 
    //          La unidad sensora queda en espera del ack (con un time-out).
    //          Si no recibe el ack o recibe error de CRC (ver más abajo)
    //          vuelve a enviar los datos pasado 1 minuto.
    //          
    // crc      Siempre. Calculado desde dir_ip (icluido) 
    //          hasta el byte anterior (incluido) al CRC.
    //          
    // "dir_ip=<ip del sensor>[&nivel=<valor>][&temper=<valor>][&vertic=<valor>]
    //  [&alarm=<valor>][&es_recog=<valor>][&ack=1]
    //  &crc=<crc calculado por el sensor>"                
    //
    // Si ack = 1 y el CRC calculado por el host es igual que el enviado por la 
    // unidad sensora y la dirección IP del sensor está dada de alta en el host,
    // éste responde con:
    // 
    // dir_ip   Siempre, IP de la unidad sensora codificado en unsigned 32 bits.
    // 
    // crc      Siempre, retorna el CRC recibido. 
    // 
    // "Respuesta desde el Host: &dir_ip=<ip del sensor>&crc=<crc recibido>&"
    //
    // Si ack = 1 y el CRC calculado por el host es distinto del enviado por 
    // la unidad sensora, el host responde con error de CRC:
    //
    // "Respuesta desde el Host: &crc=???&"
    // 
    // Si ack = 1 y la dirección IP del sensor no está dada de alta en el host, 
    // éste responde con error de identificación:
    // 
    // "Respuesta desde el Host: &id=???&"
    // 
    // Con independencia del ack, si hay error de CRC o de identificación, 
    // el host no incorpora los datos enviados desde la unidad sensora.
    //
    /*------------------------------------------------------------*/


    /*** Petición de envio de datos de la unidad sensora al host ***/
    //
    // action = request
    // 
    // Datos:
    //
    // dir_ip       Siempre, IP de la unidad sensora codificado. 
    // 
    // get_per_actualiza    Opcional.
    //                      Si clave presente y valor = 1, se incluye 
    //                      petición de envio del periodo de petición de 
    //                      envio de datos (este periodo). 
    //                         
    // get_per_nivel        Opcional.
    //                      Si clave presente y valor = 1, se incluye petición 
    //                      de envio del periodo de actualización del nivel.        
    //
    // crc          Siempre. Calculado desde dir_ip (icluido) 
    //              hasta el byte anterior (incluido) al CRC.
    //              
    //  "dir_ip=<ip del sensor>[&get_per_actualiza=1][&get_per_nivel=1]
    //   &crc=<crc calculado por el sensor>"
    // 
    // La unidad sensora hace esta petición al host después de un corte 
    // de alimentación (ejem. en la primera puesta en servicio) o una vez 
    // transcurrido el periodo de petición de envio de datos.
    // Queda en espera de los datos solicitados (con un time-out). 
    // Si no recibe los datos o recibe un error de CRC, vuelve a hacer la 
    // petición pasado 1 minuto.
    // Hasta que no recibe los datos (de ajuste), no envía valores de proceso. 
    //
    // Si el CRC calculado por el host es igual que el enviado por la unidad 
    // sensora y la dirección IP del sensor está dada de alta en el host, 
    // éste responde con:  
    // 
    // dir_ip   Siempre, IP de la unidad sensora codificado.
    // 
    // set_per_actualiza    Si lo ha solicitado la unidad sensora,  
    //                      (0 a 65535 minutos).
    //                      Periodo de petición de envio de datos.
    //                      
    // set_per_nivel        Si lo ha solicitado la unidad sensora.  
    //                      (unsigned: 0 a 65535 minutos).
    //                      Periodo de actualización del nivel.
    //                                     
    // crc      Siempre. Calculado desde dir_ip (icluido) 
    //          hasta el byte anterior (incluido) al CRC.
    //
    // "Respuesta desde el Host: &dir_ip=<ip del sensor>
    // [&set_per_actualiza=<valor>][&set_per_nivel=<valor>]
    // &crc=<crc calculado por el host>&"
    // 
    // Si el CRC calculado por el host es distinto al enviado por 
    // la unidad sensora, el host responde con error de CRC:
    // 
    // "Respuesta desde el Host: &crc=???&"
    // 
    // Si la dirección IP del sensor no está dada de alta en el host, 
    // éste responde con error de identificación:
    // 
    // "Respuesta desde el Host: &id=???&"
    //
    /*------------------------------------------------------------*/    
   
    // inicio de sesión unidad sensora
    session_start();
    
    // action ?
    $action = strtolower(filter_input(INPUT_GET, 'action'));
    
    // comprobamos el CRC     
    foreach (filter_input_array(INPUT_GET) as $clave => $valor) {
        // quitamos de la cadena el campo inicial 'action' 
        if (strpos($clave, 'action') !== false) {continue;} 
        // quitamos de la cadena el campo crc
        if (strpos($clave, 'crc') !== false) {continue;}            
        // quitamos de la cadena la info del debuger en caso de debug
        if (strpos($clave, 'XDEBUG') !== false) {continue;}

        // añadimos clave/valor
        $crc_data .= ($clave . '=' . $valor . '&');
    } 

    $data_clear = rtrim($crc_data, '&'); // se suprime el '&' final

    // CRC recibido ?
    $crc_rcv = (int)strtolower(filter_input(INPUT_GET, 'crc'));

    // Calculamos el CRC con los datos adecuados (ver memoria del PFC).
    // Polinomio CRC 16 XModem. Ecuación: x^16 + x^12 + x^5 + 1. 
    // Valor inicial = 0x0000.        
    // Si error CRC, se retorna el error y no se procesan los datos recibidos
    if($crc_rcv != crc16($data_clear, 0x0000)) {           
        echo "Respuesta desde el Host: &crc=???&";                    
        return;           
    }     
     
    if($action == 'send') { // action = "send" -> Envio de datos desde la unidad sensora        
    
        foreach (filter_input_array(INPUT_GET) as $clave => $valor) {        
            
            switch ($clave)
            {
                case 'dir_ip':
                    $dir_ip = filter_input(INPUT_GET, 'dir_ip');
                    break;
                case 'nivel':
                    $nivel = filter_input(INPUT_GET, 'nivel');
                    break;                
                case 'temper':
                    $temper = filter_input(INPUT_GET, 'temper');
                    break;                
                case 'vertic':
                    $vertic = filter_input(INPUT_GET, 'vertic');
                    break;
                case 'alarm':
                    $alarm = filter_input(INPUT_GET, 'alarm');
                    break;               
                case 'es_recog':
                    $es_recog = filter_input(INPUT_GET, 'es_recog');
                    break; 
                case 'ack':
                    $ack = filter_input(INPUT_GET, 'ack');
                    break;
            }
        }

        receive_new_data_from_sensor($dir_ip,  $nivel, $temper, $vertic, 
                                     $alarm, $es_recog, $ack, $crc_rcv);
    }
    else if($action == 'request') { // action = "request" -> Peticion de envio de  
                                    // datos de ajuste desde la unidad sensora

        foreach (filter_input_array(INPUT_GET) as $clave => $valor) {
            
            switch ($clave)
            {
                case 'dir_ip':
                    $dir_ip = filter_input(INPUT_GET, 'dir_ip');
                    break;
                case 'get_per_actualiza':
                    $mem = filter_input(INPUT_GET, 'get_per_actualiza');
                    if($mem != null) {$get_per_actualiza = $mem;}
                    break;
                case 'get_per_nivel':
                    $mem = filter_input(INPUT_GET, 'get_per_nivel');
                    if($mem != null) {$get_per_nivel = $mem;}                    
                    break;                                          
            }
        }
        
        send_settings_to_sensor($dir_ip, $get_per_actualiza, $get_per_nivel);
    } 
    else {  // action = desconocida -> Entrada inválida 
        echo "Respuesta desde el Host: &action=???&";          
        return;
    };

?>
