/*
 * gpio.h
 *
 * Este módulo incorpora las funciones necesarias para trabajar y separar de la aplicación
 * el GPIO del microcontrolador.
 *
 * author: Miguel A. Fdez.
 *
 * revision 1.0 date 3/11/2015
 */

#ifndef GPIO_H_
#define GPIO_H_

#include "type.h"

// GPIO:
#define  GPIO_INPUT 	0	// Pin es input
#define  GPIO_OUTPUT 	1	// Pin es output

#define  GPIO_PULLUP  	0	// Pin con resistencia pull-up
#define  GPIO_REPEATER  1	// Pull-up si el pin = 1,  Pull-Down si el pin = 0
#define  GPIO_NOPULL  	2	// Pin sin resistencias pull-up ni pull-down
#define  GPIO_PULLDOWN  3	// Pin con resistencia pull-down

/** Configura el pin como entrada o como salida
 *
 * @precondition 	El pin está configurado como GPIO.
 *
 * @param portNum   Número de puerto al que pertenece el pin (0-4).
 * @param bitPos    Posición del pin (bit) dentro del puerto.
 * @param dir   	Dirección del pin. entrada: 0 o 'INPUT', salida: 1 o 'OUTPUT'.
 *
 * @return No
 */
void GPIO_setDir(uint32_t portNum, uint32_t bitPos, uint32_t dir);

/** Configura las resistencias del pin
 *
 * @precondition 	El pin está configurado como GPIO.
 *
 * @param portNum   Número de puerto al que pertenece el pin (0-4).
 * @param bitPos    Posición del pin (bit) dentro del puerto.
 * @param pull   	0 o 'PULLUP': pin con resistencia pull-up,  1 o 'REPEATER': Pull-up si el pin = 1,  Pull-Down si el pin = 0,
 *                  2 o 'NOPULL': pin sin resistencias pull-up ni pull-down, 3 o 'PULLDOWN': Pin con resistencia pull-down
 *
 * @return No
 */
void GPIO_setPull(uint32_t portNum, uint32_t bitPos, uint32_t pull);

/** Pone el valor del pin
 *
 * @precondition 	El pin está configurado como GPIO y como output.
 *
 * @param portNum   Número de puerto al que pertenece el pin (0-4).
 * @param bitPos    Posición del pin (bit) dentro del puerto.
 * @param bitVal   	0 o 'FALSE': pone el pin a 0 lógico,  1 o 'TRUE': pone el pin a 1 lógico.
 *
 * @return No
 */
void GPIO_setValue(uint32_t portNum, uint32_t bitPos, bool_t bitVal);

/** Obtiene el valor lógico del pin
 *
 * @precondition 	El pin está configurado como GPIO input u output o como función digital alternativa (UARTx receive ...).
 *
 * @param portNum   Número de puerto al que pertenece el pin (0-4).
 * @param bitPos    Posición del pin (bit) dentro del puerto.
 *
 * @return 0 o 'FALSE' si el pin está a 0,  1 o 'TRUE' si el pin está a 1.
 */
bool_t GPIO_getValue (uint32_t portNum, uint32_t bitPos);

#endif /* GPIO_H_ */
