/*
 * printf.h
 *
 * This module allows send and receive data through an UART port
 *
 * author: Miguel A. Fdez.
 *
 * revision 1.0 date 5/4/2015
 */

#ifndef PRINTF_H_
#define PRINTF_H_

#include "type.h"

// available UART ports
#define UART_PORT_0	 0
#define UART_PORT_1	 1
#define UART_PORT_2	 2
#define UART_PORT_3	 3

#define MAX_UART_PORTS  4		// the number of available UART ports
#define UART_TX_BUFSIZE	512		// Tx buffer length

#define uartsSEMAPHORES_TAKE_WAIT_MS	60000 // milliseconds to wait for take the uarts semaphores

/** Initializes an UART port
 *
 * @precondition The UART port is available in hardware
 *
 * @param portNum   the UART port to initialize
 * @param baudrate  the baud rate for the port
 *
 * @return TRUE or FALSE. Returns FALSE if the interrupt handler
 *         can't be installed to the VIC table or if the hardware
 *         port don't is present, otherwise returns TRUE
 */
bool_t PrintF_start(uint32_t portNum, uint32_t baudrate);

/** Prints a formated string through an UART port in a similar way
 * that 'C' standard input-output library function printf().
 * Every port uses an independent Tx semaphore.
 *
 * @precondition 	the UART port has been initialized
 *
 * @param portNum    the UART port to send the formated string
 * @param format     pointer to the formated string
 * @param ...	     additional parameters to configure the final string
 *
 * @return TRUE or FALSE. Returns FALSE if the UART has not been initialized or if the
 * 		   hardware port is not present, otherwise returns TRUE
 */
bool_t PrintF_print(uint32_t portNum, char* format, ...);

/** Scans an input string through an UART port.
 * Every port uses an independent Rx semaphore.
 *
 * @precondition the UART port has been initialized
 *
 * @param portNum    		the UART port to scan the string
 * @param scannedString 	a pointer to the input scanned string
 * @param timeout	    	the time in milliseconds to wait for an input string
 *
 * @return 	the length of the scanned string
 */
int32_t PrintF_scan(uint32_t portNum, char* scannedString, uint32_t timeout);

#endif /* PRINTF_H_ */
