/*
 * rtc.h
 *
 * Copyright(C) NXP Semiconductors, 2014
 * All rights reserved.
 *
 * Modificado: Miguel A. Fdez.
 *
 */

#ifndef RTC_H_
#define RTC_H_

/*
 * Entre una ejecución del programa y la siguiente el uP pasa al estado deep power-down.
 * En este estado solo se conserva la memoria del RTC, y se pierden los datos de la memoria RAM del uP.
 * Para conservar datos entre distintas ejecuciones, utilizamos la memoria de usuario del RTC.
 *
 * Localización de las variables en la memoria de usuario del RTC (registros 0 a 4):
 *
 * Registro 0: Temperatura
 * bits
 * 15:0  Temperatura en el último ciclo medido. Para detectar la alarma por temperatura.
 * 	     Coma fija: -50.0 a 150.0 ºC.
 * 16    Alarma por temperatura activa ? 0:No, 1:Si.
 *
 * Registro 1: Verticalidad
 * bits
 * 15:0  Número de alarmas consecutivas de verticalidad sin consolidar como activas.
 * 		 Para evitar alarma en el momento de la descarga.
 * 16    Alarma de verticalidad activa ? 0:No, 1:Si.
 *
 * Registro 2: Nivel
 * bits
 * 15:0   Nivel de residuos en el último ciclo medido, para detectar la descarga del contenedor.
 * 		  Coma fija con signo: 15.0 a 300.0 cm.
 * 16  	  Error de lectura del sensor, 0: No error, 1:Error
 *
 * Registro 3: Periodos de medida (recibidos desde el host)
 * bits
 * 15:0   Periodo de actualización de datos (minutos).
 * 	      La unidad sensora queda en espera de recibir los datos de ajuste desde el host.
 * 31:16  Periodo de medida del nivel (minutos).
 *
 * Registro 4: Ciclos
 * bits
 * 15:0   Número de ciclos base acumulados. Para hacer la petición de datos al Host.
 * 31:16  Número de ciclos base acumulados. Para hacer la lectura del nivel.
 *
 *
 * En los registros se puede leer y escribir solo como palabra (16 bits) baja (bits 15-0),
 * alta (bits 31-16) o el registro completo (bits 31-0).
 *
 */

// Definiciones de los registros según la descripción anterior:
#define RTC_REG_TEMPER	0
#define RTC_REG_VERT	1
#define RTC_REG_NIVEL	2
#define RTC_REG_PER		3
#define RTC_REG_CICLO	4

#define RTC_REG_LOW_WORD	0	// leer/escribir en la palabra baja
#define RTC_REG_HIGHT_WORD	1	// leer/escribir en la palabra alta
#define RTC_REG_BOTH_WORDS	2	// leer/escribir en el registro completo

#define TEMP_ALARM_MASK		0x01	// palabra alta, bit 0
#define VERT_ALARM_MASK		0x01	// palabra alta, bit 0

/*
 *
 * Macros para lectura y escritura en la memoria no volatil del RTC.
 *
 */

// Valor de la temperatura
#define ROM_GET_TEMPERATURE_VALUE()		 \
	(uint16_t) Rtc_getUserRegisterValue((uint8_t)RTC_REG_TEMPER, (uint8_t)RTC_REG_LOW_WORD)

#define ROM_SET_TEMPERATURE_VALUE(value) \
	Rtc_setUserRegisterValue((uint8_t)RTC_REG_TEMPER, (uint32_t)value, (uint8_t)RTC_REG_LOW_WORD)

// Alarma de temperatura
#define ROM_GET_TEMPERATURE_ALARM()			  \
	(uint16_t) Rtc_getUserRegisterValue((uint8_t)RTC_REG_TEMPER, (uint8_t)RTC_REG_HIGHT_WORD)

#define ROM_SET_TEMPERATURE_ALARM(alarmState) \
	Rtc_setUserRegisterValue((uint8_t)RTC_REG_TEMPER, (uint32_t)alarmState, (uint8_t)RTC_REG_HIGHT_WORD)


// Ciclos consecutivos con alarma de verticalidad aùn no validada
#define ROM_GET_VERT_ALARM_ACCU_VALUE()		 \
	(uint16_t) Rtc_getUserRegisterValue((uint8_t)RTC_REG_VERT, (uint8_t)RTC_REG_LOW_WORD)

#define ROM_SET_VERT_ALARM_ACCU_VALUE(value) \
	Rtc_setUserRegisterValue((uint8_t)RTC_REG_VERT, (int32_t)value, (uint8_t)RTC_REG_LOW_WORD)

// Alarma de verticalidad (válida)
#define ROM_GET_VERT_ALARM()		\
	(uint16_t) Rtc_getUserRegisterValue((uint8_t)RTC_REG_VERT, (uint8_t)RTC_REG_HIGHT_WORD)

#define ROM_SET_VERT_ALARM(value)	\
	Rtc_setUserRegisterValue((uint8_t)RTC_REG_VERT, (int32_t)value, (uint8_t)RTC_REG_HIGHT_WORD)


// Valor del nivel
#define ROM_GET_NIVEL_VALUE()		\
	(uint16_t) Rtc_getUserRegisterValue((uint8_t)RTC_REG_NIVEL, (uint8_t)RTC_REG_LOW_WORD)

#define ROM_SET_NIVEL_VALUE(value)	\
	Rtc_setUserRegisterValue((uint8_t)RTC_REG_NIVEL, (int32_t)value, (uint8_t)RTC_REG_LOW_WORD)

// Error de lectura del sensor de nivel
#define ROM_GET_NIVEL_ALARM()		\
	(uint16_t) Rtc_getUserRegisterValue((uint8_t)RTC_REG_NIVEL, (uint8_t)RTC_REG_HIGHT_WORD)

#define ROM_SET_NIVEL_ALARM(value)	\
	Rtc_setUserRegisterValue((uint8_t)RTC_REG_NIVEL, (int32_t)value, (uint8_t)RTC_REG_HIGHT_WORD)


// Periodo de actualización de datos enviado desde el host
#define ROM_GET_PER_DATOS()			\
	(uint16_t) Rtc_getUserRegisterValue((uint8_t)RTC_REG_PER, (uint8_t)RTC_REG_LOW_WORD)

#define ROM_SET_PER_DATOS(value)	\
	Rtc_setUserRegisterValue((uint8_t)RTC_REG_PER, (int32_t)value, (uint8_t)RTC_REG_LOW_WORD)

// Periodo de medida del nivel enviado desde el host
#define ROM_GET_PER_NIVEL()			\
	(uint16_t) Rtc_getUserRegisterValue((uint8_t)RTC_REG_PER, (uint8_t)RTC_REG_HIGHT_WORD)

#define ROM_SET_PER_NIVEL(value)	\
	Rtc_setUserRegisterValue((uint8_t)RTC_REG_PER, (int32_t)value, (uint8_t)RTC_REG_HIGHT_WORD)


// Número de ciclos base acumulados para hacer la petición de datos al host
#define ROM_GET_CICLOS_DATOS()		\
	(uint16_t) Rtc_getUserRegisterValue((uint8_t)RTC_REG_CICLO, (uint8_t)RTC_REG_LOW_WORD)

#define ROM_SET_CICLOS_DATOS(value)	\
	Rtc_setUserRegisterValue((uint8_t)RTC_REG_CICLO, (int32_t)value, (uint8_t)RTC_REG_LOW_WORD)

// Número de ciclos base acumulados para hacer la lectura de nivel
#define ROM_GET_CICLOS_NIVEL()		\
	(uint16_t) Rtc_getUserRegisterValue((uint8_t)RTC_REG_CICLO, (uint8_t)RTC_REG_HIGHT_WORD)

#define ROM_SET_CICLOS_NIVEL(value)	\
	Rtc_setUserRegisterValue((uint8_t)RTC_REG_CICLO, (int32_t)value, (uint8_t)RTC_REG_HIGHT_WORD)

/*-----------------------------------------------------------*/

/**
 * @brief	Gets the RTC user register value
 * @param	regNbr	: El número de registro a leer (0-4)
 * @param	word	: LOW_WORD, HIGHT_WORD o BOTH_WORDS.
 * 					  Indica si se retorna solo la palabra (16 bits) baja, la alta o ambas.
 *
 * @return	El valor leido (0 si no se ha realizado la lectura).
 */
uint32_t Rtc_getUserRegisterValue(uint8_t regNbr, uint8_t word);

/**
 * @brief	Sets the RTC user register value
 * @param	regNbr	: El número de registro a escribir (0-4)
 * @param	regValue: El valor para el número de registro indicado en regNbr
 * @param   word	: LOW_WORD, HIGHT_WORD o BOTH_WORDS.
 * 					  Indica si se escribe solo la palabra (16 bits) baja, la alta o ambas.
 *
 *  @return	TRUE si se ha realizado la escritura o FALSE en caso contrario
 */
bool_t Rtc_setUserRegisterValue(uint8_t regNbr, uint32_t regValue, uint8_t word);

/**
 * @brief Register File register block structure
 */
typedef struct {
	__IO uint32_t REGFILE[5];	/*!< General purpose storage register */
} LPC_REGFILE_T;

/*
 * @brief ILR register definitions
 */
/** ILR register mask */
#define RTC_ILR_BITMASK         ((0x00000003))
/** Bit inform the source interrupt is counter increment*/
#define RTC_IRL_RTCCIF          ((1 << 0))
/** Bit inform the source interrupt is alarm match*/
#define RTC_IRL_RTCALF          ((1 << 1))

/*
 * @brief CCR register definitions
 */
/** CCR register mask */
#define RTC_CCR_BITMASK         ((0x00000013))
/** Clock enable */
#define RTC_CCR_CLKEN           ((1 << 0))
/** Clock reset */
#define RTC_CCR_CTCRST          ((1 << 1))
/** Calibration counter enable */
#define RTC_CCR_CCALEN          ((1 << 4))

/*
 * @brief CIIR and AMR register definitions
 */
/** Counter Increment Interrupt bit for second */
#define RTC_AMR_CIIR_IMSEC          ((1 << 0))
/** Counter Increment Interrupt bit for minute */
#define RTC_AMR_CIIR_IMMIN          ((1 << 1))
/** Counter Increment Interrupt bit for hour */
#define RTC_AMR_CIIR_IMHOUR         ((1 << 2))
/** Counter Increment Interrupt bit for day of month */
#define RTC_AMR_CIIR_IMDOM          ((1 << 3))
/** Counter Increment Interrupt bit for day of week */
#define RTC_AMR_CIIR_IMDOW          ((1 << 4))
/** Counter Increment Interrupt bit for day of year */
#define RTC_AMR_CIIR_IMDOY          ((1 << 5))
/** Counter Increment Interrupt bit for month */
#define RTC_AMR_CIIR_IMMON          ((1 << 6))
/** Counter Increment Interrupt bit for year */
#define RTC_AMR_CIIR_IMYEAR         ((1 << 7))
/** CIIR bit mask */
#define RTC_AMR_CIIR_BITMASK        ((0xFF))

/*
 * @brief RTC_AUX register definitions
 */
/** RTC Oscillator Fail detect flag */
#define RTC_AUX_RTC_OSCF        ((1 << 4))

/*
 * @brief RTC_AUXEN register definitions
 */
/** Oscillator Fail Detect interrupt enable*/
#define RTC_AUXEN_RTC_OSCFEN    ((1 << 4))

/*
 * @brief Consolidated Time Register 0 definitions
 */
#define RTC_CTIME0_SECONDS_MASK     ((0x3F))
#define RTC_CTIME0_MINUTES_MASK     ((0x3F00))
#define RTC_CTIME0_HOURS_MASK       ((0x1F0000))
#define RTC_CTIME0_DOW_MASK         ((0x7000000))

/*
 * @brief Consolidated Time Register 1 definitions
 */
#define RTC_CTIME1_DOM_MASK         ((0x1F))
#define RTC_CTIME1_MONTH_MASK       ((0xF00))
#define RTC_CTIME1_YEAR_MASK        ((0xFFF0000))

/*
 * @brief Consolidated Time Register 2 definitions
 */
#define RTC_CTIME2_DOY_MASK         ((0xFFF))

/*
 * @brief Time Counter Group and Alarm register group
 */
/** SEC register mask */
#define RTC_SEC_MASK            (0x0000003F)
/** MIN register mask */
#define RTC_MIN_MASK            (0x0000003F)
/** HOUR register mask */
#define RTC_HOUR_MASK           (0x0000001F)
/** DOM register mask */
#define RTC_DOM_MASK            (0x0000001F)
/** DOW register mask */
#define RTC_DOW_MASK            (0x00000007)
/** DOY register mask */
#define RTC_DOY_MASK            (0x000001FF)
/** MONTH register mask */
#define RTC_MONTH_MASK          (0x0000000F)
/** YEAR register mask */
#define RTC_YEAR_MASK           (0x00000FFF)

#define RTC_SECOND_MAX      59	/*!< Maximum value of second */
#define RTC_MINUTE_MAX      59	/*!< Maximum value of minute*/
#define RTC_HOUR_MAX        23	/*!< Maximum value of hour*/
#define RTC_MONTH_MIN       1	/*!< Minimum value of month*/
#define RTC_MONTH_MAX       12	/*!< Maximum value of month*/
#define RTC_DAYOFMONTH_MIN  1	/*!< Minimum value of day of month*/
#define RTC_DAYOFMONTH_MAX  31	/*!< Maximum value of day of month*/
#define RTC_DAYOFWEEK_MAX   6	/*!< Maximum value of day of week*/
#define RTC_DAYOFYEAR_MIN   1	/*!< Minimum value of day of year*/
#define RTC_DAYOFYEAR_MAX   366	/*!< Maximum value of day of year*/
#define RTC_YEAR_MAX        4095/*!< Maximum value of year*/

/*
 * @brief Calibration register
 */
/** Calibration value */
#define RTC_CALIBRATION_CALVAL_MASK     ((0x1FFFF))
/** Calibration direction */
#define RTC_CALIBRATION_LIBDIR          ((1 << 17))
/** Calibration max value */
#define RTC_CALIBRATION_MAX             ((0x20000))
/** Calibration definitions */
#define RTC_CALIB_DIR_FORWARD           ((uint8_t) (0))
#define RTC_CALIB_DIR_BACKWARD          ((uint8_t) (1))

// milliseconds to wait for take the semaphore
#define rtcSEMAPHORE_TAKE_WAIT_MS	60000

/**
 * @brief RTC enumeration
 */

/** @brief RTC interrupt source */
typedef enum {
	RTC_INT_COUNTER_INCREASE = RTC_IRL_RTCCIF,	/*!<  Counter Increment Interrupt */
	RTC_INT_ALARM = RTC_IRL_RTCALF				/*!< The alarm interrupt */
} RTC_INT_OPT_T;

typedef struct {
	uint32_t time[RTC_TIMETYPE_LAST];
} RTC_TIME_T;


/**
 * @brief	Initialize the RTC peripheral and implicitly the semaphore
 * @param	pRTC	: RTC peripheral selected
 * @return	None
 */
void Rtc_init(LPC_RTC_TypeDef *pRTC);

/**
 * @brief	Explicit initialize the module's semaphore
 * @param	None
 * @return	None
 */

void Rtc_semaphoreInit(void);

/**
 * @brief	De-initialize the RTC peripheral
 * @param	pRTC	: RTC peripheral selected
 * @return	None
 */
void Rtc_deInit(LPC_RTC_TypeDef *pRTC);

/**
 * @brief	Reset clock tick counter in the RTC peripheral
 * @param	pRTC	: RTC peripheral selected
 * @return	None
 */
void Rtc_resetClockTickCounter(LPC_RTC_TypeDef *pRTC);

/**
 * @brief	Start/Stop RTC peripheral
 * @param	pRTC		: RTC peripheral selected
 * @param	NewState	: New State of this function, should be:
 *							- ENABLE	:The time counters are enabled
 *							- DISABLE	:The time counters are disabled
 * @return	None
 */
void Rtc_enable(LPC_RTC_TypeDef *pRTC, FunctionalState NewState);

/**
 * @brief	Enable/Disable Counter increment interrupt for a time type in the RTC peripheral
 * @param	pRTC		: RTC peripheral selected
 * @param	cntrMask	: Or'ed bit values for time types (RTC_AMR_CIIR_IM*)
 * @param	NewState	: ENABLE or DISABLE
 * @return	None
 */
void Rtc_cntIncrIntConfig(LPC_RTC_TypeDef *pRTC, uint32_t cntrMask, FunctionalState NewState);

/**
 * @brief	Enable/Disable Alarm interrupt for a time type in the RTC peripheral
 * @param	pRTC		: RTC peripheral selected
 * @param	alarmMask	: Or'ed bit values for ALARM types (RTC_AMR_CIIR_IM*)
 * @param	NewState	: ENABLE or DISABLE
 * @return	None
 */
void Rtc_alarmIntConfig(LPC_RTC_TypeDef *pRTC, uint32_t alarmMask, FunctionalState NewState);

/**
 * @brief	Set full time in the RTC peripheral
 * @param	pRTC		: RTC peripheral selected
 * @param	pFullTime	: Pointer to full time data
 * @return	None
 */
void Rtc_setFullTime(LPC_RTC_TypeDef *pRTC, RTC_TIME_T *pFullTime);

/**
 * @brief	Get full time from the RTC peripheral
 * @param	pRTC		: RTC peripheral selected
 * @param	pFullTime	: Pointer to full time record to fill
 * @return	None
 */
void Rtc_getFullTime(LPC_RTC_TypeDef *pRTC, RTC_TIME_T *pFullTime);

/**
 * @brief	Set full alarm time in the RTC peripheral
 * @param	pRTC		: RTC peripheral selected
 * @param	pFullTime	: Pointer to full time record to set alarm
 * @return	None
 */
void Rtc_setFullAlarmTime(LPC_RTC_TypeDef *pRTC, RTC_TIME_T *pFullTime);

/**
 * @brief	Get full alarm time in the RTC peripheral
 * @param	pRTC		: RTC peripheral selected
 * @param	pFullTime	: Pointer to full time record to fill
 * @return	None
 */
void Rtc_getFullAlarmTime(LPC_RTC_TypeDef *pRTC, RTC_TIME_T *pFullTime);

/**
 * @brief	Enable/Disable calibration counter in the RTC peripheral
 * @param	pRTC		: RTC peripheral selected
 * @param	NewState	: New State of this function, should be:
 *							- ENABLE	:The calibration counter is enabled and counting
 *							- DISABLE	:The calibration counter is disabled and reset to zero
 * @return	None
 */
void Rtc_calibCounterCmd(LPC_RTC_TypeDef *pRTC, FunctionalState NewState);

/**
 * @brief	Clear specified Location interrupt pending in the RTC peripheral
 * @param	pRTC	: RTC peripheral selected
 * @param	IntType	: Interrupt location type, should be:
 *						- RTC_INT_COUNTER_INCREASE	:Clear Counter Increment Interrupt pending.
 *						- RTC_INT_ALARM				:Clear alarm interrupt pending
 * @return	None
 */
static inline void Rtc_clearIntPending(LPC_RTC_TypeDef *pRTC, uint32_t IntType)
{
	pRTC->ILR = IntType;
}

/**
 * @brief	Check whether if specified location interrupt in the RTC peripheral is set or not
 * @param	pRTC	: RTC peripheral selected
 * @param	IntType	: Interrupt location type, should be:
 *						- RTC_INT_COUNTER_INCREASE: Counter Increment Interrupt block generated an interrupt.
 *						- RTC_INT_ALARM: Alarm generated an interrupt.
 * @return	New state of specified Location interrupt in RTC peripheral, SET OR RESET
 */
static inline ITStatus Rtc_getIntPending(LPC_RTC_TypeDef *pRTC, uint32_t IntType)
{
	return (pRTC->ILR & IntType) ? SET : RESET;
}

/**
 * @brief	Programs the RTC for the next wake-up
 * @param	powerDownInterval : El tiempo en segundos hasta el próximo wake-up
 *
 * @return	None
 */
void Rtc_programNextWakeUp(uint32_t secondsPowerDownInterval);


#endif /* RTC_H_ */
