/*
 * utils.h
 *
 * This module incorporates utilities for common use
 *
 * author: Miguel A. Fdez.
 *
 * revision 1.0 date 8/4/2015
 */

#ifndef UTILS_H_
#define UTILS_H_

#include "type.h"

#define UTILS_SEMAPH_TAKE_WAIT_MS	60000 	// milliseconds to wait for take the semaphores


/** Causes an active or passive delay
 *
 * @precondition None
 *
 * @param isPassive   Selects between active or passive delay. FALSE: active, TRUE: passive
 * @param msToDelay   Milliseconds to delay
 *
 * @return None
 */
void Utils_delayMs(bool_t isPassive, uint32_t msToDelay);


/** Filtro simple de un conjunto de valores:
 *  Del conjunto de valores pasados en 'valuesToFilter[]', elimina
 *  el valor máximo, elimina el valor mínimo y promedia el resto de valores.
 *
 * @precondition 	numberOfValuesToFilter > 2.
 *
 * @param valuesToFilter[]   El conjunto de valores a filtrar.
 * @param numberOfValuesToFilter  La cantidad de valores pasados en el conjunto para filtrar.
 *
 * @return el valor obtenido del filtraje.
 */
double Utils_simpleFilter(double *valuesToFilter, uint32_t numberOfValuesToFilter);


/**
 * Calcul of CRC 16 CCITT polynomial. Equation: x^16 + x^12 + x^5 + 1.
 *
 * @precondition None
 *
 * @param data 			Pointer to block of data.
 * @param dataLength 	Length of data in bytes;
 * @param initialCrc 	Initial value. If data is split into multiple segments, previous CRC is used as initial.
 *
 * @return 				The calculated CRC.
 */
uint16_t Utils_crc16_ccitt(const char data[], uint16_t dataLen, uint16_t initialCrc);


/** Busca la subcadena 'compareStr' dentro de la cadena 'inputStr' y si la encuentra,
 *  la elimina y retorna la cadena de entrada 'inputStr' con la cadena de comparación eliminada
 *
 * @precondition La cadena de comparación 'compareStr' está al final
 *               de la cadena de entrada 'inputStr'
 *
 * @param inputStr   		la cadena de entrada
 * @param compareStr  		la subcadena a eliminar en la cadena de entrada
 * @param returnedSubStr  	la cadena de entrada con la subcadena 'compareStr' eliminada
 *
 * @return La longitud de la cadena retornada ('returnedSubStr').
 */
uint16_t Utils_clearString(char* inputStr, char* compareStr, char* returnedSubStr);


/** Convierte una dirección IPV4 pasada como argumento (xxx.xxx.xxx.xxx)
 *  en un entero de 4 bytes.
 *
 * @precondition 	None
 *
 * @param ipStr   	La cadena con la dirección IP a convertir.
 *
 * @return el entero de 32 biys con la dirección codificada.
 */
uint32_t Utils_ipStrToUint32(char* ipStr);


/** Busca el valor correspondiente a la clave pasada como argumento y si existe
 *  lo entrega en el parámetro de salida.
 *
 * @precondition 	None
 *
 * @param inputStr   	La cadena en la que buscar la clave/valor.
 * @param key   	    La clave.
 * @param valueLen   	La longitud del valor.
 * @param valueOut   	La salida con el valor.
 *
 * @return 	TRUE si se ha encontrado el valor correspondiente a la clave,
 * 			FALSE en caso contrario.
 */
bool_t Utils_getValueByKeyByLen(char* inputStr, char* key, uint32_t valueLen, char* valueOut);


/** Devuelve el valor identificado por una clave (clave/valor) contenido en la cadena de entrada
 *
 * @precondition None
 *
 * @param inputStr  		 La cadena de la que se quiere extraer el valor.
 * @param tokenDelimiters[]  El/los delimitadores de cada conjunto clave/valor.
 * @param key  			 	 La clave.
 * @param initialPosition  	 La posición inicial del valor (0 based).
 * @param returnedValue  	 El valor buscado.
 *
 * @return 	TRUE si se ha encontrado el par clave/valor en la cadena pasada 'inputStr'.
 * 			FALSE en caso contrario.
 */
bool_t Utils_getValueByKey(char* inputStr, const char tokenDelimiters[], char* key, char* returnedValue);


#endif /* UTILS_H_ */
