/*
 *wifly.h
 *
 *  Created on: 27/10/2015
 *      Author: Miguel A. Fdez.
 */

#ifndef WIFLY_H_
#define WIFLY_H_

#include "type.h"

typedef enum { OPEN = 0, WEP, WPA } securityType;	// Network security types

#define WIFLY_PORT				UART_PORT_0	 // The microcontroller UART port to communicate with the Wifly
#define WIFLY_BAUD_RATE			9600 // The baud rate for the micro UART port (= Wifli setting baud rate)

#define WIFLY_IP_ADDR	  		"192.168.1.36"	  		// This sensor (wifly) IP address
#define ACCESS_POINT_NAME 		"XXXX"					// The wifi access point name
#define ACCESS_POINT_SEC_TYPE	(securityType) WPA		// The wifi access point security type (see typedef securityType)
#define ACCESS_POINT_PASS		"XXXX"					// The wifi access point password
#define REMOTE_HOST_NAME  		"www.irisgest.com"  	// The remote host name

#define WIFLY_RESET_PIN		1, 30	// Wifly reset, pin (P1.30): pin port (1), pin bit (30)
#define WIFLY_WAKEUP_PIN	2, 4	// Wifly wakeup, pin (P2.4): pin port (2), pin bit (4)
#define WIFLY_READY_PIN		2, 0	// Wifly ready, pin (P2.0): pin port (2), pin bit (0)

#define WIFLY_SEMAPHORE_TAKE_WAIT_MS	60000 	// milliseconds to wait for take the semaphore

/*-----------------------------------------------------------*/

/** Conexión inicial del Wifly a una red wifi.
 *  Incluye reset hardware del Wifly.
 *
 *    - Inicializa el puerto UART del LPC1769 utilizado para comunicarse con el Wifly
 * 	    con las ajustes programados en 'whifly.h'
 * 	  - Inicializa el GPIO del LPC1769 para trabajar con el Wifly
 * 	  - Hace un reset hardware del Wifly
 * 	  - Ajusta los parámetros para trabajar con el cliente HTML del Wifly
 * 	  - Conecta al Wifly a una red wifi
 *
 * @param None
 *
 * @return TRUE or FALSE. Retorna TRUE si se ha conseguido conectar a una red wifi,
 *                        FALSE en caso contrario.
 */
bool_t Wifly_init(void);


/** Activa el WiFly desde el modo 'sleep' y conecta con una red wifi.
 *  NO incluye reset hardware del Wifly.
 *
 *    - Inicializa el puerto UART del LPC1769 utilizado para comunicarse con el Wifly
 * 	    con las ajustes programados en 'whifly.h'
 * 	  - Inicializa el GPIO del LPC1769 para trabajar con el Wifly
 * 	  - Ajusta los parámetros para trabajar con el cliente HTML del Wifly
 * 	  - Conecta al Wifly a una red wifi
 *
 * @param None.
 *
 * @return TRUE or FALSE. Retorna TRUE si se ha conseguido conectar a una red wifi,
 *                        FALSE en caso contrario.
 */
bool_t Wifly_wakeUp(void);


/** Pone al WiFly en modo sleep.
 *
 * @preconditions El Wifly ha sido inicializado
 *
 * @param None.
 *
 * @return TRUE or FALSE. Retorna TRUE si el Wifly ha pasado a modo sleep,
 *						  FALSE en caso contrario.
 */
bool_t Wifly_enterSleep(void);


/** Envia peticiones via HTTP:
 * 	   - Pone al Wifly en modo datos
 * 	   - Abre la conexión cliente HTTP.
 * 	   - Envía la petición.
 * 	   - Recibe la respuesta.
 *
 * @preconditions El Wifly ha sido inicializado y está conectado a una red wifi
 *
 * @param request  La cadena de caracteres con la petición
 * @param remoteAnswer La cadena de salida con la respuesta del host
 *
 * @return TRUE or FALSE. Retorna TRUE si el proceso descrito anteriormente ha ido bien,
 *                        FALSE en caso contrario.
 */
bool_t Wifly_sendHttp(char* request, char* remoteAnswer);


/*************************************************************************************
 * Las funciones siguientes están implementadas aunque no se utilizan
 * en esta aplicación.
 *************************************************************************************/

/** Escanea las redes wifi al alcance.
 *
 * @preconditions El Wifly ha sido inicializado.
 *
 * @param outWifly_Scan   Cadena de salida con las redes al alcance.
 *
 * @return TRUE or FALSE. Retorna TRUE si consigue hacer el scan,
 *                        FALSE en caso contrario.
 */
bool_t Wifly_scan(char* outWifly_Scan);


/** Hace ping a un servidor de internet.
 *
 * @preconditions El Wifly ha sido inicializado y está conectado a una red wifi
 *
 * @param pingDestination_Domain  La dirección URL del servidor al que se quiere enviar el ping
 * @param pingRetry_Times  El número de pings a enviar
 *
 * @return TRUE or FALSE. Retorna TRUE si recibe respuesta a los pings,
 *                        FALSE en caso contrario.
 */
bool_t Wifly_ping(char* pingDestination_Domain, uint32_t pingRetry_Times);


/** Obtiene la dirección IP actual del Wifly.
 *
 * @preconditions El Wifly ha sido inicializado.
 *
 * @param outWifly_Ip  Cadena de salida con la dirección IP del Wifly
 *
 * @return TRUE or FALSE. Retorna TRUE si se ha conseguido obtener la IP,
 *                        FALSE en caso contrario.
 */
bool_t Wifly_getIp(char* outWifly_Ip);


/** Obtiene el canal de comunicación wifi del Wifly.
 *
 * @preconditions El Wifly ha sido inicializado y está conectado a una red wifi.
 *
 * @param No
 *
 * @return El número de canal.
 */
uint32_t Wifly_getChannel(void);


/** Obtiene el RSSI del Wifly.
 *
 * @preconditions El Wifly ha sido inicializado y está conectado a una red wifi.
 *
 * @param No
 *
 * @return El RSSI del Wifly en dBm.
 */
uint32_t Wifly_getRssi(void);


/** Obtiene el tiempo transcurrido desde el boot del Wifly.
 *
 * @preconditions El Wifly ha sido inicializado.
 *
 * @param No
 *
 * @return El tiempo transcurrido desde el boot del Wifly en segundos.
 */
uint32_t Wifly_getSecondsSinceBoot(void);


/** Obtiene la potencia de transmisión con la que está trabajando el Wifly.
 *
 * @preconditions El Wifly ha sido inicializado.
 *
 * @param No
 *
 * @return La potencia en dBm. (1..12 dBm).
 */
uint32_t Wifly_getTxPower(void);


/** Ajusta la potencia de transmisión con la que trabaja el Wifly.
 *
 * @preconditions El Wifly ha sido inicializado.
 *
 * @param txPower La potencia deseada en dBm. (1..12 dBm  0 = max power = 12).
 *
 * @return TRUE or FALSE. Retorna TRUE si se ha conseguido ajustar la potencia en el Wifly,
 *                        FALSE en caso contrario.
 */
bool_t Wifly_setTxPower(uint32_t txPower);


#endif /* WIFLY_H_ */
