/*
 * adc.c
 *
 *  Created on: 11/11/2015
 *      Author: Miguel A. Fdez.
 */

#include "FreeRTOS.h"
#include "semphr.h"
#include "LPC17xx.h"
#include "type.h"
#include "adc.h"

#define adcDONE            	0x80000000 	// Mask to check if conversion is ready
#define adcOVERRUN         	0x40000000	// Mask to check the overrun
#define adcNUM_CHANNELS		4 			// Number of ADC used channels
#define adcCLK	1000000		// Clock del ADC, set to 1Mhz

/*-----------------------------------------------------------*/

// The time to wait for the semaphore in ticks.
// Defined in adc.h in milliseconds.
volatile portTickType l_adc_semaphoreTake_Ticks = 0;

// The ADC semaphore
volatile xSemaphoreHandle l_adc_mutex = NULL;

/*-----------------------------------------------------------*/

void Adc_init(void)
{
	uint32_t pclkdiv, pclk;

	// Se crea el semáforo
	l_adc_mutex = xSemaphoreCreateMutex();

	// Conversión a ticks del tiempo de espera para obtener el semáforo
	l_adc_semaphoreTake_Ticks = (portTickType) adcSEMAPHORE_TAKE_WAIT_MS / portTICK_RATE_MS;

	if(xSemaphoreTake(l_adc_mutex, l_adc_semaphoreTake_Ticks) == pdTRUE)
	{
		/* Enable CLOCK into ADC controller */
		LPC_SC->PCONP |= (1 << 12);

		/*
		 * Este driver está preparado para trabajar con los canales
		 * de entradas analógicas 0 a 7.
		 * En este caso trabajamos solo con los canales 0 a 3.
		 * Los pins correspondientes al resto de canales (4-7) en
		 * general les utilizamos para otras funciones.
		 * Debido a esto, comentamos el código correspondiente
		 * para trabajar solamente con los canales 0 a 3.
		 */

		/* all the related pins are set to ADC inputs, AD0.0~7 */
	//	LPC_PINCON->PINSEL0 &= ~0x000000F0;	/* P0.2~3, A0.6~7, function 10 */
	//	LPC_PINCON->PINSEL0 |= 0x000000A0;
		LPC_PINCON->PINSEL1 &= ~0x003FC000;	/* P0.23~26, A0.0~3, function 01 */
		LPC_PINCON->PINSEL1 |= 0x00154000;
	//	LPC_PINCON->PINSEL3 |= 0xF0000000;	/* P1.30~31, A0.4~5, function 11 */

		/* No pull-up no pull-down (function 10) on these ADC pins. */
	//	LPC_PINCON->PINMODE0 &= ~0x000000F0;
	//	LPC_PINCON->PINMODE0 |= 0x000000A0;
		LPC_PINCON->PINMODE1 &= ~0x003FC000;
		LPC_PINCON->PINMODE1 |= 0x002A8000;
	//	LPC_PINCON->PINMODE3 &= ~0xF0000000;
	//	LPC_PINCON->PINMODE3 |= 0xA0000000;

		/* By default, the PCLKSELx value is zero, thus, the PCLK for
		all the peripherals is 1/4 of the SystemFrequency. */
		/* Bit 24~25 is for ADC */
		pclkdiv = (LPC_SC->PCLKSEL0 >> 24) & 0x03;
		switch ( pclkdiv )
		{
			case 0x00:
			default:
				pclk = SystemCoreClock/4;
				break;
			case 0x01:
				pclk = SystemCoreClock;
				break;
			case 0x02:
				pclk = SystemCoreClock/2;
				break;
			case 0x03:
				pclk = SystemCoreClock/8;
				break;
		}

		LPC_ADC->ADCR = ( 0x00 << 1 ) |  				/* SEL=1, select channel 0~7 on ADC0 */
			(( pclk  / (uint32_t) adcCLK - 1 ) << 8) | 	/* CLKDIV = Fpclk / ADC_Clk - 1 */
			( 0 << 16 ) | 								/* BURST = 0, no BURST, software controlled */
			( 0 << 17 ) |  								/* CLKS = 0, 11 clocks/10 bits */
			( 1 << 21 ) |  								/* PDN = 1, normal operation */
			( 0 << 24 ) |  								/* START = 0 A/D conversion stops */
			( 0 << 27 );								/* EDGE = 0 (CAP/MAT signal falling, trigger A/D conversion) */

		xSemaphoreGive(l_adc_mutex);
	}

	return;
}

/*-----------------------------------------------------------*/

uint32_t Adc_read(uint8_t channelNum)
{
	uint32_t regVal, ADC_Data = 0;

	if(xSemaphoreTake(l_adc_mutex, l_adc_semaphoreTake_Ticks) == pdTRUE)
	{
		if (channelNum >= adcNUM_CHANNELS) channelNum = 0; // Default to channel 0

		/* switch channel, start A/D convert */
		LPC_ADC->ADCR &= 0xFFFFFF00;
		LPC_ADC->ADCR |= (1 << 24) | (1 << channelNum);

		while ( 1 )			/* wait until end of A/D convert */
		{
			switch (channelNum) {
				case 0:
					regVal = LPC_ADC->ADDR0;
					break;
				case 1:
					regVal = LPC_ADC->ADDR1;
					break;
				case 2:
					regVal = LPC_ADC->ADDR2;
					break;
				case 3:
					regVal = LPC_ADC->ADDR3;
					break;
				case 4:
					regVal = LPC_ADC->ADDR4;
					break;
				case 5:
					regVal = LPC_ADC->ADDR5;
					break;
				case 6:
					regVal = LPC_ADC->ADDR6;
					break;
				case 7:
					regVal = LPC_ADC->ADDR7;
					break;
			}

			/* read result of A/D conversion */
			if (regVal & adcDONE) break;
		}

		/* stop ADC now */
		LPC_ADC->ADCR &= 0xF8FFFFFF;

		/* save data when it's not overrun, otherwise, return zero */
		if (regVal & adcOVERRUN) return ( 0 );

		ADC_Data = (regVal >> 4) & 0xFFF;


		xSemaphoreGive(l_adc_mutex);
	}

	return (ADC_Data);	/* return A/D conversion value */
}

/*-----------------------------------------------------------*/
