/*
 * power.c
 *
 *  Created on: 13/11/2015
 *      Author: NXP Semiconductors, 2014.
 *      Modified: Miguel A. Fdez.
 */

#include "FreeRTOS.h"
#include "semphr.h"
#include "LPC17xx.h"
#include "power.h"

/*-----------------------------------------------------------*/

// Funciones locales

/*-----------------------------------------------------------*/

/**
 * @brief	Enter MCU Sleep mode
 * @param	pPMU	: Pointer to PMU register block
 * @return	None
 * @note	The sleep mode affects the ARM Cortex-M0+ core only. Peripherals
 * and memories are active.
 */
static void _sleepState(LPC_PMU_T *pPMU)
{
	/* Select Sleep/Deep Sleep mode */
	pPMU->PCON &= ~(PMU_PCON_PM1_FLAG | PMU_PCON_PM0_FLAG);

	/* Clearing SLEEPDEEP bit in SCR makes it Sleep mode */
	SCB->SCR &= ~(1UL << SCB_SCR_SLEEPDEEP_Pos);

	/* Enter sleep mode */
	__WFI();
}
/*-----------------------------------------------------------*/

/**
 * @brief	Enter MCU Deep Sleep mode
 * @param	pPMU	: Pointer to PMU register block
 * @return	None
 * @note	In Deep-sleep mode, the peripherals receive no internal clocks.
 * The flash is in stand-by mode. The SRAM memory and all peripheral registers
 * as well as the processor maintain their internal states. The WWDT, WKT,
 * and BOD can remain active to wake up the system on an interrupt.
 */
static void _deepSleepState(LPC_PMU_T *pPMU)
{
	/* Select Sleep/Deep Sleep mode */
	pPMU->PCON &= ~(PMU_PCON_PM1_FLAG | PMU_PCON_PM0_FLAG);

	/* Setting SLEEPDEEP bit in SCR makes it Deep Sleep mode */
	SCB->SCR |= (1UL << SCB_SCR_SLEEPDEEP_Pos);

	/* Enter sleep mode */
	__WFI();
}
/*-----------------------------------------------------------*/

/**
 * @brief	Enter MCU Power down mode
 * @param	pPMU	: Pointer to PMU register block
 * @return	None
 * @note	In Power-down mode, the peripherals receive no internal clocks.
 * The internal SRAM memory and all peripheral registers as well as the
 * processor maintain their internal states. The flash memory is powered
 * down. The WWDT, WKT, and BOD can remain active to wake up the system
 * on an interrupt.
 */
static void _powerDownState(LPC_PMU_T *pPMU)
{
	/* Select power down mode */
	pPMU->PCON = (pPMU->PCON & ~PMU_PCON_PM1_FLAG) | PMU_PCON_PM0_FLAG;

	/* Setting SLEEPDEEP bit in SCR makes it power down mode */
	SCB->SCR |= (1UL << SCB_SCR_SLEEPDEEP_Pos);

	/* Enter sleep mode */
	__WFI();
}
/*-----------------------------------------------------------*/

/**
 * @brief	Enter MCU Deep Power down mode
 * @param	pPMU	: Pointer to PMU register block
 * @return	None
 * @note	For maximal power savings, the entire system is shut down
 * except for the general purpose registers in the PMU and the self
 * wake-up timer. Only the general purpose registers in the PMU maintain
 * their internal states. The part can wake up on a pulse on the WAKEUP
 * pin or when the self wake-up timer times out. On wake-up, the part
 * reboots.
 */
static void _deepPowerDownState(LPC_PMU_T *pPMU)
{
	/* Select deep power down mode */
	pPMU->PCON |= PMU_PCON_PM1_FLAG | PMU_PCON_PM0_FLAG;

	/* Setting SLEEPDEEP bit in SCR makes it deep power down mode */
	SCB->SCR |= (1UL << SCB_SCR_SLEEPDEEP_Pos);

	/* Enter sleep mode */
	__WFI();
}

/*-----------------------------------------------------------*/

// Funciones globales

/*-----------------------------------------------------------*/

/* Put some of the peripheral in sleep mode */
void Power_sleep(LPC_PMU_T *pPMU, CHIP_PMU_MCUPOWER_T sleepMode)
{
	if (sleepMode == PMU_MCU_DEEP_SLEEP) {
		_deepSleepState(pPMU);
	}
	else if (sleepMode == PMU_MCU_POWER_DOWN) {
		_powerDownState(pPMU);
	}
	else if (sleepMode == PMU_MCU_DEEP_PWRDOWN) {
		_deepPowerDownState(pPMU);
	}
	else {
		/* PMU_MCU_SLEEP */
		_sleepState(pPMU);
	}
}

/*-----------------------------------------------------------*/
