/*
 * printf.c
 *
 *  Created on: 5/4/2015
 *      Author: Miguel A. Fdez.
 */

#include "FreeRTOS.h"
#include "semphr.h"
#include "LPC17xx.h"
#include "uart.h"
#include "utils.h"
#include "printF.h"
#include <string.h>
#include <stdio.h>
#include <stdarg.h>

// The time to wait for all semaphores in ticks.
// Defined in PrintF.h in milliseconds.
volatile portTickType l_PrintF_uartsSemaphores_Take_Ticks = 0;

// Semáforo para la inicialización
volatile xSemaphoreHandle l_PrintF_uartsMutex = NULL;

// Semáforos para la transmisión por las UARTS
volatile xSemaphoreHandle l_PrintF_uart0Mutex_Tx = NULL;
volatile xSemaphoreHandle l_PrintF_uart1Mutex_Tx = NULL;
volatile xSemaphoreHandle l_PrintF_uart2Mutex_Tx = NULL;
volatile xSemaphoreHandle l_PrintF_uart3Mutex_Tx = NULL;

// Semáforos para la recepción por las UARTS
volatile xSemaphoreHandle l_PrintF_uart0Mutex_Rx = NULL;
volatile xSemaphoreHandle l_PrintF_uart1Mutex_Rx = NULL;
volatile xSemaphoreHandle l_PrintF_uart2Mutex_Rx = NULL;
volatile xSemaphoreHandle l_PrintF_uart3Mutex_Rx = NULL;

// parameters declared in 'uart.c'
extern volatile uint32_t UART0Count, UART1Count, UART2Count, UART3Count;
extern volatile uint8_t UART0Buffer[UART_RX_BUFSIZE], UART1Buffer[UART_RX_BUFSIZE],
						UART2Buffer[UART_RX_BUFSIZE], UART3Buffer[UART_RX_BUFSIZE];

/*-----------------------------------------------------------*/

bool_t PrintF_start(uint32_t portNum, uint32_t baudrate)
{
	bool_t success = FALSE;

	if(portNum < MAX_UART_PORTS)  // No es necesario, se comprueba en el driver. Se repite aquí para evitar
	{							  // perder tiempo en la llamada al driver en caso de no cumplir la condición.

		if(l_PrintF_uartsMutex == NULL) // sirve como referencia para la inicialización
		{
			// Todos las variables se inicializarán con la primera llamada
			// a la función para iniciar la UART con la que se parametrize.

			// Creación de todos los semáforos
			l_PrintF_uartsMutex = xSemaphoreCreateMutex();

			l_PrintF_uart0Mutex_Tx = xSemaphoreCreateMutex();
			l_PrintF_uart1Mutex_Tx = xSemaphoreCreateMutex();
			l_PrintF_uart2Mutex_Tx = xSemaphoreCreateMutex();
			l_PrintF_uart3Mutex_Tx = xSemaphoreCreateMutex();

			l_PrintF_uart0Mutex_Rx = xSemaphoreCreateMutex();
			l_PrintF_uart1Mutex_Rx = xSemaphoreCreateMutex();
			l_PrintF_uart2Mutex_Rx = xSemaphoreCreateMutex();
			l_PrintF_uart3Mutex_Rx = xSemaphoreCreateMutex();

			// Inicialización del tiempo de espera para todos los semáforos en ticks
			l_PrintF_uartsSemaphores_Take_Ticks = (portTickType) uartsSEMAPHORES_TAKE_WAIT_MS / portTICK_RATE_MS;
		}

		// Si todos los parámetros están iniciados...
		if(l_PrintF_uartsMutex != NULL &&
		   l_PrintF_uart0Mutex_Tx != NULL && l_PrintF_uart1Mutex_Tx != NULL &&
		   l_PrintF_uart2Mutex_Tx != NULL && l_PrintF_uart3Mutex_Tx != NULL &&
		   l_PrintF_uart0Mutex_Rx != NULL && l_PrintF_uart1Mutex_Rx != NULL &&
		   l_PrintF_uart2Mutex_Rx != NULL && l_PrintF_uart3Mutex_Rx != NULL &&
		   l_PrintF_uartsSemaphores_Take_Ticks > 0)
		{
			// Inicialización de la UART correspondiente
			xSemaphoreTake(l_PrintF_uartsMutex, l_PrintF_uartsSemaphores_Take_Ticks);
			success = UARTInit(portNum, baudrate);
			xSemaphoreGive(l_PrintF_uartsMutex);
		}
	}

	return success;
}
/*-----------------------------------------------------------*/

bool_t PrintF_print(uint32_t portNum, char* formatedString, ...)
{
	bool_t success = FALSE;

	// Thread-safe: Variables distintas en cada llamada (se usa la pila).
	char msg[UART_TX_BUFSIZE]; memset(msg, 0x00, sizeof(msg));
	va_list args;
	va_start(args, formatedString);
	vsprintf(msg, formatedString, args);
	va_end(args);

	// if l_PrintF_uartsMutex != NULL, the others mutex
	// also are != NULL ( see 'PrintF_start()' )
	if(portNum < MAX_UART_PORTS && l_PrintF_uartsMutex != NULL)
	{
		if(portNum == UART_PORT_0 &&
		   xSemaphoreTake(l_PrintF_uart0Mutex_Tx, l_PrintF_uartsSemaphores_Take_Ticks) == pdTRUE)
		{
			UARTSend(portNum, (uint8_t *) msg, strlen(msg));	// sends the string

			success = TRUE;

			// We have finished accessing the shared resource.  Release the semaphore.
			xSemaphoreGive(l_PrintF_uart0Mutex_Tx);
		}
		else if(portNum == UART_PORT_1 &&
				xSemaphoreTake(l_PrintF_uart1Mutex_Tx, l_PrintF_uartsSemaphores_Take_Ticks) == pdTRUE)
		{
			UARTSend(portNum, (uint8_t *) msg, strlen(msg));	// sends the string

			success = TRUE;

			// We have finished accessing the shared resource.  Release the semaphore.
			xSemaphoreGive(l_PrintF_uart1Mutex_Tx);
		}
		else if(portNum == UART_PORT_2 &&
				xSemaphoreTake(l_PrintF_uart2Mutex_Tx, l_PrintF_uartsSemaphores_Take_Ticks) == pdTRUE)
		{
			UARTSend(portNum, (uint8_t *) msg, strlen(msg));	// sends the string

			success = TRUE;

			// We have finished accessing the shared resource.  Release the semaphore.
			xSemaphoreGive(l_PrintF_uart2Mutex_Tx);
		}
		else if(portNum == UART_PORT_3 &&
				xSemaphoreTake(l_PrintF_uart3Mutex_Tx, l_PrintF_uartsSemaphores_Take_Ticks) == pdTRUE)
		{
			UARTSend(portNum, (uint8_t *) msg, strlen(msg));	// sends the string

			success = TRUE;

			// We have finished accessing the shared resource.  Release the semaphore.
			xSemaphoreGive(l_PrintF_uart3Mutex_Tx);
		}
	}

	return success;
}
/*-----------------------------------------------------------*/

int32_t PrintF_scan(uint32_t portNum, char* scannedString, uint32_t timeoutMs)
{
	int32_t retVal = 0;

	// if l_PrintF_uartsMutex != NULL, the others mutex
	// also are != NULL ( see 'PrintF_start()' )
	if(portNum < MAX_UART_PORTS && l_PrintF_uartsMutex != NULL)
	{
		if(portNum == UART_PORT_0 &&
		   xSemaphoreTake(l_PrintF_uart0Mutex_Rx, l_PrintF_uartsSemaphores_Take_Ticks) == pdTRUE)
		{
			// Una vez obtenido el semáforo, las variables siguientes están protegidas
			// ya que son diferentes para cada puerto (para cada sentencia 'if')
			int oldCount = 0, newCount = 0;

			// clean RX buff
			UARTClearRx(portNum);

			LPC_UART0->IER = IER_THRE | IER_RLS | IER_RBR; 	// enable interrupt RX UART0

			Utils_delayMs(FALSE, timeoutMs);	// wait some time for receive some commands

			do{
				oldCount = newCount;
				Utils_delayMs(FALSE, 5);
				newCount = UART0Count;
			}while(oldCount != newCount);	// wait a little time while we have new chars

			LPC_UART0->IER = IER_THRE | IER_RLS;	// disable interrupt RX UART0

			strncpy((char*)scannedString, (char*)UART0Buffer, newCount);
			scannedString[newCount] = 0;	// add end string. Now we have a string.

			// Valor de retorno
			retVal = newCount;

			// We have finished accessing the shared resource.  Release the semaphore.
			xSemaphoreGive(l_PrintF_uart0Mutex_Rx);
		}
		else if(portNum == UART_PORT_1 &&
				xSemaphoreTake(l_PrintF_uart1Mutex_Rx, l_PrintF_uartsSemaphores_Take_Ticks) == pdTRUE)
		{
			// Una vez obtenido el semáforo, las variables siguientes están protegidas
			// ya que son diferentes para cada puerto (para cada sentencia 'if')
			int oldCount = 0, newCount = 0;

			// clean RX buff
			UARTClearRx(portNum);

			LPC_UART1->IER = IER_THRE | IER_RLS | IER_RBR; 	// enable interrupt RX UART1

			Utils_delayMs(FALSE, timeoutMs);	// wait some time for receive some commands

			do{
				oldCount = newCount;
				Utils_delayMs(FALSE, 5);
				newCount = UART1Count;
			}while(oldCount != newCount); 			// wait a little time while we have new chars

			LPC_UART1->IER = IER_THRE | IER_RLS;	// disable interrupt RX UART1

			strncpy((char*)scannedString, (char*)UART1Buffer, newCount);
			scannedString[newCount] = 0;	// add end string. Now we have a string.

			// Valor de retorno
			retVal = newCount;

			// We have finished accessing the shared resource.  Release the semaphore.
			xSemaphoreGive(l_PrintF_uart1Mutex_Rx);
		}
		else if(portNum == UART_PORT_2 &&
				xSemaphoreTake(l_PrintF_uart2Mutex_Rx, l_PrintF_uartsSemaphores_Take_Ticks) == pdTRUE)
		{
			// Una vez obtenido el semáforo, las variables siguientes están protegidas
			// ya que son diferentes para cada puerto (para cada sentencia 'if')
			int oldCount = 0, newCount = 0;

			// clean RX buff
			UARTClearRx(portNum);

			LPC_UART2->IER = IER_THRE | IER_RLS | IER_RBR; 	// enable interrupt RX UART2

			Utils_delayMs(FALSE, timeoutMs);	// wait sometime for receive some commands

			do{
				oldCount = newCount;
				Utils_delayMs(FALSE, 5);
				newCount = UART2Count;
			}while(oldCount != newCount); 			// wait a little time while we have new chars

			LPC_UART2->IER = IER_THRE | IER_RLS;	// disable interrupt RX UART2

			strncpy((char*)scannedString, (char*)UART2Buffer, newCount);
			scannedString[newCount] = 0;	// add end string. Now we have a string.

			// Valor de retorno
			retVal = newCount;

			// We have finished accessing the shared resource.  Release the semaphore.
			xSemaphoreGive(l_PrintF_uart2Mutex_Rx);
		}
		else if(portNum == UART_PORT_3 &&
				xSemaphoreTake(l_PrintF_uart3Mutex_Rx, l_PrintF_uartsSemaphores_Take_Ticks) == pdTRUE)
		{
			// Una vez obtenido el semáforo, las variables siguientes están protegidas
			// ya que son diferentes para cada puerto (para cada sentencia 'if')
			int oldCount = 0, newCount = 0;

			// clean RX buff
			UARTClearRx(portNum);

			LPC_UART3->IER = IER_THRE | IER_RLS | IER_RBR; 	// enable interrupt RX UART3

			Utils_delayMs(FALSE, timeoutMs);	// wait sometime for receive some commands

			do{
				oldCount = newCount;
				Utils_delayMs(FALSE, 5);
				newCount = UART3Count;
			}while(oldCount != newCount); 			// wait a little time while we have new chars

			LPC_UART3->IER = IER_THRE | IER_RLS;	// disable interrupt RX UART3

			strncpy((char*)scannedString, (char*)UART3Buffer, newCount);
			scannedString[newCount] = 0;	// add end string. Now we have a string.

			// Valor de retorno
			retVal = newCount;

			// We have finished accessing the shared resource.  Release the semaphore.
			xSemaphoreGive(l_PrintF_uart3Mutex_Rx);
		}
	}

	return retVal;
}
/*-----------------------------------------------------------*/
