/*
 * rtc.c
 *
 * Copyright(C) NXP Semiconductors, 2014
 * All rights reserved.
 *
 * Modificado: Miguel A. Fdez.
 *
 */

#include "FreeRTOS.h"
#include "semphr.h"
#include "LPC17xx.h"
#include "type.h"
#include "log.h"
#include "rtc.h"

/*-----------------------------------------------------------*/

#define rtcDEBUG	0 //  debug mode ?

// The time to wait for the semaphore in ticks.
// Defined in rtc.h in milliseconds.
volatile portTickType l_rtc_semaphoreTake_Ticks = 0;

// The RTC semaphore
volatile xSemaphoreHandle l_rtc_mutex = NULL;

/*-----------------------------------------------------------*/

// Interrupción por alarma de tiempo generada por el RTC para el wake-up del uP.
void RTC_IRQHandler(void)
{
	if(Rtc_getIntPending(LPC_RTC, RTC_INT_ALARM))
	{
		Rtc_clearIntPending(LPC_RTC, RTC_INT_ALARM);
	}
}
/*-----------------------------------------------------------*/

/* Initialize the RTC peripheral */
void Rtc_init(LPC_RTC_TypeDef *pRTC)
{
	// Semaphore initialization
	Rtc_semaphoreInit();

	// Power RTC
	LPC_SC->PCONP |= 1<<9;

	/* Disable RTC */
	Rtc_enable(pRTC, DISABLE); // tiene el semáforo

	/* Disable Calibration */
	Rtc_calibCounterCmd(pRTC, DISABLE); // tiene el semáforo

	/* Reset RTC Clock */
	Rtc_resetClockTickCounter(pRTC); // tiene el semáforo

	if(xSemaphoreTake(l_rtc_mutex, l_rtc_semaphoreTake_Ticks) == pdTRUE)
	{
		/* Clear counter increment and alarm interrupt */
		pRTC->ILR = RTC_IRL_RTCCIF | RTC_IRL_RTCALF;
		while (pRTC->ILR != 0) {}

		/* Clear all register to be default */
		pRTC->CIIR = 0x00;
		pRTC->AMR = 0xFF;
		pRTC->CALIBRATION = 0x00;

		xSemaphoreGive(l_rtc_mutex);
	}
}
/*-----------------------------------------------------------*/

void Rtc_semaphoreInit(void)
{
	// Debido a que la inicialización del RTC (Rtc_init()) solo se hace cuando se alimenta al módulo por primera vez,
	// en los sucesivos cambios de estado del uP a modo deep power-down, hay que iniciar explicitamente el semáforo

	// Si aún no se ha creado, se crea el semáforo
	if(l_rtc_mutex == NULL) l_rtc_mutex = xSemaphoreCreateMutex();

	// Si aún no se ha hecho, se hace la conversión a ticks del tiempo de espera para obtener el semáforo
	if(l_rtc_semaphoreTake_Ticks == 0) l_rtc_semaphoreTake_Ticks = (portTickType) rtcSEMAPHORE_TAKE_WAIT_MS / portTICK_RATE_MS;
}
/*-----------------------------------------------------------*/

/*De-initialize the RTC peripheral */
void Rtc_deInit(LPC_RTC_TypeDef *pRTC)
{
	if(xSemaphoreTake(l_rtc_mutex, l_rtc_semaphoreTake_Ticks) == pdTRUE)
	{
		pRTC->CCR = 0x00;
		LPC_SC->PCONP |= ~(1 << 9);

		xSemaphoreGive(l_rtc_mutex);
	}
}
/*-----------------------------------------------------------*/

/* Reset clock tick counter in the RTC peripheral */
void Rtc_resetClockTickCounter(LPC_RTC_TypeDef *pRTC)
{
	if(xSemaphoreTake(l_rtc_mutex, l_rtc_semaphoreTake_Ticks) == pdTRUE)
	{
		do {
			/* Reset RTC clock*/
			pRTC->CCR |= RTC_CCR_CTCRST;
		} while ((pRTC->CCR & RTC_CCR_CTCRST) != RTC_CCR_CTCRST);

		do {
			/* Finish resetting RTC clock */
			pRTC->CCR &= (~RTC_CCR_CTCRST) & RTC_CCR_BITMASK;
		} while (pRTC->CCR & RTC_CCR_CTCRST);

		xSemaphoreGive(l_rtc_mutex);
	}
}
/*-----------------------------------------------------------*/

/* Start/Stop RTC peripheral */
void Rtc_enable(LPC_RTC_TypeDef *pRTC, FunctionalState NewState)
{
	if(xSemaphoreTake(l_rtc_mutex, l_rtc_semaphoreTake_Ticks) == pdTRUE)
	{
		if (NewState == ENABLE) {
			do {
				pRTC->CCR |= RTC_CCR_CLKEN;
			} while ((pRTC->CCR & RTC_CCR_CLKEN) == 0);
		}
		else {
			pRTC->CCR &= (~RTC_CCR_CLKEN) & RTC_CCR_BITMASK;
		}

		xSemaphoreGive(l_rtc_mutex);
	}
}
/*-----------------------------------------------------------*/

/* Enable/Disable Counter increment interrupt for a time type in the RTC peripheral */
void Rtc_cntIncrIntConfig(LPC_RTC_TypeDef *pRTC, uint32_t cntrMask, FunctionalState NewState)
{
	if(xSemaphoreTake(l_rtc_mutex, l_rtc_semaphoreTake_Ticks) == pdTRUE)
	{
		if (NewState == ENABLE) {
			pRTC->CIIR |= cntrMask;
		}

		else {
			pRTC->CIIR &= (~cntrMask) & RTC_AMR_CIIR_BITMASK;
			while (pRTC->CIIR & cntrMask) {}
		}

		xSemaphoreGive(l_rtc_mutex);
	}
}
/*-----------------------------------------------------------*/

/* Enable/Disable Alarm interrupt for a time type in the RTC peripheral */
void Rtc_alarmIntConfig(LPC_RTC_TypeDef *pRTC, uint32_t alarmMask, FunctionalState NewState)
{
	if(xSemaphoreTake(l_rtc_mutex, l_rtc_semaphoreTake_Ticks) == pdTRUE)
	{
		if (NewState == ENABLE) {
			pRTC->AMR &= (~alarmMask) & RTC_AMR_CIIR_BITMASK;
		}
		else {
			pRTC->AMR |= (alarmMask);
			while ((pRTC->AMR & alarmMask) == 0) {}
		}

		xSemaphoreGive(l_rtc_mutex);
	}
}
/*-----------------------------------------------------------*/

/* Set full time in the RTC peripheral */
void Rtc_setFullTime(LPC_RTC_TypeDef *pRTC, RTC_TIME_T *pFullTime)
{
	if(xSemaphoreTake(l_rtc_mutex, l_rtc_semaphoreTake_Ticks) == pdTRUE)
	{
		RTC_TIMEINDEX_T i;
		uint32_t ccr_val = pRTC->CCR;

		/* Temporarily disable */
		if (ccr_val & RTC_CCR_CLKEN) {
			pRTC->CCR = ccr_val & (~RTC_CCR_CLKEN) & RTC_CCR_BITMASK;
		}

		/* Date time setting */
		for (i = RTC_TIMETYPE_SECOND; i < RTC_TIMETYPE_LAST; i++) {
			pRTC->TIME[i] = pFullTime->time[i];
		}

		/* Restore to old setting */
		pRTC->CCR = ccr_val;

		if(rtcDEBUG)
		{
			Log_log("Tiempo ajustado: %.2d:%.2d:%.2d %.2d/%.2d/%.4d\r\n\r\n",
					 pFullTime->time[RTC_TIMETYPE_HOUR],
					 pFullTime->time[RTC_TIMETYPE_MINUTE],
					 pFullTime->time[RTC_TIMETYPE_SECOND],
					 pFullTime->time[RTC_TIMETYPE_DAYOFMONTH],
					 pFullTime->time[RTC_TIMETYPE_MONTH],
					 pFullTime->time[RTC_TIMETYPE_YEAR]);
		}

		xSemaphoreGive(l_rtc_mutex);
	}
}
/*-----------------------------------------------------------*/

/* Get full time from the RTC peripheral */
void Rtc_getFullTime(LPC_RTC_TypeDef *pRTC, RTC_TIME_T *pFullTime)
{
	if(xSemaphoreTake(l_rtc_mutex, l_rtc_semaphoreTake_Ticks) == pdTRUE)
	{
		RTC_TIMEINDEX_T i;
		uint32_t secs = 0xFF;

		/* Read full time, but verify second tick didn't change during the read. If
		   it did, re-read the time again so it will be consistent across all fields. */
		while (secs != pRTC->TIME[RTC_TIMETYPE_SECOND]) {
			secs = pFullTime->time[RTC_TIMETYPE_SECOND] = pRTC->TIME[RTC_TIMETYPE_SECOND];
			for (i = RTC_TIMETYPE_MINUTE; i < RTC_TIMETYPE_LAST; i++) {
				pFullTime->time[i] = pRTC->TIME[i];
			}
		}

		if(rtcDEBUG)
		{
			Log_log("Tiempo leido: %.2d:%.2d:%.2d %.2d/%.2d/%.4d\r\n\r\n",
					 pFullTime->time[RTC_TIMETYPE_HOUR],
					 pFullTime->time[RTC_TIMETYPE_MINUTE],
					 pFullTime->time[RTC_TIMETYPE_SECOND],
					 pFullTime->time[RTC_TIMETYPE_DAYOFMONTH],
					 pFullTime->time[RTC_TIMETYPE_MONTH],
					 pFullTime->time[RTC_TIMETYPE_YEAR]);
		}

		xSemaphoreGive(l_rtc_mutex);
	}
}
/*-----------------------------------------------------------*/

/* Set full alarm time in the RTC peripheral */
void Rtc_setFullAlarmTime(LPC_RTC_TypeDef *pRTC, RTC_TIME_T *pFullTime)
{
	if(xSemaphoreTake(l_rtc_mutex, l_rtc_semaphoreTake_Ticks) == pdTRUE)
	{
		RTC_TIMEINDEX_T i;

		for (i = RTC_TIMETYPE_SECOND; i < RTC_TIMETYPE_LAST; i++) {
			pRTC->ALRM[i] = pFullTime->time[i];
		}

		xSemaphoreGive(l_rtc_mutex);
	}
}
/*-----------------------------------------------------------*/

/* Get full alarm time in the RTC peripheral */
void Rtc_getFullAlarmTime(LPC_RTC_TypeDef *pRTC, RTC_TIME_T *pFullTime)
{
	if(xSemaphoreTake(l_rtc_mutex, l_rtc_semaphoreTake_Ticks) == pdTRUE)
	{
		RTC_TIMEINDEX_T i;

		for (i = RTC_TIMETYPE_SECOND; i < RTC_TIMETYPE_LAST; i++) {
			pFullTime->time[i] = pRTC->ALRM[i];
		}

		xSemaphoreGive(l_rtc_mutex);
	}
}
/*-----------------------------------------------------------*/

/* Enable/Disable calibration counter in the RTC peripheral */
void Rtc_calibCounterCmd(LPC_RTC_TypeDef *pRTC, FunctionalState NewState)
{
	if(xSemaphoreTake(l_rtc_mutex, l_rtc_semaphoreTake_Ticks) == pdTRUE)
	{
		if (NewState == ENABLE) {
			do {
				pRTC->CCR &= (~RTC_CCR_CCALEN) & RTC_CCR_BITMASK;
			} while (pRTC->CCR & RTC_CCR_CCALEN);
		}
		else {
			pRTC->CCR |= RTC_CCR_CCALEN;
		}

		xSemaphoreGive(l_rtc_mutex);
	}
}
/*-----------------------------------------------------------*/

/* Programs the RTC for the next wake-up */
void Rtc_programNextWakeUp(uint32_t secondsPowerDownInterval)
{
	uint32_t seconds, minutes, hours;
	RTC_TIME_T fullTime;

	// Como solo utilizamos el RTC para generar la alarma de wake-up
	// iniciamos la fecha-hora del RTC a 0

	fullTime.time[RTC_TIMETYPE_SECOND] = 0;
	fullTime.time[RTC_TIMETYPE_MINUTE] = 0;
	fullTime.time[RTC_TIMETYPE_HOUR] = 0;
	fullTime.time[RTC_TIMETYPE_DAYOFMONTH] = 0;
	fullTime.time[RTC_TIMETYPE_DAYOFWEEK] = 0;
	fullTime.time[RTC_TIMETYPE_DAYOFYEAR] = 0;
	fullTime.time[RTC_TIMETYPE_MONTH] = 0;
	fullTime.time[RTC_TIMETYPE_YEAR] = 0;

	Rtc_setFullTime(LPC_RTC, &fullTime);

	// Calculamos la fecha-hora del próximo wake-up

	seconds = secondsPowerDownInterval;

	minutes  = seconds / 60;
	seconds -= minutes * 60;

	hours    = minutes / 60;
	minutes -= hours   * 60;

	fullTime.time[RTC_TIMETYPE_SECOND] = seconds;
	fullTime.time[RTC_TIMETYPE_MINUTE] = minutes;
	fullTime.time[RTC_TIMETYPE_HOUR] = hours;

	// Programamos el próximo wake-up
	Rtc_setFullAlarmTime(LPC_RTC, &fullTime);

}
/*-----------------------------------------------------------*/

/* Sets the RTC user register value */
bool_t Rtc_setUserRegisterValue(uint8_t regNbr, uint32_t regValue, uint8_t word)
{
	if(regNbr > 4) return FALSE; // El RTC tiene 5 registros de usuario

	if(xSemaphoreTake(l_rtc_mutex, l_rtc_semaphoreTake_Ticks) == pdTRUE)
	{
		// escribimos en el registro la palabra (16 bits) de menor peso, la de mayor peso o ambas
		uint32_t auxReg = LPC_RTC->GPREG[regNbr];

		if(word == RTC_REG_LOW_WORD)
		{
			auxReg &= 0xFFFF0000;
			auxReg |= (regValue & 0x0000FFFF);
		}
		else if(word == RTC_REG_HIGHT_WORD)
		{
			auxReg &= 0x0000FFFF;
			auxReg |= (regValue << 16);
		}
		else if(word == RTC_REG_BOTH_WORDS)
		{
			auxReg = regValue;
		}

		LPC_RTC->GPREG[regNbr] = auxReg;

		xSemaphoreGive(l_rtc_mutex);
	}

	return TRUE;
}
/*-----------------------------------------------------------*/

/* Gets the RTC user register value */
uint32_t Rtc_getUserRegisterValue(uint8_t regNbr, uint8_t word)
{
	if(regNbr > 4) return 0; // El RTC tiene 5 registros de usuario

	uint32_t ret = 0;

	if(xSemaphoreTake(l_rtc_mutex, l_rtc_semaphoreTake_Ticks) == pdTRUE)
	{
		// retornamos del registro la palabra (16 bits) de menor peso, la de mayor peso o ambas
		ret = LPC_RTC->GPREG[regNbr];

		if(word == RTC_REG_LOW_WORD) ret &= 0x0000FFFF;
		else if(word == RTC_REG_HIGHT_WORD) ret = (ret & 0xFFFF0000) >> 16;
		else if(word == RTC_REG_BOTH_WORDS); // retorna el registro completo
		else ret = 0;

		xSemaphoreGive(l_rtc_mutex);
	}

	return ret;
}

//-----------------------------------------------------------------------
