/*
 * Utils.c
 *
 *  Created on: 8/4/2015
 *      Author: Miguel A. Fdez.
 */

#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"
#include "LPC17xx.h"
#include <string.h>
#include <stdlib.h>
#include <float.h>
#include "utils.h"

#include "uart.h"

// The time to wait for the semaphores in ticks.
// Defined in utils.h in milliseconds.
volatile portTickType l_Utils_semaphTake_Ticks = 0;

// Semaphore for Utils_delayMs() function
volatile xSemaphoreHandle l_Utils_delayMs_Mutex = NULL;
// Semaphore for Utils_simpleFilter() function
volatile xSemaphoreHandle l_Utils_simpleFilter_Mutex = NULL;
// Semaphore for Utils_get_crc16() function
volatile xSemaphoreHandle l_Utils_crc16_ccitt_Mutex = NULL;
// Semaphore for Utils_clearStr() function
volatile xSemaphoreHandle l_Utils_clearStr_Mutex = NULL;
// Semaphore for Utils_ipStrToUint32() function
volatile xSemaphoreHandle l_Utils_ipStrToUint32_Mutex = NULL;
// Semaphore for Utils_getSubStr() function
volatile xSemaphoreHandle l_Utils_getValueByKeyByLen_Mutex = NULL;
// Semaphore for Utils_getValueFromKey() function
volatile xSemaphoreHandle l_Utils_getValueByKey_Mutex = NULL;

// Table to calculate the CRC16 CCITT
static const uint16_t l_crc16_ccitt_table[256] =
{
    0x0000, 0x1021, 0x2042, 0x3063, 0x4084, 0x50A5, 0x60C6, 0x70E7,
    0x8108, 0x9129, 0xA14A, 0xB16B, 0xC18C, 0xD1AD, 0xE1CE, 0xF1EF,
    0x1231, 0x0210, 0x3273, 0x2252, 0x52B5, 0x4294, 0x72F7, 0x62D6,
    0x9339, 0x8318, 0xB37B, 0xA35A, 0xD3BD, 0xC39C, 0xF3FF, 0xE3DE,
    0x2462, 0x3443, 0x0420, 0x1401, 0x64E6, 0x74C7, 0x44A4, 0x5485,
    0xA56A, 0xB54B, 0x8528, 0x9509, 0xE5EE, 0xF5CF, 0xC5AC, 0xD58D,
    0x3653, 0x2672, 0x1611, 0x0630, 0x76D7, 0x66F6, 0x5695, 0x46B4,
    0xB75B, 0xA77A, 0x9719, 0x8738, 0xF7DF, 0xE7FE, 0xD79D, 0xC7BC,
    0x48C4, 0x58E5, 0x6886, 0x78A7, 0x0840, 0x1861, 0x2802, 0x3823,
    0xC9CC, 0xD9ED, 0xE98E, 0xF9AF, 0x8948, 0x9969, 0xA90A, 0xB92B,
    0x5AF5, 0x4AD4, 0x7AB7, 0x6A96, 0x1A71, 0x0A50, 0x3A33, 0x2A12,
    0xDBFD, 0xCBDC, 0xFBBF, 0xEB9E, 0x9B79, 0x8B58, 0xBB3B, 0xAB1A,
    0x6CA6, 0x7C87, 0x4CE4, 0x5CC5, 0x2C22, 0x3C03, 0x0C60, 0x1C41,
    0xEDAE, 0xFD8F, 0xCDEC, 0xDDCD, 0xAD2A, 0xBD0B, 0x8D68, 0x9D49,
    0x7E97, 0x6EB6, 0x5ED5, 0x4EF4, 0x3E13, 0x2E32, 0x1E51, 0x0E70,
    0xFF9F, 0xEFBE, 0xDFDD, 0xCFFC, 0xBF1B, 0xAF3A, 0x9F59, 0x8F78,
    0x9188, 0x81A9, 0xB1CA, 0xA1EB, 0xD10C, 0xC12D, 0xF14E, 0xE16F,
    0x1080, 0x00A1, 0x30C2, 0x20E3, 0x5004, 0x4025, 0x7046, 0x6067,
    0x83B9, 0x9398, 0xA3FB, 0xB3DA, 0xC33D, 0xD31C, 0xE37F, 0xF35E,
    0x02B1, 0x1290, 0x22F3, 0x32D2, 0x4235, 0x5214, 0x6277, 0x7256,
    0xB5EA, 0xA5CB, 0x95A8, 0x8589, 0xF56E, 0xE54F, 0xD52C, 0xC50D,
    0x34E2, 0x24C3, 0x14A0, 0x0481, 0x7466, 0x6447, 0x5424, 0x4405,
    0xA7DB, 0xB7FA, 0x8799, 0x97B8, 0xE75F, 0xF77E, 0xC71D, 0xD73C,
    0x26D3, 0x36F2, 0x0691, 0x16B0, 0x6657, 0x7676, 0x4615, 0x5634,
    0xD94C, 0xC96D, 0xF90E, 0xE92F, 0x99C8, 0x89E9, 0xB98A, 0xA9AB,
    0x5844, 0x4865, 0x7806, 0x6827, 0x18C0, 0x08E1, 0x3882, 0x28A3,
    0xCB7D, 0xDB5C, 0xEB3F, 0xFB1E, 0x8BF9, 0x9BD8, 0xABBB, 0xBB9A,
    0x4A75, 0x5A54, 0x6A37, 0x7A16, 0x0AF1, 0x1AD0, 0x2AB3, 0x3A92,
    0xFD2E, 0xED0F, 0xDD6C, 0xCD4D, 0xBDAA, 0xAD8B, 0x9DE8, 0x8DC9,
    0x7C26, 0x6C07, 0x5C64, 0x4C45, 0x3CA2, 0x2C83, 0x1CE0, 0x0CC1,
    0xEF1F, 0xFF3E, 0xCF5D, 0xDF7C, 0xAF9B, 0xBFBA, 0x8FD9, 0x9FF8,
    0x6E17, 0x7E36, 0x4E55, 0x5E74, 0x2E93, 0x3EB2, 0x0ED1, 0x1EF0
};

/*-----------------------------------------------------------*/

void Utils_delayMs(bool_t isPassive, uint32_t msToDelay)
{
	// Si aún no se ha iniciado el semáforo de esta función, se inicia
	if(l_Utils_delayMs_Mutex == NULL) l_Utils_delayMs_Mutex = xSemaphoreCreateMutex();

	// Si aún no se ha iniciado el tiempo de espera en ticks, se inicia
	if(l_Utils_semaphTake_Ticks == 0)
		l_Utils_semaphTake_Ticks = (portTickType) UTILS_SEMAPH_TAKE_WAIT_MS / portTICK_RATE_MS;

	if (isPassive == TRUE)	// Passive delay for Tasks
	{
		// código protegido
		if(xSemaphoreTake(l_Utils_delayMs_Mutex, l_Utils_semaphTake_Ticks) == pdTRUE)
		{
			portTickType ticksToDelay = (portTickType) msToDelay / portTICK_RATE_MS;
			vTaskDelay(ticksToDelay);

			xSemaphoreGive(l_Utils_delayMs_Mutex);
		}
	}
	else	// Active delay for Tasks and for sequential code
	{
		uint32_t i;
		for(i=0; i<(((uint32_t) 7140 ) * msToDelay); i++); // 7140 iterations ≈ 1 millisecond
	}

}
/*-----------------------------------------------------------*/

double Utils_simpleFilter(double *valuesToFilter, uint32_t numberOfValuesToFilter)
{
	int32_t i;
	double maxValue = DBL_MIN, minValue = DBL_MAX, acumulatedValue = 0.0;

	// Si aún no se ha iniciado el semáforo de esta función, se inicia
	if(l_Utils_simpleFilter_Mutex == NULL) l_Utils_simpleFilter_Mutex = xSemaphoreCreateMutex();

	// Si aún no se ha iniciado el tiempo de espera en ticks, se inicia
	if(l_Utils_semaphTake_Ticks == 0)
		l_Utils_semaphTake_Ticks = (portTickType) UTILS_SEMAPH_TAKE_WAIT_MS / portTICK_RATE_MS;

	// función protegida
	if(xSemaphoreTake(l_Utils_simpleFilter_Mutex, l_Utils_semaphTake_Ticks) == pdTRUE)
	{
		for(i=0; i<numberOfValuesToFilter; i++)
		{
			if(valuesToFilter[i] > maxValue) maxValue = valuesToFilter[i]; 	// maximum value ?
			if(valuesToFilter[i] < minValue) minValue = valuesToFilter[i];	// minimum value ?

			acumulatedValue += valuesToFilter[i];	// Accumulate the value.
		}

		xSemaphoreGive(l_Utils_simpleFilter_Mutex);
	}

	return ((acumulatedValue - maxValue - minValue) / (numberOfValuesToFilter - 2));
}
/*-----------------------------------------------------------*/

uint16_t Utils_crc16_ccitt(const char data[], uint16_t dataLen, uint16_t initialCrc)
{
	// CRC 16 CCITT polynomial. Equation: x^16 + x^12 + x^5 + 1.
	// Calculated with the table aid to improbe speed.

	uint16_t i;

	// Si aún no se ha iniciado el semáforo de esta función, se inicia
	if(l_Utils_crc16_ccitt_Mutex == NULL) l_Utils_crc16_ccitt_Mutex = xSemaphoreCreateMutex();

	// Si aún no se ha iniciado el tiempo de espera en ticks, se inicia
	if(l_Utils_semaphTake_Ticks == 0)
		l_Utils_semaphTake_Ticks = (portTickType) UTILS_SEMAPH_TAKE_WAIT_MS / portTICK_RATE_MS;

	// función protegida
	if(xSemaphoreTake(l_Utils_crc16_ccitt_Mutex, l_Utils_semaphTake_Ticks) == pdTRUE)
	{
		for(i=0; i<dataLen; i++)
		{
			uint16_t tmp = (initialCrc >> 8) ^ (uint16_t) data[i];
			initialCrc = ((uint16_t)(initialCrc << 8)) ^ l_crc16_ccitt_table[tmp];
		}

		xSemaphoreGive(l_Utils_crc16_ccitt_Mutex);
	}

    return initialCrc;
}
/*-----------------------------------------------------------*/

uint16_t Utils_clearString(char* inputStr, char* compareStr, char* returnedSubStr)
{
	// Si aún no se ha iniciado el semáforo de esta función, se inicia
	if(l_Utils_clearStr_Mutex == NULL) l_Utils_clearStr_Mutex = xSemaphoreCreateMutex();

	// Si aún no se ha iniciado el tiempo de espera en ticks, se inicia
	if(l_Utils_semaphTake_Ticks == 0)
		l_Utils_semaphTake_Ticks = (portTickType) UTILS_SEMAPH_TAKE_WAIT_MS / portTICK_RATE_MS;

	// función protegida
	if(xSemaphoreTake(l_Utils_clearStr_Mutex, l_Utils_semaphTake_Ticks) == pdTRUE)
	{
		uint16_t subStrLen = strlen(inputStr) - strlen(compareStr);

		strncpy(returnedSubStr, inputStr, subStrLen);

		xSemaphoreGive(l_Utils_clearStr_Mutex);
	}

	return strlen(returnedSubStr);
}
/*-----------------------------------------------------------*/

uint32_t Utils_ipStrToUint32(char* ipStr)
{
	const char separador[2] = {'.', 0};
	uint8_t i = 0, ip_byte[4];
	uint32_t dir_ip_int = 0;

	// Si aún no se ha iniciado el semáforo de esta función, se inicia
	if(l_Utils_ipStrToUint32_Mutex == NULL) l_Utils_ipStrToUint32_Mutex = xSemaphoreCreateMutex();

	// Si aún no se ha iniciado el tiempo de espera en ticks, se inicia
	if(l_Utils_semaphTake_Ticks == 0)
		l_Utils_semaphTake_Ticks = (portTickType) UTILS_SEMAPH_TAKE_WAIT_MS / portTICK_RATE_MS;

	// función protegida
	if(xSemaphoreTake(l_Utils_ipStrToUint32_Mutex, l_Utils_semaphTake_Ticks) == pdTRUE)
	{
		char ipStr_Copy[64]; memset(ipStr_Copy, 0x00, sizeof(ipStr_Copy));
		strcpy(ipStr_Copy, ipStr);

		/* get the first token */
		char* token = strtok(ipStr_Copy, separador);

		/* walk through other tokens */
		while( token != NULL )
		{
			ip_byte[i++] = atoi(token);
			token = strtok(NULL, separador);
		}

		dir_ip_int = (ip_byte[0] << 24) | (ip_byte[1] << 16) | (ip_byte[2] << 8) | ip_byte[3];

		xSemaphoreGive(l_Utils_ipStrToUint32_Mutex);
	}

	return dir_ip_int;
}
/*-----------------------------------------------------------*/

bool_t Utils_getValueByKeyByLen(char* inputStr, char* key, uint32_t valueLen, char* valueOut)
{
	bool_t success = FALSE;
	uint32_t i;
	int32_t keyPosition = -1;

	// Si aún no se ha iniciado el semáforo de esta función, se inicia
	if(l_Utils_getValueByKeyByLen_Mutex == NULL) l_Utils_getValueByKeyByLen_Mutex = xSemaphoreCreateMutex();

	// Si aún no se ha iniciado el tiempo de espera en ticks, se inicia
	if(l_Utils_semaphTake_Ticks == 0)
		l_Utils_semaphTake_Ticks = (portTickType) UTILS_SEMAPH_TAKE_WAIT_MS / portTICK_RATE_MS;

	// función protegida
	if(xSemaphoreTake(l_Utils_getValueByKeyByLen_Mutex, l_Utils_semaphTake_Ticks) == pdTRUE)
	{
		// Calculamos la posición de la clave si existe
		if (inputStr && key && valueOut && (valueLen > 0))
		{
			char* resultado = strstr(inputStr, key);
			if (resultado)
			{
				keyPosition = resultado - inputStr;
			}
		}

		// Si encontramos la clave, extraemos el valor
		if((keyPosition > -1) && ((keyPosition + strlen(key) + valueLen) <=  strlen(inputStr)))
		{
			inputStr = inputStr + keyPosition + strlen(key);

			for(i=0; i<valueLen; i++)
			{
				*valueOut = *inputStr;
				valueOut ++;
				inputStr ++;
			}

			*valueOut = '\0';

			success = TRUE;
		}

		xSemaphoreGive(l_Utils_getValueByKeyByLen_Mutex);
	}

	return success;
}
/*-----------------------------------------------------------*/

bool_t Utils_getValueByKey(char* inputStr, const char tokenDelimiters[], char* key, char* returnedValue)
{
	bool_t success = FALSE;

	// Si aún no se ha iniciado el semáforo de esta función, se inicia
	if(l_Utils_getValueByKey_Mutex == NULL) l_Utils_getValueByKey_Mutex = xSemaphoreCreateMutex();

	// Si aún no se ha iniciado el tiempo de espera en ticks, se inicia
	if(l_Utils_semaphTake_Ticks == 0)
		l_Utils_semaphTake_Ticks = (portTickType) UTILS_SEMAPH_TAKE_WAIT_MS / portTICK_RATE_MS;

	// función protegida
	if(xSemaphoreTake(l_Utils_getValueByKey_Mutex, l_Utils_semaphTake_Ticks) == pdTRUE)
	{
		char l_wiflyModulRxStr[UART_RX_BUFSIZE]; memset(l_wiflyModulRxStr, 0x00, sizeof(l_wiflyModulRxStr));

		// trabajamos con una copia de la cadena de entrada para que no quede troceada por 'strtok()'
		strcpy(l_wiflyModulRxStr, inputStr);

		char *token = strtok(l_wiflyModulRxStr, tokenDelimiters);

		// buscamos el par clave/valor
		while(token != NULL)
		{
			if(strstr(token, key) != NULL)
			{
				int32_t valueLen = strlen(token) - strlen(key);
				Utils_getValueByKeyByLen(token, key, valueLen, returnedValue);
				success = TRUE;
				break;
			}

			token = strtok(NULL, tokenDelimiters);
		}

		xSemaphoreGive(l_Utils_getValueByKey_Mutex);
	}

	return success;
}
/*-----------------------------------------------------------*/
