/*
 * wifly.c
 *
 *  Created on: 27/10/2015
 *      Author: Miguel A. Fdez.
 */

#include "FreeRTOS.h"
#include "semphr.h"
#include "LPC17xx.h"
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "utils.h"
#include "gpio.h"
#include "log.h"
#include "uart.h"
#include "printF.h"
#include "wifly.h"

#define WIFLY_DEBUG	0 // WiFly debug mode ?

// Local strings to work with this module
char l_wiflyModulTxStr[UART_TX_BUFSIZE];
char l_wiflyModulRxStr[UART_RX_BUFSIZE];

// The time to wait for the semaphore in ticks.
// Defined in Wifly.h in milliseconds.
volatile portTickType l_Wifly_semaphoreTake_Ticks = 0;

// The LPC1769 UART port to comunicate with the Wifly
volatile uint32_t l_Wifly_uartPort = MAX_UART_PORTS;

// The wifly semaphore
volatile xSemaphoreHandle l_Wifly_mutex = NULL;

// Flags the initial connection to the host
volatile bool_t l_openConnectionToHost = TRUE;

/*-----------------------------------------------------------*/

// Funciones locales

/*-----------------------------------------------------------*/

/** Inicializa los pines para el WiFly.
 *
 * @preconditions Los pines del LPC1769 están conectados a
 *                los pines correspondientes del Wifly (ver esquema).
 *
 * @param No
 *
 * @return No
 */
static void _hardInit(void)
{
	// No es necesario programar los pines del micro como entrada-salida de
	// proposito general (GPIO). Por defecto en el inicio ya son GPIO, entradas con pull-up.

	// Programar los correspondientes pins del micro como salidas:

	// WiFly wake-up output
	GPIO_setDir(WIFLY_WAKEUP_PIN, GPIO_OUTPUT);
	GPIO_setValue(WIFLY_WAKEUP_PIN, FALSE);

	// WiFly hard reset output
	GPIO_setDir(WIFLY_RESET_PIN, GPIO_OUTPUT);
	GPIO_setValue(WIFLY_RESET_PIN, FALSE);
}
/*-----------------------------------------------------------*/

/** Hace un reset hardware del Wifly.
 *
 * @preconditions La salida digital correspondiente del LPC1769 está conectada a
 *                la entrada reset del Wifly.
 *
 * @param No
 *
 * @return No
 */
static void _hardReset(void)
{
	// Wifly reset output pin = 0
	GPIO_setValue(WIFLY_RESET_PIN, FALSE);

	// Espera 1 milisegundo (mínimo 160uS)
	Utils_delayMs(FALSE, 1);

	// Wifly reset output pin = 1
	GPIO_setValue(WIFLY_RESET_PIN, TRUE);
}
/*-----------------------------------------------------------*/

/** Envía una petición al Wifly por el puerto serie y obtiene la respuesta.
 *
 * No es tread-safe. Llamar con semáforo obtenido si es el caso.
 *
 * @preconditions El Wifly ha sido inicializado y está en modo comando
 *
 * @param requestStr 		  La cadena de caracteres con la petición.
 * @param paramStr 		      La cadena de caracteres con los parámetros adicionales de la petición,
 *                            NULL si no hay parámetros adicionales.
 * @param add_CR     		  TRUE si se ha de añadir retorno de carro ('\r') a la petición.
 *                            FALSE en caso contrario.
 * @param responseSuccess_Str La cadena de caracteres con el identificador para saber si el Wifly
 *                            ha interpretado bién el comando.
 * @param responseOut_Str     La cadena de caracteres de salida con la respuesta del Wifly.
 * @param responseTimeout_Ms  El tiempo de espera para la respuesta en milisegundos.
 *
 * @return TRUE or FALSE. Retorna TRUE si el proceso de petición-respuesta ha ido bien,
 *                        FALSE en caso contrario.
 */
static bool_t _request(char* requestStr, char* paramStr, bool_t add_CR,
					   char* responseSuccess_Str, char* responseOut_Str, uint32_t responseTimeout_Ms)
{
	bool_t success = FALSE;

	// Solicitud al Wifly
	if(requestStr != NULL)
	{
		char txStr[UART_TX_BUFSIZE]; memset(txStr, 0x00, sizeof(txStr));

		strcpy(txStr, requestStr);

		if(paramStr != NULL)
		{
			strcat(txStr, " ");
			strcat(txStr, paramStr);
		}

		if(add_CR == TRUE) strcat(txStr, "\r");

		success = PrintF_print(l_Wifly_uartPort, txStr);
	}
	else success = TRUE;

	// string para recepción desde el Wifly.
	char rxStr[UART_RX_BUFSIZE]; memset(rxStr, 0x00, sizeof(rxStr));

	if(success == TRUE) // Solicitud enviada correctamente al Wifly
	{
		// Esperamos la respuesta desde el Wifly
		uint32_t nbrChars = PrintF_scan(l_Wifly_uartPort, rxStr, responseTimeout_Ms);
		if(nbrChars) // Hay respuesta desde el Wifly
		{
			// ¿ Es correcta la respuesta desde el Wyfly ?
			if(strstr(rxStr, responseSuccess_Str) == NULL) success = FALSE;
		}
		else success = FALSE;
	}

	// Si se ha pasado una cadena para devolver la respuesta, copiar la respuesta
	// del Wifly en la cadena pasada como argumento
	if(responseOut_Str != NULL)	strcpy(responseOut_Str, rxStr);

	if(WIFLY_DEBUG) Log_log(rxStr);

	return success;
}
/*-----------------------------------------------------------*/

/** Extrae de una cadena de caracteres el valor de un campo definido por un identificador
 *  de entre un conjunto de campos delimitados por otro identificador.
 *
 * No es tread-safe. Llamar con semáforo obtenido si es el caso.
 *
 * @preconditions No
 *
 * @param wholeStr 		      La cadena de caracteres completa.
 * @param tokenDelimiter 	  El delimitador de los campos en la cadena.
 * @param fieldId     		  El identificador del campo a extraer.
 * @param fieldInit_Position  La posición inicial del valor del campo.
 * @param valueIncludeSpaces  TRUE or FALSE: True si el valor del campo contiene espacios, FALSE en caso contrario.
 * @param outField_Value      La cadena de caracteres de salida con el valor del campo
 * 							  definido por 'fieldId'.
 *
 * @return TRUE or FALSE. Retorna TRUE si ha encontrado el valor del campo definido por 'fieldId',
 *                        FALSE en caso contrario.
 */
static bool_t _extractFieldValue(char* wholeStr, char* tokenDelimiter, char* fieldId,
								 uint32_t fieldInit_Position, uint32_t valueIncludeSpaces, char* outField_Value)
{
	bool_t success = FALSE;

	char *token = strtok(wholeStr, tokenDelimiter);
	while(token != NULL)
	{
		if(strstr(token, fieldId) != NULL) break;
		token = strtok(NULL, tokenDelimiter);
	}

	if(token != NULL)
	{
		char rxStr[UART_RX_BUFSIZE]; memset(rxStr, 0x00, sizeof(rxStr));
		int i;
		for(i=fieldInit_Position; (token[i] != ' ' || valueIncludeSpaces) && token[i] != 0x00; i++)
			rxStr[i-fieldInit_Position] = token[i];

		strcpy(outField_Value, rxStr);

		if(strlen(rxStr) > 0) success = TRUE;
		else success = FALSE;
	}

	return success;
}
/*-----------------------------------------------------------*/

/** Establece el protocolo IP del cliente HTML del Wifly.
 *
 * No es tread-safe. Llamar con semáforo obtenido si es el caso.
 *
 * @preconditions El Wifly ha sido inicializado y está en modo comando
 *
 * @param ipProtocol   El protocolo IP (ver doc 'Wifly command reference - DS50002230A' pag.22)
 *
 * @return TRUE or FALSE. Retorna TRUE si el ajuste del protocolo IP ha ido bien,
 *                        FALSE en caso contrario.
 */
static bool_t _setIpProtocol(char* ipProtocol)
{
	bool_t success = FALSE;

	if(WIFLY_DEBUG) Log_log("--- Solicitud al Wifly: set ip proto ---");

	// Enviar la solicitud
	success = _request("set ip proto", ipProtocol, TRUE, "AOK", NULL, 100);

	return success;
}
/*-----------------------------------------------------------*/

/** Establece la dirección IP del cliente HTML del Wifly.
 *
 * No es tread-safe. Llamar con semáforo obtenido si es el caso.
 *
 * @preconditions El Wifly ha sido inicializado y está en modo comando
 *
 * @param ipAddress   La dirección IP (ver doc 'Wifly command reference - DS50002230A' pag.20)
 *
 * @return TRUE or FALSE. Retorna TRUE si el ajuste de la dirección IP ha ido bien,
 *                        FALSE en caso contrario.
 */
static bool_t _setIpAddress(char* ipAddress)
{
	bool_t success = FALSE;

	// Solo para debugar
	if(WIFLY_DEBUG) Log_log("--- Solicitud al Wifly: set ip address ---");

	// Enviar la solicitud
	success = _request("set ip address", ipAddress, TRUE, "AOK", NULL, 100);

	return success;
}
/*-----------------------------------------------------------*/

/** Establece el puerto remoto IP del cliente HTML del Wifly.
 *
 * No es tread-safe. Llamar con semáforo obtenido si es el caso.
 *
 * @preconditions El Wifly ha sido inicializado y está en modo comando
 *
 * @param ipRemote   El puerto remoto (ver doc 'Wifly command reference - DS50002230A' pag.22)
 *
 * @return TRUE or FALSE. Retorna TRUE si el ajuste del puerto ha ido bien,
 *                        FALSE en caso contrario.
 */
static bool_t _setIpRemote(char* ipRemote)
{
	bool_t success = FALSE;

	// Solo para debugar
	if(WIFLY_DEBUG) Log_log("--- Solicitud al Wifly: set ip remote ---");

	// Enviar la solicitud
	success = _request("set ip remote", ipRemote, TRUE, "AOK", NULL, 100);

	return success;
}
/*-----------------------------------------------------------*/

/** Establece el nombre del servidor para las conexiones TCP/IP del Wifly.
 *
 * No es tread-safe. Llamar con semáforo obtenido si es el caso.
 *
 * @preconditions El Wifly ha sido inicializado y está en modo comando
 *
 * @param dnsName   El nombre del servidor (ver doc 'Wifly command reference - DS50002230A' pag.18)
 *
 * @return TRUE or FALSE. Retorna TRUE si el ajuste del nombre del servidor ha ido bien,
 *                        FALSE en caso contrario.
 */
static bool_t _setDnsName(char* dnsName)
{
	bool_t success = FALSE;

	if(WIFLY_DEBUG) Log_log("--- Solicitud al Wifly: set dns name ---");

	// Enviar la solicitud
	success = _request("set dns name", dnsName, TRUE, "AOK", NULL, 100);

	return success;
}
/*-----------------------------------------------------------*/

/** This command sets the remote host’s IP address. 0 turns on the DNS.
 *
 * No es tread-safe. Llamar con semáforo obtenido si es el caso.
 *
 * @preconditions El Wifly ha sido inicializado y está en modo comando
 *
 * @param comRemote The remote IP or 0.
 *
 * @return TRUE or FALSE. Retorna TRUE si el ajuste de la ip ha ido bien,
 *                        FALSE en caso contrario.
 */
static bool_t _setHostIp(char* hostIp)
{
	bool_t success = FALSE;

	if(WIFLY_DEBUG) Log_log("--- Solicitud al Wifly: set ip host ---");

	// Enviar la solicitud
	success = _request("set ip host", hostIp, TRUE, "AOK", NULL, 100);

	return success;
}

/*-----------------------------------------------------------*/

/** This command sets the ASCII string that is sent to the remote TCP client
 *  when the TCP port is opened
 *
 * No es tread-safe. Llamar con semáforo obtenido si es el caso.
 *
 * @preconditions El Wifly ha sido inicializado y está en modo comando
 *
 * @param comRemote The string to send.
 *
 * @return TRUE or FALSE. Retorna TRUE si el ajuste de la petición ha ido bien,
 *                        FALSE en caso contrario.
 */
static bool_t _setComRemote(char* comRemote)
{
	bool_t success = FALSE;

	if(WIFLY_DEBUG) Log_log("--- Solicitud al Wifly: set com remote ---");

	// Enviar la solicitud
	success = _request("set com remote", comRemote, TRUE, "AOK", NULL, 100);

	return success;
}

/*-----------------------------------------------------------*/

/** Establece el formato IP del cliente HTML del Wifly.
 *
 * No es tread-safe. Llamar con semáforo obtenido si es el caso.
 *
 * @preconditions El Wifly ha sido inicializado y está en modo comando
 *
 * @param optionFormat   El formato IP (ver doc 'Wifly command reference - DS50002230A' pags.75 a 79)
 *
 * @return TRUE or FALSE. Retorna TRUE si el ajuste del formato ha ido bien,
 *                        FALSE en caso contrario.
 */
static bool_t _setOptionFormat(char* optionFormat)
{
	bool_t success = FALSE;

	if(WIFLY_DEBUG) Log_log("--- Solicitud al Wifly: set option format ---");

	// Enviar la solicitud
	success = _request("set option format", optionFormat, TRUE, "AOK", NULL, 100);

	return success;
}
/*-----------------------------------------------------------*/

/** Sets the Wifly UART mode register.
 *
 * No es tread-safe. Llamar con semáforo obtenido si es el caso.
 *
 * @preconditions El Wifly ha sido inicializado y está en modo comando
 *
 * @param uartMode   Máscara para seleccionar el modo de trabajo de la Uart
 * 				     (ver doc 'Wifly command reference - DS50002230A' pag.32)
 *
 * @return TRUE or FALSE. Retorna TRUE si el ajuste del modo de trabajo ha ido bien,
 *                        FALSE en caso contrario.
 */
static bool_t _setUartMode(char* uartMode)
{
	bool_t success = FALSE;

	if(WIFLY_DEBUG) Log_log("--- Solicitud al Wifly: set uart mode ---");

	// Enviar la solicitud
	success = _request("set uart mode", uartMode, TRUE, "AOK", NULL, 100);

	return success;
}
/*-----------------------------------------------------------*/

/** Sets the IP address of the DNS sever.
 *
 * No es tread-safe. Llamar con semáforo obtenido si es el caso.
 *
 * @preconditions El Wifly ha sido inicializado y está en modo comando
 *
 * @param dnsAddr   the IP address of the DNS sever in the form "xxx.xxx.xxx.xxx".
 * 				     (ver doc 'Wifly command reference - DS50002230A' pag.18)
 *
 * @return TRUE or FALSE. Retorna TRUE si el ajuste de la direccion del servidor DNS
 * 						  ha ido bien, FALSE en caso contrario.
 */
static bool_t _setDnsAddress(char* dnsAddr)
{
	bool_t success = FALSE;

	if(WIFLY_DEBUG) Log_log("--- Solicitud al Wifly: set dns address ---");

	// Enviar la solicitud
	success = _request("set dns address", dnsAddr, TRUE, "AOK", NULL, 100);

	return success;
}
/*-----------------------------------------------------------*/

/** Inicializa el cliente HTML del Wifly.
 *
 * No es tread-safe. Llamar con semáforo obtenido si es el caso.
 *
 * @preconditions El Wifly ha sido inicializado y está en modo comando
 *
 * @param ipProtocol   El protocolo IP            	 (ver doc 'Wifly command reference - DS50002230A' pag.22)
 * @param ipAddress    La dirección IP            	 (ver doc 'Wifly command reference - DS50002230A' pag.20)
 * @param hostName     Nombre del host remoto     	 (ver doc 'Wifly command reference - DS50002230A' pag.18)
 * @param ipRemote 	   El puerto remoto          	 (ver doc 'Wifly command reference - DS50002230A' pag.22)
 * @param ipHost 	   La IP del host             	 (ver doc 'Wifly command reference - DS50002230A' pag.21)
 * @param comRemote	   La string enviada al host 	 (ver doc 'Wifly command reference - DS50002230A' pag.74 a 78)
 * @param optionFormat Opciones de envío HTML    	 (ver doc 'Wifly command reference - DS50002230A' pags.75 a 79)
 * @param uartMode 	   Modo de trabajo de la Uart 	 (ver doc 'Wifly command reference - DS50002230A' pag.32)
 * @param dnsAddr 	   Dirección IP del servidor DNS (ver doc 'Wifly command reference - DS50002230A' pag.18)
 *
 * @return TRUE or FALSE. Retorna TRUE si el proceso de iniciar el cliente HTML ha ido bien,
 *                        FALSE en caso contrario.
 */

static bool_t _initHtmlClient(char* ipProtocol, char* ipAddress, char* hostName, char* ipRemote, char* ipHost,
							  char* comRemote, char* optionFormat, char* uartMode, char* dnsAddr)
{
	bool_t success = FALSE;

	if(WIFLY_DEBUG) Log_log("/*** Solicitud al Wifly: Init HTML Client ***/");

	// IP Ptotocol
	success = _setIpProtocol(ipProtocol);

	// Wifly IP Address
	if(success) success = _setIpAddress(ipAddress);

	// Host Name
	if(success) success = _setDnsName(hostName);

	// IP Remote
	if(success) success = _setIpRemote(ipRemote);

	// Host IP
	if(success) success = _setHostIp(ipHost);

	// Com Remote
	if(success) success = _setComRemote(comRemote);

	// Option format
	if(success) success = _setOptionFormat(optionFormat);

	// Uart Mode
	if(success) success = _setUartMode(uartMode);

	// DNS server
	if(success) success = _setDnsAddress(dnsAddr);

	return success;
}
/*-----------------------------------------------------------*/

/** Conexión del Wifly a una red wifi.
 *
 * No es tread-safe. Llamar con semáforo obtenido si es el caso.
 *
 * @preconditions El Wifly ha sido inicializado.
 *
 * @param netName      El SSID de la red wifi
 * @param netSecurity  El tipo de seguridad de la red wifi
 * @param netPwd	   El password the conexión a la red si no es abierta o NULL en caso contrario
 *
 * @return TRUE or FALSE. Retorna TRUE si consigue conectarse a la red solicitada,
 *                        FALSE en caso contrario.
 */

static bool_t _netConnect(char* netName, securityType netSecurity, char* netPwd)
{
	bool_t success = FALSE;

	if(netName != NULL)
	{
		// Envío del password al wifly en caso necesario
		switch(netSecurity)
		{
			case OPEN:
				success = TRUE;
				break;

			case WEP:
				if(netPwd != NULL)
				{
					if(WIFLY_DEBUG) Log_log("--- Solicitud al Wifly: set wlan key ---");

					success = _request("set wlan key", netPwd, TRUE, "AOK", NULL, 600);
				}
				break;

			case WPA:
				if(netPwd != NULL)
				{
					if(WIFLY_DEBUG) Log_log("--- Solicitud al Wifly: set wlan phrase ---");

					success = _request("set wlan phrase", netPwd, TRUE, "AOK", NULL, 600);
				}
				break;
		}

		if(success == TRUE)	// El Wifly ha recibido correctamente el password, o no era necesario (OPEN)
		{
			if(WIFLY_DEBUG) Log_log("--- Solicitud al Wifly: join ---");

			// Asociar el Wifly a la red wifi
			success = _request("join", netName, TRUE, "Associated!", NULL, 2000);
		}
	}

	return success;
}
/*-----------------------------------------------------------*/

/** Pone al Wifly en modo 'sleep'.
 *
 * No es tread-safe. Llamar con semáforo obtenido si es el caso.
 *
 * @preconditions El Wifly ha sido inicializado y está en modo comando
 *
 * @param None
 *
 * @return None
 */

static void _setModeSleep(void)
{
	if(WIFLY_DEBUG) Log_log("--- Solicitud al Wifly: sleep ---");

	// Pone al WiFly en mode sleep. No retorna nada después de pasar a sleep.
	_request("sleep", NULL, TRUE, "AOK", NULL, 100);

	return;
}
/*-----------------------------------------------------------*/

/** Inicia la entrada para el wake-up del WiFly.
 *
 * No es tread-safe. Llamar con semáforo obtenido si es el caso.
 *
 * @preconditions El Wifly ha sido inicializado y está en modo comando
 *
 * @param mask La máscara que determina la entrada.
 *
 * @return TRUE or FALSE. Retorna TRUE si se ha conseguido iniciar la entrada,
 *                        FALSE en caso contrario.
 */

static bool_t _setSysTrigger(uint8_t mask)
{
	bool_t success = FALSE;

	if(WIFLY_DEBUG) Log_log("--- Solicitud al Wifly: set sys trig ---");

	// Ajustamos en el WiFly la entrada que utilizaremos para el wake-up
	char intToChar[2] = {0x00, 0x00};
	snprintf(intToChar, sizeof(intToChar), "%d", mask);
	success = _request("set sys trig", intToChar, TRUE, "AOK", NULL, 100);

	return success;
}
/*-----------------------------------------------------------*/

/** Pone al Wifly en modo comando.
 *
 * No es tread-safe. Llamar con semáforo obtenido si es el caso.
 *
 * @preconditions El Wifly ha sido inicializado.
 *
 * @param No
 *
 * @return TRUE or FALSE. Retorna TRUE si el Wifly ha entrado en modo comando,
 *                        FALSE en caso contrario.
 */

static bool_t _enterCmdMode(void)
{
	bool_t success = FALSE;

	if(WIFLY_DEBUG) Log_log("--- Solicitud al Wifly: $$$ ---");

	// 250 mS before set command mode (see Wifly doc 'Command reference' pag. 10)
	Utils_delayMs(FALSE, 300);

	// Ponemos el Wifly en modo comando.
	success = _request("$$$", NULL, FALSE, "CMD", NULL, 250);

	// 250 mS after set command mode (see Wifly doc 'Command reference' pag. 10)
	Utils_delayMs(FALSE, 300);

	if(WIFLY_DEBUG && (success == FALSE))
	{
		Log_log("No se ha podido poner al Wifly en modo comando.");
	}

	return success;
}
/*-----------------------------------------------------------*/

/** Inicializa el hardware del microprocesador para trabajajar con el Wifly, el hardware
 *  del Wifly, ajusta el software de ambos elementos y conecta el Wifly a una red wifi:
 *
 *    - Inicializa el puerto UART del LPC1769 utilizado para comunicarse con el Wifly
 * 	    con las ajustes programados en 'whifly.h'
 * 	  - Inicializa el GPIO del LPC1769 para trabajar con el Wifly
 * 	  - Hace un reset hardware del Wifly en la primera inicialización
 * 	  - Ajusta los parámetros para trabajar con el cliente HTML del Wifly
 * 	  - Conecta al Wifly a una red wifi
 *
 * @param  None
 *
 * @return TRUE or FALSE. Retorna TRUE si se ha conseguido conectar a una red wifi,
 *                        FALSE en caso contrario.
 */
static bool_t _initAndConnect(void)
{
	bool_t success = FALSE;

	if(l_Wifly_mutex == NULL) // la primera vez que se llama a la función...
	{
		if(WIFLY_PORT < MAX_UART_PORTS)
		{
			// Inicialización del puerto uart en el LPC1769 para comunicación con el Wifly
			success = PrintF_start(WIFLY_PORT, WIFLY_BAUD_RATE);
			if(success == FALSE)
			{
				if(WIFLY_DEBUG) Log_log("--- WiFly_init(): No se ha podido inicializar el puerto UART del LPC1769 ---");

				return FALSE;
			}

			// Se consolida el puerto para el Wifly.
			l_Wifly_uartPort = WIFLY_PORT;
		}

		// se crea el semáforo para el módulo
		l_Wifly_mutex = xSemaphoreCreateMutex();

		// conversión a ticks del tiempo de espera para obtener el semáforo
		if(l_Wifly_semaphoreTake_Ticks == 0)
			l_Wifly_semaphoreTake_Ticks = (portTickType) WIFLY_SEMAPHORE_TAKE_WAIT_MS / portTICK_RATE_MS;

		// inicialización del hardware del micro para control del WiFly
		_hardInit();

		// Reset hardware del wifly
		_hardReset();

		// Confirmación del reset desde el wifly
		success = _request(NULL, NULL, FALSE, "*READY*", NULL, 100);
		if(success == FALSE)
		{
			if(WIFLY_DEBUG) Log_log("--- WiFly init: No se ha podido hacer el reset hardware del WiFly ---");

			return FALSE;
		}
	}
	else success = TRUE;

	if(success == TRUE)
	{
		// Obtenemos el semáforo para llamar a funciones que no son thread-safe
		if(xSemaphoreTake(l_Wifly_mutex, l_Wifly_semaphoreTake_Ticks) == pdTRUE)
		{
			// Ponemos al Wifly en modo comando.
			success = _enterCmdMode();
			if(WIFLY_DEBUG && (success == FALSE))
			{
				Log_log("--- WiFly init: No se ha podido poner al Wifly en modo comando ---");
			}

			if(success)
			{
				// HTML Client Settings:
				// IP Protocol = 18 -> Enable the HTML client. Enable HTTP client = 0x10 and TCP mode = 0x02
				// IP Address = WIFLY_IP_ADDR -> Sets the WiFly module’s IP address
				// DNS Name = REMOTE_HOST_NAME -> Sets the web server name
				// IP Remote = 80 -> Web server port 80 (standard)
				// IP Host = 0 -> Turn on DNS
				// Com remote = GET$/uoc/pfc/rest/?action= -> The constant host string
				// Option format = 1 -> Automatically send HTML data header
				// Uart Mode = 2 -> Uart sends direct to the host
				// DNS server IP = 192.168.1.1 -> Tied to the local router
				if(WIFLY_DEBUG) Log_log("*** Inicializando el cliente HTML ***\r\n");
				success = _initHtmlClient("18", WIFLY_IP_ADDR, REMOTE_HOST_NAME, "80", "0",
										  "GET$/uoc/pfc/rest/?action=", "1", "2", "192.168.1.1");
				if(WIFLY_DEBUG) Log_log("Inicializado el cliente HTML con éxito: %d\r\n\r\n", success);

				// Conectar a la red wifi
				if(success == TRUE)
				{
					if(WIFLY_DEBUG) Log_log("*** Conectando el Wifly a la red wifi ***\r\n");
					success = _netConnect(ACCESS_POINT_NAME, ACCESS_POINT_SEC_TYPE, ACCESS_POINT_PASS);
					if(WIFLY_DEBUG) Log_log("Conectado con éxito: %d\r\n\r\n", success);
				}
			}

			// devolvemos el semáforo
			xSemaphoreGive(l_Wifly_mutex);
		}
	}

	return success;
}

/*-----------------------------------------------------------*/

// Funciones globales

/*-----------------------------------------------------------*/

bool_t Wifly_init(void)
{
	// Inicialización y conexión inicial a red wifi del Wifly.
	return _initAndConnect();
}
/*-----------------------------------------------------------*/

bool_t Wifly_wakeUp(void)
{
	bool_t success = FALSE;

	// Inicialización del hardware del micro para control del WiFly
	_hardInit();

	// Pulso para despertar al WiFly
	GPIO_setValue(WIFLY_WAKEUP_PIN, TRUE);
	Utils_delayMs(FALSE, 1);
	GPIO_setValue(WIFLY_WAKEUP_PIN, FALSE);

	// Inicialización y conexión a la red wifi del Wifly después
	// del wakeup.
	success = _initAndConnect();

	return success;
}
/*-----------------------------------------------------------*/

bool_t Wifly_enterSleep(void)
{
	bool_t success = FALSE;

	if((l_Wifly_uartPort < MAX_UART_PORTS) && (l_Wifly_mutex != NULL))
	{
		// Obtenemos el semáforo para llamar a funciones que no son thread-safe
		if(xSemaphoreTake(l_Wifly_mutex, l_Wifly_semaphoreTake_Ticks) == pdTRUE)
		{
			// entrar en modo comando
			_enterCmdMode();

			if(WIFLY_DEBUG) Log_log("--- Solicitud al Wifly: sleep ---");

			// Programamos la entrada CTS del WiFly como entrada de wake-up
			success = _setSysTrigger(0x02);

			// Ponemos al WiFly en modo sleep
			if(success == TRUE)
			{
				_setModeSleep();
			}
			else
			{
				if(WIFLY_DEBUG) Log_log("--- Wifly_setSleep(): No se ha podido programar la entrada CTS del WiFly como entrada de wake-up ---");
			}

			// devolvemos el semáforo
			xSemaphoreGive(l_Wifly_mutex);
		}
	}

	return success;
}
/*-----------------------------------------------------------*/

bool_t Wifly_sendHttp(char* request, char* remoteAnswer)
{
	bool_t success = FALSE;
	uint16_t i, initConnTimeout = 1000; // tiempo de espera inicial para conexión con el host

	if((l_Wifly_uartPort < MAX_UART_PORTS) && (l_Wifly_mutex != NULL))
	{
		if(xSemaphoreTake(l_Wifly_mutex, l_Wifly_semaphoreTake_Ticks) == pdTRUE)
		{
			if(WIFLY_DEBUG) Log_log("--- Solicitud al Wifly: open ---\r\n");

			if(l_openConnectionToHost == TRUE)	// Conexión inicial con el host
			{
				// Cuatro intentos de conexión con el host.
				// En cada nuevo intento doblamos el tiempo de espera.
				for(i=0; i<4; i++)
				{
					// espera para el nuevo intento de conexión con el host
					Utils_delayMs(FALSE, initConnTimeout);

					if(WIFLY_DEBUG) Log_log("Intento de conexión con el host: %d\r\n", i+1);

					memset(l_wiflyModulRxStr, 0x00, sizeof(l_wiflyModulRxStr));
					success = _request("open", NULL, TRUE, "*OPEN*", l_wiflyModulRxStr, 100);

					if(WIFLY_DEBUG) Log_log("Respuesta recibida desde el host en el intento de apertura de la conexión: %s\r\n",
							                l_wiflyModulRxStr);

					if(success == TRUE) break;
					else initConnTimeout *= 2;
				}

				if(WIFLY_DEBUG)
				{
					if(success == TRUE) Log_log("--- Conectado con el host ---");
					else Log_log("¡¡¡ Sin conexión con el host !!!");
				}

				// no volver a intentar conectar con el host o bién porque ya estamos conectados
				// o bién porque han fallado todos los intentos de conectar
				l_openConnectionToHost = FALSE;
			}
			else success = TRUE;  // ya se habia abierto previamente la conexión con el host

			if(success == TRUE)	// si está abierta la conexión con el host...
			{
				// espera para la nueva comunicación con el host
				Utils_delayMs(FALSE, 100);

				// enviar la petición y obtener la respuesta
				memset(l_wiflyModulRxStr, 0x00, sizeof(l_wiflyModulRxStr));
				success = _request(request, NULL, FALSE, "dir_ip=", l_wiflyModulRxStr, 200);

				// Si ha ido bién el proceso de petición-respuesta con el host,
				// copiar la respuesta en el parámetro de salida
				if(success == TRUE)
				{
					strcpy(remoteAnswer, l_wiflyModulRxStr); // ponemos la respuesta html completa en el parámetro de salida

					if(WIFLY_DEBUG) Log_log("--- Completada la comunicación con el host ---\r\n\r\n%s", l_wiflyModulRxStr);
				}
				else // error petición-respuesta con el host
				{
					memset(remoteAnswer, 0x00, sizeof(remoteAnswer)); // reset el parámetro de salida

					if(WIFLY_DEBUG) Log_log("¡¡¡ Error en la comunicación con el host !!!\r\n\r\n%s", l_wiflyModulRxStr);
				}

			}

			xSemaphoreGive(l_Wifly_mutex);
		}
	}

	return success;
}
/*-----------------------------------------------------------*/

/*************************************************************************************
 * Las funciones siguientes están implementadas aunque no se utilizan
 * en esta aplicación.
 *************************************************************************************/

bool_t Wifly_scan(char* outWifly_Scan)
{
	bool_t success = FALSE;

	if((l_Wifly_uartPort < MAX_UART_PORTS) && (l_Wifly_mutex != NULL))
	{
		if(xSemaphoreTake(l_Wifly_mutex, l_Wifly_semaphoreTake_Ticks) == pdTRUE)
		{
			if(WIFLY_DEBUG) Log_log("--- Solicitud al Wifly: scan ---");

			// Hacer un scan para descubrir las redes wifi accesibles
			// 200 mS (default) x 13 channels = 2600 mS min.
			success = _request("scan", NULL, TRUE, "END:", outWifly_Scan, 3000);

			xSemaphoreGive(l_Wifly_mutex);
		}
	}

	return success;
}
/*-----------------------------------------------------------*/

bool_t Wifly_ping(char* pingDestination_Domain, uint32_t pingRetry_Times)
{
	bool_t success = FALSE;

	if((l_Wifly_uartPort < MAX_UART_PORTS) && (l_Wifly_mutex != NULL))
	{
		if(xSemaphoreTake(l_Wifly_mutex, l_Wifly_semaphoreTake_Ticks) == pdTRUE)
		{
			if(WIFLY_DEBUG) Log_log("--- Solicitud al Wifly: ping ---");

			if(pingRetry_Times < 1) pingRetry_Times = 1;
			else if(pingRetry_Times > 8) pingRetry_Times = 8;

			char intToChar[2] = "\0\0";
			intToChar[0] = pingRetry_Times + 0x030;

			memset(l_wiflyModulTxStr, 0x00, sizeof(l_wiflyModulTxStr));

			strcpy(l_wiflyModulTxStr, "ping ");
			strcat(l_wiflyModulTxStr, pingDestination_Domain);

			// Solicitud de envío de ping
			uint32_t responseTimeout = 1000 * pingRetry_Times;
			success = _request(l_wiflyModulTxStr, intToChar, TRUE, "PING reply from", NULL, responseTimeout);

			xSemaphoreGive(l_Wifly_mutex);
		}
	}

	return success;
}
/*-----------------------------------------------------------*/

bool_t Wifly_getIp(char* outWifly_Ip)
{
	bool_t success = FALSE;

	if((l_Wifly_uartPort < MAX_UART_PORTS) && (l_Wifly_mutex != NULL))
	{
		if(xSemaphoreTake(l_Wifly_mutex, l_Wifly_semaphoreTake_Ticks) == pdTRUE)
		{
			if(WIFLY_DEBUG) Log_log("--- Solicitud al Wifly: get ip ---");

			// Solicitud de la IP al wifly
			memset(l_wiflyModulRxStr, 0x00, sizeof(l_wiflyModulRxStr));
			success = _request("get ip", NULL, TRUE, "IP=", l_wiflyModulRxStr, 100);

			if(success == TRUE)
			{
				// Parse IP
				memset(l_wiflyModulTxStr, 0x00, sizeof(l_wiflyModulTxStr)); // considerada IPv6
				success = _extractFieldValue(l_wiflyModulRxStr, "\r\n", "IP=", 3, FALSE, l_wiflyModulTxStr);

				if(success) strcpy(outWifly_Ip, l_wiflyModulTxStr);
			}

			xSemaphoreGive(l_Wifly_mutex);
		}
	}

	return success;
}
/*-----------------------------------------------------------*/

uint32_t Wifly_getChannel(void) // Channel: 1-13
{
	uint32_t ret = 0;

	if((l_Wifly_uartPort < MAX_UART_PORTS) && (l_Wifly_mutex != NULL))
	{
		if(xSemaphoreTake(l_Wifly_mutex, l_Wifly_semaphoreTake_Ticks) == pdTRUE)
		{
			if(WIFLY_DEBUG) Log_log("--- Solicitud al Wifly: show connection (channel) ---");

			// Solicitud de los datos de conexión al wifly (ver "Wifly Command Reference" doc. pag.39, apartado 2.5.2)
			// show connection retorna '\n8<XYZ>' (8XYZ hexadecimal)
			//
			// 8: Fixed. value = 8.
			// X: channel number
			// Y: Y.3-DNS found. 1-DNS resolved.
			//    Y.2-DNS server. 1-Server is contacted.
			//    Y.1-Authentication. 1-Ok.
			//    Y.0-Association. 1-Ok.
			// Z: TCP status: 0-idle, 1-connected, 3-No IP, 4-connecting
			memset(l_wiflyModulRxStr, 0x00, sizeof(l_wiflyModulRxStr));
			bool_t success = _request("show connection", NULL, TRUE, "\r\n", l_wiflyModulRxStr, 100);

			if((success == TRUE) && (sizeof(l_wiflyModulRxStr) > 4) && (l_wiflyModulRxStr[1] == '8')) // Fixed value = 8
			{
				// Parse channel
				char channel[2]; channel[0] = l_wiflyModulRxStr[2]; channel[1] = 0x00;

				ret = (uint32_t) strtol(channel, NULL, 16); // strtol() returns 0 if not valid conversion can be performed
			}

			xSemaphoreGive(l_Wifly_mutex);
		}
	}

	return ret;
}
/*-----------------------------------------------------------*/

uint32_t Wifly_getRssi(void) // -dBm
{
	uint32_t ret = 0;

	if((l_Wifly_uartPort < MAX_UART_PORTS) && (l_Wifly_mutex != NULL))
	{
		if(xSemaphoreTake(l_Wifly_mutex, l_Wifly_semaphoreTake_Ticks) == pdTRUE)
		{
			if(WIFLY_DEBUG) Log_log("--- Solicitud al Wifly: show rssi ---");

			// Solicitud de la RSSI
			memset(l_wiflyModulRxStr, 0x00, sizeof(l_wiflyModulRxStr));
			bool_t success = _request("show rssi", NULL, TRUE, "RSSI=", l_wiflyModulRxStr, 10);

			if(success == TRUE)
			{
				// Parse RSSI
				memset(l_wiflyModulTxStr, 0x00, sizeof(l_wiflyModulTxStr));
				success = _extractFieldValue(l_wiflyModulRxStr, "\r\n", "RSSI=", 5, FALSE, l_wiflyModulTxStr);

				if((success == TRUE) && (sizeof(l_wiflyModulTxStr) > 3) && (l_wiflyModulTxStr[0] == '('))
				{
					char rssi[4] = {0x00, 0x00, 0x00, 0x00};
					uint32_t i;
					for(i=0; i<3; i++) rssi[i] = l_wiflyModulTxStr[i+1];

					ret = atoi(rssi); // atoi() returns 0 if not valid conversion can be performed
				}
			}

			xSemaphoreGive(l_Wifly_mutex);
		}
	}

	return ret;
}
/*-----------------------------------------------------------*/

uint32_t Wifly_getSecondsSinceBoot(void)
{
	uint32_t ret = 0;

	if((l_Wifly_uartPort < MAX_UART_PORTS) && (l_Wifly_mutex != NULL))
	{
		if(xSemaphoreTake(l_Wifly_mutex, l_Wifly_semaphoreTake_Ticks) == pdTRUE)
		{
			if(WIFLY_DEBUG) Log_log("--- Solicitud al Wifly: show time ---");

			// Solicitud de la IP actual al wifly
			memset(l_wiflyModulRxStr, 0x00, sizeof(l_wiflyModulRxStr));
			bool_t success = _request("show time", NULL, TRUE, "UpTime=", l_wiflyModulRxStr, 10);

			if(success == TRUE)
			{
				// int -> 4 bytes. +int -> ((2^32) / 2) -1) = 10 cifras. outField_Value = 10 + '\0' -> 11 chars.
				// max. time = ((2^32) / 2) -1) = ( > 68 anys ).
				memset(l_wiflyModulTxStr, 0x00, sizeof(l_wiflyModulTxStr));
				success = _extractFieldValue(l_wiflyModulRxStr, "\r\n", "UpTime=", 7, FALSE, l_wiflyModulTxStr);

				// atoi() returns 0 if not valid conversion can be performed
				if(success == TRUE) ret = atoi(l_wiflyModulTxStr);
			}

			xSemaphoreGive(l_Wifly_mutex);
		}
	}

	return ret;
}
/*-----------------------------------------------------------*/

uint32_t Wifly_getTxPower(void) // 1..12 dBm
{
	uint32_t ret = 0;

	if((l_Wifly_uartPort < MAX_UART_PORTS) && (l_Wifly_mutex != NULL))
	{
		if(xSemaphoreTake(l_Wifly_mutex, l_Wifly_semaphoreTake_Ticks) == pdTRUE)
		{
			if(WIFLY_DEBUG) Log_log("--- Solicitud al Wifly: get wlan (TX power) ---");

			// Solicitud de varios parámetros (entre ellos la potencia de transmisión definida actualmente)
			memset(l_wiflyModulRxStr, 0x00, sizeof(l_wiflyModulRxStr));
			bool_t success = _request("get wlan", NULL, TRUE, "TxPower=", l_wiflyModulRxStr, 1000);

			if(success == TRUE)
			{
				// Parse Tx power
				memset(l_wiflyModulTxStr, 0x00, sizeof(l_wiflyModulTxStr));
				success = _extractFieldValue(l_wiflyModulRxStr, "\r\n", "TxPower=", 8, FALSE, l_wiflyModulTxStr);

				// atoi() returns 0 if not valid conversion can be performed
				if(success == TRUE) ret = (uint32_t) atoi(l_wiflyModulTxStr);
			}

			xSemaphoreGive(l_Wifly_mutex);
		}
	}

	return ret;
}
/*-----------------------------------------------------------*/

bool_t Wifly_setTxPower(uint32_t txPower) // 1..12 dBm  0 = max power = 12
{
	bool_t success = FALSE;

	if((l_Wifly_uartPort < MAX_UART_PORTS) && (l_Wifly_mutex != NULL))
	{
		if(xSemaphoreTake(l_Wifly_mutex, l_Wifly_semaphoreTake_Ticks) == pdTRUE)
		{
			if(WIFLY_DEBUG) Log_log("--- Solicitud al Wifly: set wlan tx ---");

			if(txPower < 0) txPower = 1; 		// 1: the minimun power
			else if(txPower > 12) txPower = 12;	// 12: the maximum power

			// txPower to string
			char intToChar[3] = {0x00, 0x00, 0x00};
			snprintf(intToChar, sizeof(intToChar), "%d", txPower);

			// Enviar nueva potencia de transmisión
			success = _request("set wlan tx", intToChar, TRUE, "AOK", NULL, 100);

			xSemaphoreGive(l_Wifly_mutex);
		}
	}

	return success;
}
/*-----------------------------------------------------------*/
