 /* author: Miguel A. Fdez.
  *
  * revision 1.0 date 5/4/2015
  */

#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "LPC17xx.h"
#include "type.h"
#include "led.h"
#include "log.h"
#include "printF.h"
#include "utils.h"
#include "rtc.h"
#include "adc.h"
#include "nivel.h"
#include "temperatura.h"
#include "verticalidad.h"
#include "control.h"

#define MAIN_DEBUG_CONFIG	0 	// debug config mode ? Se muestran los datos de inicialización y errores
#define MAIN_DEBUG_DATA		1	// debug data mode ? se muestran los datos netos de trabajo

/* Prioridad para las tasks productoras. */
#define producerTASK_PRIORITY	(tskIDLE_PRIORITY + 2)

/* En este caso, la prioridad para la task consumidora está por debajo
 * de la prioridad para las tasks productoras.
 * De esta manera, la task consumidora recibirá los datos cuando
 * todas las tasks productoras los hayan enviado a la cola.
 *
 */
#define consumerTASK_PRIORITY	(tskIDLE_PRIORITY + 1)

/*-----------------------------------------------------------*/

/* Tasks a crear. */
static void vProducer_Task_Temperatura(void *pvParameters);
static void vProducer_Task_Verticalidad(void *pvParameters);
static void vProducer_Task_Nivel(void *pvParameters);
static void vConsumer_Task(void *pvParameters);

/*-----------------------------------------------------------*/

// Variable para almacenar la cola que utilizan las tasks
// productoras para pasar los datos a la task consumidora()
volatile xQueueHandle l_mainQueue = NULL;

/*-----------------------------------------------------------*/

/* Función para inicialización del hardware. */
static bool_t _hardwareInit(void);

/*-----------------------------------------------------------*/

int main(void)
{
	// Inicialización del harware
	bool_t success = _hardwareInit();

	// Se ha conseguido inicializar el hardware ?
	if(success == TRUE) // Si
	{
		// Se crea la cola en la que se pasan los datos entre las tasks
		l_mainQueue = xQueueCreate(xQUEUE_LENGTH, sizeof(ACTUAL_DATA));
	}

	// Se ha creado la cola para el paso de datos entre las tasks productoras y la task consumidora ?
	if(l_mainQueue != NULL) // Si
	{
		/* Creación de las tasks productoras que escribirán en la cola */
		xTaskCreate(vProducer_Task_Temperatura, (signed char *) "ProdTemp", 512, NULL, producerTASK_PRIORITY, NULL);
		xTaskCreate(vProducer_Task_Verticalidad, (signed char *) "ProdVert", 512, NULL, producerTASK_PRIORITY, NULL);
		xTaskCreate(vProducer_Task_Nivel, (signed char *) "ProdNive", 512, NULL, producerTASK_PRIORITY, NULL);

		/* Creación de la task consumidora que leerá de la cola */
		xTaskCreate(vConsumer_Task, (signed char *) "Consumer", 1024, NULL, consumerTASK_PRIORITY, NULL);

		/* Ejecución de las tasks */
		vTaskStartScheduler();
	}
	else
	{
		// No se ha podido crear la cola.
		if(MAIN_DEBUG_CONFIG) Log_log("main(): No se ha podido crear la cola.\r\n\r\n");
	}

	/* Si va bien, nunca llegaremos a este punto ya que el planificador estará
	 * ejecutando las tasks. Si el programa alcanza este punto, es porque no hay
	 * suficiente memoria en el heap para crear un recurso o porque
	 * no se ha conseguido iniciar el hardware (_hardwareInit())
	 */
	for( ;; );

	return 0;
}
/*-----------------------------------------------------------*/

static bool_t _hardwareInit(void)
{
	bool_t success = FALSE, rtcInitialized = FALSE;

	// Lo primero es desconectar la alimentación del sensor de nivel.
	// (Leer motivo en la implementación de la función 'Nivel_powerOff()')
	Nivel_powerOff();

	/* Log port init */
	success = Log_start(UART_PORT_3, 9600);

	// Las dos impresiones siguientes solo se harán si se ha conseguido inicializar el Log.
	if(MAIN_DEBUG_CONFIG)
	{
		Log_log("\r\n\r\n*** HardwareInit(): Inicializando el Log... ***\r\n");
		Log_log("HardwareInit(): Inicializado con éxito: %d\r\n\r\n", success);
	}

	if(success == FALSE)
	{
		// Si no se ha podido inicializar el Log, tampoco se puede imprimir nada.
		// Retorna a main()
		return FALSE;
	}

	// Inicialización del RTC (reloj de tiempo real)
	// Como trabajamos con el modo deep-powerdown, en el que solo queda alimentado el RTC,
	// cada vez que se hace un wake-up, se vuelve a hacer la inicialización del hardware excepto del RTC.
	// Solamente se iniciará el RTC después de un corte de alimentación.

	// El bit DPDFLAG (bit 11) del registro 'PCON', indica si el RTC ya ha sido inicializado (1) o no (0).
	// (Si ha perdido (0) o no (1) la alimentación).
	rtcInitialized = (LPC_SC->PCON & 0x00000800) >> 11;

	if(rtcInitialized) // Si el RTC ya ha sido inicializado
	{
		// no es primer ciclo después de alimentación
		Control_setIsFirstCicle(FALSE);

		// Iniciamos explícitamente el semáforo del módulo
		Rtc_semaphoreInit();

		LPC_SC->PCON |= 0x00000800; // Reseteamos el bit DPDFLAG (bit 11). (¡ojo!, se resetea con un 1 (ver manual pag. 62).
									// Cada vez que entremos de nuevo en modo deep power-down, se pondrá de nuevo a 1.

		// Habilitamos la interrupcion del RTC en el vector de interrupciones
		// (se pierde la habilitación cuando el uP pasa al estado deep power-down)
		NVIC_EnableIRQ((IRQn_Type) RTC_IRQn);

		if(MAIN_DEBUG_CONFIG) Log_log("HardwareInit(): El RTC ya se había inicializado.\r\n\r\n");
	}
	else // Si el RTC NO ha sido inicializado, le inicializamos
	{
		// Es primer ciclo después de alimentación
		Control_setIsFirstCicle(TRUE);

		// Inicialización del RTC (reloj de tiempo real)
		// Implícitamente inicia el semáforo del módulo
		Rtc_init(LPC_RTC);

		// Habilitamos la alarma de segundos, minutos y horas
		Rtc_alarmIntConfig(LPC_RTC, (RTC_AMR_CIIR_IMSEC | RTC_AMR_CIIR_IMMIN | RTC_AMR_CIIR_IMHOUR), ENABLE);

		// Limpiamos las interrupciones pendientes
		Rtc_clearIntPending(LPC_RTC, RTC_INT_COUNTER_INCREASE | RTC_INT_ALARM);

		// Habilitamos la interrupcion del RTC en el vector de interrupciones
		NVIC_EnableIRQ((IRQn_Type) RTC_IRQn);

		// Habilitamos el RTC
		Rtc_enable(LPC_RTC, ENABLE);

		// Reseteamos la cuenta de intérvalos base acumulados para petición al host
		// contabilizando el ciclo actual (1)
		ROM_SET_CICLOS_DATOS(1);
		// Primer ciclo -> Pedir datos de ajuste al host
		Control_setPedirDatosAlHost(TRUE);

		// Reseteamos la cantidad de intervalos base acumulados para la medida del nivel
		// contabilizando el ciclo actual (1)
		ROM_SET_CICLOS_NIVEL(1);
		// Primer ciclo -> Medir nivel
		Control_setMedirNivelEnEsteCiclo(TRUE);

		if(MAIN_DEBUG_CONFIG) Log_log("HardwareInit(): Se ha inicializado el RTC.\r\n\r\n");
	}

	// Si es el primer ciclo, ya se hace la petición de datos de ajuste y se mide el nivel,
	// en caso contrario miramos si hay que hacerlo
	if(Control_getIsFirstCicle() == FALSE)
	{
		// leemos la cantidad de intérvalos base acumulados para peticion al host,
		// para saber si hay que hacer una petición de datos de ajuste.
		uint32_t intervalos_base_acumulados_peticion_host = ROM_GET_CICLOS_DATOS();
		// añadimos el ciclo actual
		intervalos_base_acumulados_peticion_host ++;

		// obtenemos el ajuste de periodo de petición al host
		uint32_t periodo_peticion_host = ROM_GET_PER_DATOS();

		// ¿ Hay que hacer una petición al host en este ciclo ?
		if(intervalos_base_acumulados_peticion_host >= periodo_peticion_host)
		{
			// Si. Marcamos el flag.
			Control_setPedirDatosAlHost(TRUE);
		}
		else // No hay que hacer una petición de datos de ajuste en este ciclo
		{
			// memorizamos la nueva cuenta de intérvalos base acumulados para petición al host
			ROM_SET_CICLOS_DATOS(intervalos_base_acumulados_peticion_host);
		}

		// leemos la cantidad de intérvalos base acumulados de nivel, para saber si hay que medir el nivel.
		uint32_t intervalos_base_acumulados_nivel = ROM_GET_CICLOS_NIVEL();
		// añadimos el ciclo actual
		intervalos_base_acumulados_nivel ++;

		// obtenemos el ajuste de periodo de medida de nivel
		uint32_t periodo_medida_nivel = ROM_GET_PER_NIVEL()	;

		// ¿ Hay que medir el nivel en este ciclo ?
		if(intervalos_base_acumulados_nivel >= periodo_medida_nivel)
		{
			// Hay que medir el nivel en este ciclo
			Control_setMedirNivelEnEsteCiclo(TRUE);
		}
		else // NO hay que medir el nivel en este ciclo
		{
			// memorizamos la nueva cuenta de intérvalos base acumulados para medida de nivel
			ROM_SET_CICLOS_NIVEL(intervalos_base_acumulados_nivel);
		}
	}

	// Si hay que medir el nivel, inicializamos el sensor.
	if(Control_getMedirNivelEnEsteCiclo())
	{
		if(MAIN_DEBUG_CONFIG) Log_log("HardwareInit(): *** Inicializando el sensor de nivel... ***\r\n");
		success = Nivel_init(UART_PORT_1, 9600);
		if(MAIN_DEBUG_CONFIG) Log_log("HardwareInit(): Inicializado con éxito: %d\r\n\r\n", success);
		if(success == FALSE)
		{
			// No se ha podido inicializar el sensor de distancia.
			if(MAIN_DEBUG_CONFIG) Log_log("HardwareInit(): No se ha podido inicializar.\r\n<<< Fin del programa. >>>\r\n\r\n");
			return FALSE;
		}
	}

	/*
	 * La verticalidad y la temperatura las medimos en cada ciclo:
	 */

	/* Inicialización del acelerómetro */
	if(MAIN_DEBUG_CONFIG) Log_log("HardwareInit(): *** Inicializando el acelerómetro... ***\r\n");
	Verticalidad_init();

	/* Inicialización del sensor de temperatura */
	if(MAIN_DEBUG_CONFIG) Log_log("*** Inicializando el sensor de temperatura... ***\r\n");
	Temperatura_init();

	// Inicialización del conversor A/D
	if(MAIN_DEBUG_CONFIG) Log_log("HardwareInit(): *** Inicializando el conversor A/D... ***\r\n");
	Adc_init();

	// Led de la placa del LPC1769, válido solamente para pruebas.
	 Led_ledInit(LED_2);
	 Led_ledOn(LED_2);
//	 Led_ledOff(LED_2);
//	 Led_ledToggle(LED_2);

	return success;

}
/*-----------------------------------------------------------*/

static void vProducer_Task_Nivel(void *pvParameters)
{
	ACTUAL_DATA distancia;

	/* Como la mayoria de las tasks, esta task se ejecuta dentro de un bucle infinito. */
	for( ;; )
	{
		if(Control_getMedirNivelEnEsteCiclo()) // SI hay que medir el nivel en este ciclo
		{
			// Obtenemos los datos de distancia libre de llenado
			Nivel_get(&distancia);

			if(MAIN_DEBUG_DATA) Log_log("*** El nivel se ha medido en este ciclo. ***\r\n\r\n");
		}
		else // NO hay que medir el nivel en este ciclo
		{
			// iniciamos a valores por defecto
			distancia.id = ID_NIVEL; distancia.value = 0; distancia.alarm = FALSE; distancia.alarmFlange = FALSE;

			if(MAIN_DEBUG_DATA) Log_log("*** El nivel no se ha medido en este ciclo. ***\r\n\r\n");
		}

		// Siempre enviamos los datos a la cola, aunque sean los valores por defecto
		portBASE_TYPE xStatus = xQueueSendToBack(l_mainQueue, &distancia, 0);

		if(MAIN_DEBUG_DATA && (xStatus != pdPASS))
		{
			Log_log("*** NIVEL TASK PRODUCTOR ***\r\nNo se pueden enviar los datos a la cola.\r\n\r\n");
		}

		// una vez obtenido el nivel (si corresponde en este ciclo), se suspende la task.
		// Ya no hará falta porque pasaremos a modo deep power-down
		vTaskSuspend(NULL);
	}
}

/*-----------------------------------------------------------*/

static void vProducer_Task_Temperatura(void *pvParameters)
{
	ACTUAL_DATA temperatura;

	/* Como la mayoria de las tasks, esta task se ejecuta dentro de un bucle infinito. */
	for( ;; )
	{
		// Obtenemos los datos de temperatura
		Temperatura_get(&temperatura);

		// Enviamos los datos a la cola
		portBASE_TYPE xStatus = xQueueSendToBack(l_mainQueue, &temperatura, 0);

		if(MAIN_DEBUG_DATA && (xStatus != pdPASS))
		{
			Log_log("*** TEMPERATURA TASK PRODUCTOR ***\r\nNo se pueden enviar los datos a la cola.\r\n\r\n");
		}

		// una vez obtenida la temperatura, se suspende la task. Ya no hará falta
		// porque pasaremos a modo deep power-down
		vTaskSuspend(NULL);
	}
}
/*-----------------------------------------------------------*/

static void vProducer_Task_Verticalidad(void *pvParameters)
{
	ACTUAL_DATA verticalidad;

	/* Como la mayoria de las tasks, esta task se ejecuta dentro de un bucle infinito. */
	for( ;; )
	{
		// Obtenemos los datos de verticalidad
		Verticalidad_get(&verticalidad);

		// Enviamos los datos a la cola
		portBASE_TYPE xStatus = xQueueSendToBack(l_mainQueue, &verticalidad, 0);

		if(MAIN_DEBUG_DATA && (xStatus != pdPASS))
		{
			Log_log("*** VERTICALIDAD TASK PRODUCTOR ***\r\nNo se pueden enviar los datos a la cola.\r\n\r\n");
		}

		// una vez obtenida la verticalidad, se suspende la task. Ya no hará falta
		// porque pasaremos a modo deep power-down
		vTaskSuspend(NULL);
	}
}
/*-----------------------------------------------------------*/

static void vConsumer_Task(void *pvParameters)
{
	portBASE_TYPE xStatus;
	ACTUAL_DATA received_data, nivel_data, temper_data, vertic_data;

	// ponemos a 0 los IDs para su identificación posterior
	temper_data.id = vertic_data.id = nivel_data.id = ID_NO;

	/* Como la mayoria de las tasks, esta task se ejecuta dentro de un bucle infinito. */
	for( ;; )
	{
		// 100 mS son suficientes para recibir las tres estructuras de datos
		xStatus = xQueueReceive(l_mainQueue, &received_data, (portTickType) (100 / portTICK_RATE_MS));

		if(xStatus == pdPASS) // recibimos los datos de los sensores
		{
			if(received_data.id == ID_NIVEL)
			{
				nivel_data = received_data;

				if(MAIN_DEBUG_DATA && Control_getMedirNivelEnEsteCiclo())
					Log_log("vConsumer_Task(): Datos de nivel recibidos en la cola:\r\nId: %d\r\nValor (x1): %d\r\nAlarma: %d\r\nCambio estado alarma desde la lectura anterior: %d\r\n\r\n",
							  nivel_data.id, nivel_data.value, nivel_data.alarm, nivel_data.alarmFlange);
			}
			else if(received_data.id == ID_TEMPERATURA)
			{
				temper_data = received_data;

				if(MAIN_DEBUG_DATA)
					Log_log("vConsumer_Task(): Datos de temperatura recibidos en la cola:\r\nId: %d\r\nValor (x10): %d\r\nAlarma: %d\r\nCambio estado alarma desde la lectura anterior: %d\r\n\r\n",
							  temper_data.id, temper_data.value, temper_data.alarm, temper_data.alarmFlange);
			}
			else if(received_data.id == ID_VERTICALIDAD)
			{
				vertic_data = received_data;

				if(MAIN_DEBUG_DATA)
					Log_log("vConsumer_Task(): Datos de verticalidad recibidos en la cola:\r\nId: %d\r\nValor (x10): %d\r\nAlarma: %d\r\nCambio estado alarma desde la lectura anterior: %d\r\n\r\n",
							  vertic_data.id, vertic_data.value, vertic_data.alarm, vertic_data.alarmFlange);
			}
		}
		// hemos recibido todos los datos de los sensores ?
		else if((nivel_data.id == ID_NIVEL) && (temper_data.id == ID_TEMPERATURA) &&
				(vertic_data.id == ID_VERTICALIDAD))
		{
			// Comenzamos la gestión de la lógica de los datos,
			// incluida la comunicación con el servidor y el paso pasterior a modo sleep
			Control_manager(&nivel_data, &temper_data, &vertic_data);
		}

		// No hace falta suspender o eliminar esta task porque a partir de la
		// llamada a Control_manager(), el sistema pasará a modo deep power-down.
	}
}
/*-----------------------------------------------------------*/

/** Añade funcionalidad especifica de la aplicación en la 'idle task'
 *  (la task que crea automáticamente el scheduler).
 *
 * @precondition no
 *
 * @param void
 *
 * @return void
 */
void vApplicationIdleHook( void )
{
	// This application does not use the idle hook to perform any processing.
}
/*-----------------------------------------------------------*/

/** Función llamada por el kernel cada tick interrupt
 *
 * @precondition no
 *
 * @param void
 *
 * @return void
 */
void vApplicationTickHook( void )
{
	// This application does not use the tick hook to perform any processing.
}
/*-----------------------------------------------------------*/

/** Función llamada por el kernel cuando falla la creación de una task, queue o
 *  semaphore debido a que no queda suficiente memoria RAM en el heap o montón.
 *
 * @precondition no
 *
 * @param void
 *
 * @return void
 */
void vApplicationMallocFailedHook( void )
{
	// This function will only be called if an API call to create a task, queue
	// or semaphore fails because there is too little heap RAM remaining.
	for( ;; );
}
/*-----------------------------------------------------------*/

/** Función llamada por el kernel cuando detecta un desbordamiento del stack
 *  o pila de cualquier task
 *
 * @precondition no
 *
 * @param *pxTask		Task Handle from the task with overflows its stack.
 * @param *pcTaskName   Task Name from the task with overflows its stack.
 *
 * @return void
 */
void vApplicationStackOverflowHook( xTaskHandle *pxTask, signed char *pcTaskName )
{
	// This function will only be called if a task overflows its stack.  Note
	// that stack overflow checking does slow down the context switch
	// implementation.
	for( ;; );
}
/*-----------------------------------------------------------*/
