/*
 * control.c
 *
 *  Created on: 26/11/2015
 *      Author: Miguel A. Fdez.
 */

#include "LPC17xx.h"
#include "type.h"
#include "AppConfig.h"
#include "utils.h"
#include "log.h"
#include "wifly.h"
#include "rtc.h"
#include "host.h"
#include "gpio.h"
#include "led.h"
#include "power.h"
#include "control.h"

#define CONTROL_DEBUG	1 //  debug mode ?

/*-----------------------------------------------------------*/
// Variables locales:

volatile bool_t l_isFirstCycle = FALSE, l_pedirDatosAlHost = FALSE,
	l_medirNivelEnEsteCiclo = FALSE, l_descargaDetectada = FALSE;

/*-----------------------------------------------------------*/

/*** Accessors para la variables locales ***/

// Getter para la variable local 'l_is_FirstCycle'
bool_t Control_getIsFirstCicle(void)
{
	return l_isFirstCycle;
}

// Setter para la variable local 'l_is_FirstCycle'
void Control_setIsFirstCicle(bool_t value)
{
	l_isFirstCycle = value;
}

// Getter para la variable local 'l_pedir_DatosAlHost'
bool_t Control_getPedirDatosAlHost(void)
{
	return l_pedirDatosAlHost;
}

// Setter para la variable local 'l_pedir_DatosAlHost'
void Control_setPedirDatosAlHost(bool_t value)
{
	l_pedirDatosAlHost = value;
}

// Getter para la variable local 'l_medir_nivel_en_este_ciclo'
bool_t Control_getMedirNivelEnEsteCiclo(void)
{
	return l_medirNivelEnEsteCiclo;
}

// Setter para la variable local 'l_medir_nivel_en_este_ciclo'
void Control_setMedirNivelEnEsteCiclo(bool_t value)
{
	l_medirNivelEnEsteCiclo = value;
}

// Getter para la variable local 'l_descarga_Detectada'
bool_t Control_getDescargaDetectada(void)
{
	return l_descargaDetectada;
}

// Setter para la variable local 'l_descarga_Detectada'
void Control_setDescargaDetectada(bool_t value)
{
	l_descargaDetectada = value;
}

/*-----------------------------------------------------------*/

/** Pasa la unidad sensora a modo 'deep power-down' en el que solo
 *  queda alimentado y en funcionamiento el reloj de tiempo real.
 *  Todos los demás datos se pierden.
 *
 * @param None
 *
 * @return None
 */

static void _systemToSleep(void)
{
	// Programamos la alarma del RTC para el próximo wake-up
	// Si SW2 está activado, dividimos el tiempo en power-down por 6
	// para hacer pruebas más rápido (solo para pruebas 1 minuto -> 10 segundos)
	uint32_t powerDownInterval = (uint32_t) POWER_DOWN_INTERVAL;
	if(!GPIO_getValue(PCB_TEST_SWITCH)) powerDownInterval /= 6;

	Rtc_programNextWakeUp(powerDownInterval);

	if(CONTROL_DEBUG)
	{
		Log_log("Control(): Entrando en DEEP POWER-DOWN...\r\n\(El sistema se recuperará en %d segundos)\r\n\r\n", powerDownInterval);
		Log_log("------------------------------------------------\r\n");
	}

	Led_ledOff(LED_2); // solo para pruebas

	/*** Entramos en modo deep power-down ***/
	Power_sleep(LPC_PMU, PMU_MCU_DEEP_PWRDOWN);
}
/*-----------------------------------------------------------*/

void Control_manager(ACTUAL_DATA *nivel, ACTUAL_DATA *temperatura, ACTUAL_DATA *verticalidad)
{
	 /*** Los envíos de datos al servidor se hacen: ***/

//	 - En el ciclo inicial de alimentación general (bateria off->on), después de haber hecho
//	   una petición de datos de ajuste y de haberlos recibido.
//
//	   Datos enviados:
//	   ---------------
//	   Nivel: Valor
//	   Temperatura: Valor y estado de la alarma
//	   Verticalidad: Valor y estado de la alarma (byte de alarmas: 0b 0000 00X0)
//
//	 - En cada intérvalo (enviado desde el host) de lectura del Nivel.
//
//	   Datos enviados:
//	   ---------------
//	   Nivel: Valor
//	   Temperatura: Valor y estado de la alarma (byte de alarmas: 0b 0000 000X)
//	   Verticalidad: Valor y estado de la alarma (byte de alarmas: 0b 0000 00X0)
//
//	 - En cada intérvalo de lectura de la temperatura, solamente SI hay cambio de estado
//	   (flanco positivo o negativo) en la alarma de temperatura.
//
//	   Datos enviados:
//	   ---------------
//	   Temperatura: Valor y estado de la alarma (byte de alarmas: 0b 0000 000X)
//
//	 - En cada intérvalo de lectura de la verticalidad, solamente SI hay cambio de estado
//	   (flanco positivo o negativo) en la alarma de verticalidad.
//
//	   Datos enviados:
//	   ---------------
//	   Verticalidad: Valor y estado de la alarma (byte de alarmas: 0b 0000 00X0)
//
//	 - Cuando se detecta que se ha descargado el contenedor.
//
//	   Datos enviados:
//	   ---------------
//	   Nivel: Valor
//	   Descarga: (byte de descarga: 0b 0000 0001)

	/*** Las peticiones de datos al servidor se hacen: ***/

//	 - En el ciclo inicial de alimentación general (bateria off->on).
//	 - En cada intérvalo (enviado desde el host) de actualización de datos de ajuste (estos datos)
//
//	   Datos enviados en la petición:
//     ------------------------------
//	   Intérvalo para nueva petición de datos: clave del campo ('get_per_actualiza')
//	   Intérvalo de lectura de nivel: clave del campo ('get_per_nivel')


	if(l_pedirDatosAlHost || l_medirNivelEnEsteCiclo ||
	    temperatura->alarmFlange || verticalidad->alarmFlange || l_descargaDetectada)
	{
		// Si llegamos a este punto, es porque hay algo que enviar al Host: petición de ajustes o datos de proceso
		// Habilitamos el Wifly.

		bool_t success = FALSE;

		// si es el primer ciclo después de alimentación, iniciamos el wifly
		if(l_isFirstCycle == TRUE)
		{
			// Wifly init
			if(CONTROL_DEBUG) Log_log("*** Control(): Inicializando el Wifly ***\r\n");

			success = Wifly_init();

			if(CONTROL_DEBUG)
			{
				if(success)	Log_log("Inicializado con éxito\r\n");
				else Log_log("No se ha podido inicializar el WiFly\r\n");
			}
		}
		else 	// en caso contrario, le despertamos
		{
			if(CONTROL_DEBUG) Log_log("*** Control(): Esperando el wake-up del Wifly ***\r\n");

			success = Wifly_wakeUp();

			if(CONTROL_DEBUG)
			{
				if(success)	Log_log("Despertado con éxito\r\n");
				else Log_log("No se ha podido despertar al WiFly\r\n");
			}
		}

		// Si el Wifly está listo
		if(success)
		{
			// Si es primer ciclo después de alimentación o ha expirado el intérvalo de actualización
			// de datos de ajuste, se los pedimos al host
			if(l_pedirDatosAlHost)
			{
				l_pedirDatosAlHost = FALSE;

				if(CONTROL_DEBUG) Log_log("*** Pidiendo datos al servidor... ***\r\n");

				uint16_t per_actualiza_datos = 0, per_actualiza_nivel = 0;

				// petición al host
				success = Host_request(&per_actualiza_datos, &per_actualiza_nivel);

				if(CONTROL_DEBUG)
				{
					if(success == TRUE)	Log_log("Datos recibidos correctamente.\r\n");
					else Log_log("Error de recepción de los datos.\r\n");
				}

				if(success && (per_actualiza_datos > 0) && (per_actualiza_nivel > 0))
				{
					// Reseteamos los intérvalos base acumulados para petición al host
					// contando con el presente ciclo (1)
					ROM_SET_CICLOS_DATOS(1);

					// salvamos los nuevos periodos recibidos desde el host
					ROM_SET_PER_DATOS(per_actualiza_datos);
					ROM_SET_PER_NIVEL(per_actualiza_nivel);
				}
				else { 	// si NO hemos obtenido los datos desde el host,
						// en el siguiente ciclo activo se volverá a intentar de nuevo

					// hemos hecho la petición de datos de ajuste al host o porque ha expirado
					// el intérvalo de petición o porque es el primer ciclo después de alimentación.
					// En el primer caso, se volverá a hacer la petición en el siguiente periodo activo
					// pero en el caso de primer ciclo no se volvería a hacer la petición ya que
					// el siguiente periodo activo ya no será primer ciclo después de alimentación.
					//
					// Forzamos a hacer la petición en el siguiente ciclo activo igualando
					// el número de ciclos base acumulados a los ciclos programados para hacer
					// una actualización de datos

					// obtenemos el ajuste de periodo de petición de datos al host
					uint32_t periodo_peticion_host = ROM_GET_PER_DATOS();

					// igualamos la cuenta de intérvalos base acumulados para petición al host
					// al ajuste de periodo de petición al host, de esta forma en el siguiente
					// ciclo se volverán a pedir
					ROM_SET_CICLOS_DATOS(periodo_peticion_host);
				}
			}
			else success = TRUE;

			// Si ya tenemos los datos del host o no ha sido necesario pedirlos...
			if(success)
			{
				// enviar los datos de los distintos sensores al host

				uint8_t byte_alarma = ((verticalidad->alarm & 0x01) << 1) | (temperatura->alarm & 0x01);

				if(CONTROL_DEBUG) Log_log("*** Enviando datos al servidor... ***\r\n");

				success = Host_send(nivel->value, temperatura->value, verticalidad->value,
									byte_alarma, l_descargaDetectada, TRUE);

				if(CONTROL_DEBUG)
				{
					if(success == TRUE)	Log_log("Datos enviados correctamente.\r\n");
					else Log_log("No se han podido enviar los datos.\r\n");
				}

			}

			// Una vez transmitidos y recibidos los datos, ponemos al wifly en modo sleep
			Wifly_enterSleep();
		}
	}

	// Ha concluido la gestión, pasamos la unidad sensora a modo 'deep power-down'
	_systemToSleep();

}

/*-----------------------------------------------------------*/


