/*
 *	host.c
 *
 *  Created on: 21/10/2015
 *      Author: Miguel A. Fdez.
 */

#include "FreeRTOS.h"
#include "semphr.h"
#include "LPC17xx.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "type.h"
#include "uart.h"
#include "log.h"
#include "printF.h"
#include "utils.h"
#include "wifly.h"
#include "control.h"
#include "host.h"

#define HOST_DEBUG	0 // Debug mode ?

// Local strings to work with this module
char l_hostRequestStr[UART_TX_BUFSIZE];
char l_hostResponseStr[UART_RX_BUFSIZE];
char l_jokerStr[128];

// delimitador para trabajar con los pares clave/valor
const char fieldsDelimiter[2] = {'&', '\0'};

// The time to wait for the semaphore in ticks.
// Defined in host.h in milliseconds.
volatile portTickType l_host_semaphore_Take_Ticks = 0;

// Semáforo para host
volatile xSemaphoreHandle l_host_Mutex = NULL;

// structura para trabajar con los pares claves/valor
#define LENGTH_CLAVE_OR_VALOR	32
typedef struct
{
	char clave[LENGTH_CLAVE_OR_VALOR];
	char valor[LENGTH_CLAVE_OR_VALOR];
	bool_t esValido;

} clave_valor_t;

/*-----------------------------------------------------------*/

bool_t Host_send(int16_t nivel, int16_t temperatura, int16_t verticalidad,
		         uint8_t alarma, uint8_t es_recogida, bool_t solicitarAck)
{
	bool_t success = FALSE;

	// Si aún no se ha iniciado el semáforo, se inicia
	if(l_host_Mutex == NULL) l_host_Mutex = xSemaphoreCreateMutex();

	// Si aún no se ha iniciado el tiempo de espera del semáforo en ticks, se inicia
	if(l_host_semaphore_Take_Ticks == 0)
		l_host_semaphore_Take_Ticks = (portTickType) HOST_SEMAPHOR_TAKE_WAIT_MS / portTICK_RATE_MS;

	if(xSemaphoreTake(l_host_Mutex, l_host_semaphore_Take_Ticks) == pdTRUE)
	{
		uint32_t ip = Utils_ipStrToUint32(WIFLY_IP_ADDR);

		// construimos el envío al host
		memset(l_jokerStr, 0x00, sizeof(l_jokerStr));

		// Si se ha medido el nivel en este ciclo, se envia el dato de nivel
		// en caso contrario no se envía
		if(Control_getMedirNivelEnEsteCiclo())
		{
			snprintf(l_jokerStr, sizeof(l_jokerStr),
					 "dir_ip=%u&nivel=%d&temper=%d&vertic=%d&alarm=%u&es_recog=%u&ack=%d",
					 ip, nivel, temperatura, verticalidad, alarma, es_recogida, solicitarAck);
		}
		else
		{
			snprintf(l_jokerStr, sizeof(l_jokerStr),
					 "dir_ip=%u&temper=%d&vertic=%d&alarm=%u&es_recog=%u&ack=%d",
					 ip, temperatura, verticalidad, alarma, es_recogida, solicitarAck);
		}

		// Calculamos el CRC con los datos adecuados (ver memoria del PFC).
		// Polinomio CRC 16 CCITT. Ecuación: x^16 + x^12 + x^5 + 1. XMODEM: valor inicial = 0x0000.
		uint16_t crc = Utils_crc16_ccitt(l_jokerStr, strlen(l_jokerStr), 0x0000);

		// completamos la cadena de envío al host incluyendo el CRC calculado
		memset(l_hostRequestStr, 0x00, sizeof(l_hostRequestStr));
		snprintf(l_hostRequestStr, sizeof(l_hostRequestStr), "send&%s&crc=%u", l_jokerStr, crc);

		// enviamos la cadena al host y recogemos la respuesta desde el mismo
		memset(l_hostResponseStr, 0x00, sizeof(l_hostResponseStr));
		success = Wifly_sendHttp(l_hostRequestStr, l_hostResponseStr);

		if(HOST_DEBUG) Log_log(l_hostResponseStr);

		// si el proceso de petición-respuesta con el host ha ido bien y
		// hemos solicitado respuesta de reconocimiento...
		if(success && solicitarAck)
		{
			/*** comprobamos si el host ha devuelto el CRC correcto ***/

			// extraemos el CRC de la cadena html devuelta por el host
			memset(l_hostRequestStr, 0x00, sizeof(l_hostRequestStr));
			success = Utils_getValueByKey(l_hostResponseStr, fieldsDelimiter, "crc=", l_hostRequestStr);

			if(success == TRUE)
			{
				// comprobamos si el host ha devuelto el mismo CRC que enviamos con los datos
				memset(l_jokerStr, 0x00, sizeof(l_jokerStr));
				snprintf(l_jokerStr, sizeof(l_jokerStr), "%u",  crc); // crc enviado al host

				if(strcmp(l_jokerStr, l_hostRequestStr) != 0) success = FALSE;
			}

			if(HOST_DEBUG) Log_log(l_hostRequestStr);
		}

		xSemaphoreGive(l_host_Mutex);
	}

	return success;
}
/*-----------------------------------------------------------*/

bool_t Host_request(uint16_t *per_actualiza_datos, uint16_t *per_actualiza_nivel)
{
	bool_t success = FALSE;

	// Si aún no se ha iniciado el semáforo, se inicia
	if(l_host_Mutex == NULL) l_host_Mutex = xSemaphoreCreateMutex();

	// Si aún no se ha iniciado el tiempo de espera del semáforo en ticks, se inicia
	if(l_host_semaphore_Take_Ticks == 0)
		l_host_semaphore_Take_Ticks = (portTickType) HOST_SEMAPHOR_TAKE_WAIT_MS / portTICK_RATE_MS;

	if(xSemaphoreTake(l_host_Mutex, l_host_semaphore_Take_Ticks) == pdTRUE)
	{
		uint32_t ip = Utils_ipStrToUint32(WIFLY_IP_ADDR);

		// construimos la petición al host
		memset(l_jokerStr, 0x00, sizeof(l_jokerStr));
		snprintf(l_jokerStr, sizeof(l_jokerStr),
				 "dir_ip=%u&get_per_actualiza=1&get_per_nivel=1", ip);

		// Calculamos el CRC con los datos adecuados (ver memoria del PFC).
		// Polinomio CRC 16 CCITT. Ecuación: x^16 + x^12 + x^5 + 1. XMODEM: valor inicial = 0x0000.
		uint16_t crc = Utils_crc16_ccitt(l_jokerStr, strlen(l_jokerStr), 0x0000);

		// completamos la cadena de envío al host incluyendo el CRC calculado
		memset(l_hostRequestStr, 0x00, sizeof(l_hostRequestStr));
		snprintf(l_hostRequestStr, sizeof(l_hostRequestStr), "request&%s&crc=%u", l_jokerStr, crc);

		// enviamos la cadena al host y recogemos la respuesta desde el mismo
		memset(l_hostResponseStr, 0x00, sizeof(l_hostResponseStr));
		success = Wifly_sendHttp(l_hostRequestStr, l_hostResponseStr);

		if(HOST_DEBUG) Log_log(l_hostResponseStr);

		// si el proceso de petición-respuesta con el host ha ido bién
		if(success == TRUE)
		{
			// almacenamiento para los 4 posibles clave-valore recibidos desde el host
			clave_valor_t clave_valor[4];

			// inicialización de las claves
			strcpy(clave_valor[0].clave, "dir_ip=");
			strcpy(clave_valor[1].clave, "set_per_actualiza=");
			strcpy(clave_valor[2].clave, "set_per_nivel=");
			strcpy(clave_valor[3].clave, "crc=");

			// obtenemos los valores enviados por el host y vamos construyendo la cadena que posteriormente
			// nos servirá para calcular el CRC recibido
			memset(l_jokerStr, 0x00, sizeof(l_jokerStr));

			uint8_t i;
			for(i=0; i<4; i++)
			{
				memset(clave_valor[i].valor, 0x00, sizeof(clave_valor[i].valor));
				clave_valor[i].esValido = FALSE;

				memset(l_hostRequestStr, 0x00, sizeof(l_hostRequestStr));
				success = Utils_getValueByKey(l_hostResponseStr, fieldsDelimiter, clave_valor[i].clave, l_hostRequestStr);

				if((success == TRUE) && (strlen(l_hostRequestStr) < LENGTH_CLAVE_OR_VALOR))
				{
					strcpy(clave_valor[i].valor, l_hostRequestStr);
					clave_valor[i].esValido = TRUE;

					// cadena para el cálculo del CRC

					if(i != 3)	// el CRC recibido no se incluye para el cálculo
					{
						strcat(l_jokerStr, "&");
						strcat(l_jokerStr, clave_valor[i].clave);
						strcat(l_jokerStr, clave_valor[i].valor);
					}
				}
			}

			// hemos recibido todos los datos ???
			success = TRUE;
			for(i=0; i<4; i++)
			{
				if(clave_valor[i].esValido == FALSE)
				{
					success = FALSE;
					break;
				}
			}

			// hemos recibido todos los datos
			if(success == TRUE)
			{
				// Calculamos el CRC con los datos recibidos (ver memoria del PFC).
				// Polinomio CRC 16 CCITT. Ecuación: x^16 + x^12 + x^5 + 1. XMODEM: valor inicial = 0x0000.
				uint16_t crc = Utils_crc16_ccitt(l_jokerStr, strlen(l_jokerStr), 0x0000);

				// comprobamos si es igual que el CRC recibido
				if(crc == atoi(clave_valor[3].valor))
				{
					// valor del periodo de actualización de datos
					uint16_t perActualizaDatos = (uint16_t) (atoi(clave_valor[1].valor));
					// valor del periodo de actualización de nivel
					uint16_t perActualizaNivel = (uint16_t) (atoi(clave_valor[2].valor));

					// atoi() devuelve 0 si no puede hacer la conversión
					// si ambos valores son correctos se devuelven en los parámetros de salida de esta función
					if((perActualizaDatos > 0) && (perActualizaNivel > 0))
					{
						*per_actualiza_datos = perActualizaDatos;
						*per_actualiza_nivel = perActualizaNivel;
					}
					else success = FALSE;
				}
				else success = FALSE;
			}
		}

		xSemaphoreGive(l_host_Mutex);
	}

	return success;
}
/*-----------------------------------------------------------*/
