/*
 * nivel.c
 *
 *  Created on: 17/11/2015
 *      Author: Miguel A. Fdez.
 */

#include "FreeRTOS.h"
#include "semphr.h"
#include "LPC17xx.h"
#include <string.h>
#include <stdlib.h>
#include "AppConfig.h"
#include "utils.h"
#include "gpio.h"
#include "uart.h"
#include "printF.h"
#include "log.h"
#include "rtc.h"
#include "nivel.h"

#define NIVEL_DEBUG	0 //  debug mode ?

/*-----------------------------------------------------------*/

// The LPC1769 UART port to comunicate with the level sensor
volatile uint32_t l_level_uartPort = MAX_UART_PORTS;

// The time to wait for the semaphore in ticks.
// Defined in Nivel.h in milliseconds.
volatile portTickType l_level_semaphoreTake_Ticks = 0;

// The level sensor semaphore
volatile xSemaphoreHandle l_level_mutex = NULL;

// Variables locales utilizadas en Nivel_get(). No declaradas en la función
// para mejorar el consumo de pila (memoria).
char l_scannedString[UART_RX_BUFSIZE];

/*-----------------------------------------------------------*/

bool_t Nivel_init(uint32_t portNum, uint32_t baudrate)
{
	// Esta función no utiliza semáforo propio porque es la que le inicia y
	// o bien solo la llamaremos una vez a lo largo del programa, que sería
	// lo normal, o en caso necesario la llamaremos más de una vez desde código
	// secuencial, nunca desde Tasks.

	bool_t success = FALSE;

	if((l_level_mutex == NULL) && (portNum < MAX_UART_PORTS))  // En la primera llamada a la función...
	{
		// Inicialización del puerto uart en el LPC1769 para comunicación con el sensor de nivel
		success = PrintF_start(portNum, baudrate);

		if(success == TRUE) // Inicialización correcta del puerto uart del LPC1769 para comunicación con el sensor de nivel
		{
			// Puerto para el sensor de nivel. Evita pasar el puerto en funciones posteriores.
			l_level_uartPort = portNum;

			// Se crea el semáforo
			l_level_mutex = xSemaphoreCreateMutex();

			// Conversión a ticks del tiempo de espera para obtener el semáforo
			l_level_semaphoreTake_Ticks = (portTickType) NIVEL_SEMAPHORE_TAKE_WAIT_MS / portTICK_RATE_MS;

			// Habilitamos la alimentación (ya es Output desde 'Nivel_powerOff()')
			GPIO_setValue(NIVEL_POWER_PIN, TRUE);

			// Espera inicial necesaria de 250 mS después de alimentación para sensor ready (ver hoja datos sensor)
			Utils_delayMs(FALSE, 250);

			// Medida inicial de calibración (Necesaria. Ver hoja de datos del sensor).
			GPIO_setValue(NIVEL_READ_PIN, TRUE);
			Utils_delayMs(FALSE, 1); // mínimo 20 uS
			GPIO_setValue(NIVEL_READ_PIN, FALSE);

			// Espera para la medida de calibración
			Utils_delayMs(FALSE, NIVEL_UART_TIMEOUT);
		}
	}

	return success;
}
/*-----------------------------------------------------------*/

void Nivel_powerOff(void)
{
	// Debido a que por defecto, al alimentar al uP, las GPIO son entradas con pull-up,
	// en los ciclos en que NO necesitamos alimentar al sensor de nivel iniciamos
	// los pins como salida y los ponemos a 0.
	// Si los dejasemos por defecto como entradas con pull-up, debido al voltaje
	// presente en la entrada del pin vía pull-up, la alimetación se conectaría.

	// Inicialización del pin de control de la alimentación del sensor de nivel
	GPIO_setDir(NIVEL_POWER_PIN, GPIO_OUTPUT);
	// Deshabilitamos la alimentación
	GPIO_setValue(NIVEL_POWER_PIN, FALSE);

	// Pin para indicar al sensor cuando hacer una lectura
	GPIO_setDir(NIVEL_READ_PIN, GPIO_OUTPUT);
	// deshabilitamos la salida
	GPIO_setValue(NIVEL_READ_PIN, FALSE);
}

/*-----------------------------------------------------------*/

void Nivel_get(ACTUAL_DATA *nivel)
{
	bool_t alarmIsActive = FALSE;
	uint16_t i;
	char levelStr[4] = {'\0','\0','\0','\0'};
	float nivelActual = 0.0, nivelAnterior = 0.0;

	// En este proyecto, la llamada a esta función se hace siempre desde solo una 'task'
	// y por lo tanto no necesita semáforo.
	// No obstante se protege con semáforo para posibles usos posteriores desde múltiples 'tasks'.
	if(xSemaphoreTake(l_level_mutex, l_level_semaphoreTake_Ticks) == pdTRUE)
	{
		// Miramos si hay pérdida de verticalidad aunque no esté validada la alarma.
		uint32_t alarmasVertAcumuladas = ROM_GET_VERT_ALARM_ACCU_VALUE();

		// Leemos el nivel en el ciclo anterior para ver si se ha realizado una descarga
		// o para mantenerlo como nivel actual si se está realizando una descarga o si
		// se produce un error de lectura de nivel desde el sensor
		nivelAnterior = ROM_GET_NIVEL_VALUE() / 10.0;

		if(alarmasVertAcumuladas == 0) // Si NO hay perdida de verticalidad...
		{
			// Hacemos una medida
			GPIO_setValue(NIVEL_READ_PIN, TRUE);
			Utils_delayMs(FALSE, 1); // mínimo 20 uS
			GPIO_setValue(NIVEL_READ_PIN, FALSE);

			// Obtenemos la respuesta desde el sensor
			memset(l_scannedString, 0x00, sizeof(l_scannedString));
			uint32_t nbrChars = PrintF_scan(l_level_uartPort, l_scannedString, NIVEL_UART_TIMEOUT);

			// Deshabilitamos la alimentación
			GPIO_setValue(NIVEL_POWER_PIN, FALSE);

			// ¿ Hay respuesta correcta desde el sensor ?
			// La respuesta siempre es del tipo 'Rxxx\13', donde xxx es la distancia en pulgadas.
			if((nbrChars == 5) && (l_scannedString[0] == 'R') && (l_scannedString[4] == '\r'))
			{
				// Respuesta correcta -> ponemos a cero la cantidad de intervalos base acumulados para la medida del nivel
				ROM_SET_CICLOS_NIVEL(0);

				// Decodificamos la cadena recibida
				for(i=0; i<3; i++) levelStr[i] = l_scannedString[i+1];

				// La mínima medida que indica el sensor son 6" = 15 cm, aunque el objeto esté más cercano.
				// Por lo tanto una medida de 0" = 0 cm indica error en la lectura del sensor.
				nivelActual = atoi(levelStr) * 2.54; // el sensor entrega la medida en pulgadas

				// ajustamos los extremos de la medida
				if(nivelActual < 0.0) nivelActual = 0.0;
				else if(nivelActual > 300.0) nivelActual = 300.0; // en nuestro caso máximo 3 mts.

				// Comprobar si se ha realizado una descarga
				// En el primer ciclo de alimentación no se comprueba la descarga
				// porque aún no se tienen datos válidos de nivel anterior como referencia.
				if(Control_getIsFirstCicle() == FALSE)
				{
					// Si la diferencia entre el nivel disponible anterior en el contenedor y el actual
					// es mayor que la ajustada es porque se ha descargado el contenedor.
					if((nivelAnterior - nivelActual) > (float) NIVEL_DIFERENCIA_VALIDA_DESCARGA)
					{
						// Se ha producido una descarga y señalamos el flag de descarga detectada:
						Control_setDescargaDetectada(TRUE);
					}
				}

				// memorizamos el nivel de este ciclo, con signo: 0.0 a 300.0 cm
				ROM_SET_NIVEL_VALUE((int16_t)(nivelActual * 10.0));
			}
			else // No respuesta o respuesta incorrecta desde el sensor
			{
				alarmIsActive = TRUE;
			}
		}
		else // Si hay perdida de verticalidad...
		{
			// Los datos de nivel son irrelevantes, nivel actual = último nivel medido
			nivelActual = nivelAnterior;
		}

		if(NIVEL_DEBUG	)
		{
			Log_log("Cadena leida del sensor de distancia: %s\r\nNivel en centímetros: %d\r\nError de lectura: %d\r\n\r\n",
				     l_scannedString, nivelActual, alarmIsActive);
		}

		// Retornamos los datos en el parámetro pasado por referencia a esta función
		nivel->id = ID_NIVEL;
		nivel->value = (int16_t) (nivelActual * 10.0);	// coma fija con signo: 0.0 a 300.0 cm
		nivel->alarm = alarmIsActive; 	// Alarma por error de lectura del nivel. 1: Alarma, 0: No.
		nivel->alarmFlange = FALSE; 	// El flanco de la alarma no se utiliza en el nivel

		xSemaphoreGive(l_level_mutex);
	}

}
/*-----------------------------------------------------------*/


