/*
 * temperatura.c
 *
 *  Created on: 19/11/2015
 *      Author: Miguel A. Fdez.
 */

#include "FreeRTOS.h"
#include "semphr.h"
#include "LPC17xx.h"
#include "AppConfig.h"
#include "utils.h"
#include "gpio.h"
#include "log.h"
#include "rtc.h"
#include "adc.h"
#include "temperatura.h"

#define TEMP_DEBUG	0 //  debug mode ?

// número de elementos el la tabla de referencia
#define TEMP_LOOKUP_TABLE_SIZE	201

/*-----------------------------------------------------------*/

// The time to wait for the semaphore in ticks.
// Defined in Temperatura.h in milliseconds.
volatile portTickType l_temp_semaphoreTake_Ticks = 0;

// The temperature sensor semaphore
volatile xSemaphoreHandle l_temp_mutex = NULL;

// Tabla de referencia (lookup table) para calcular la temperatura a partir de la
// salida del sensor de temperatura (ver hoja de datos del sensor).
// Los valores en la matriz son mV y el índice en el que se encuentra indica los ºC.
// con un offset de -50ºC
const uint32_t l_tempLookupTable[TEMP_LOOKUP_TABLE_SIZE] =
{
	3277, // índice 0 = -50ºC
    3266,3254,3243,3232,3221,3210,3199,3186,3173,3160,3147,3134,3121,3108,3095,3082,3069,3056,3043,3030,3017,3004,2991,2978,2965,
	2952,2938,2925,2912,2899,2886,2873,2859,2846,2833,2820,2807,2793,2780,2767,2754,2740,2727,2714,2700,2687,2674,2660,2647,
	2633, // índice 50 = 0ºC
	2620,2607,2593,2580,2567,2553,2540,2527,2513,2500,2486,2473,2459,2446,2433,2419,2406,2392,2379,2365,2352,2338,2325,2311,2298,
	2285,2271,2258,2244,2231,2217,2204,2190,2176,2163,2149,2136,2122,2108,2095,2081,2067,2054,2040,2026,2012,1999,1985,1971,
	1958, // índice 100 = 50ºC
	1944,1930,1916,1902,1888,1875,1861,1847,1833,1819,1805,1791,1777,1763,1749,1735,1721,1707,1693,1679,1665,1651,1637,1623,1609,
	1595,1581,1567,1553,1539,1525,1511,1497,1483,1469,1455,1441,1427,1413,1399,1385,1371,1356,1342,1328,1314,1300,1286,1272,
	1257, // índice 150 = 100ºC
	1243,1229,1215,1201,1186,1172,1158,1144,1130,1115,1101,1087,1073,1058,1044,1030,1015,1001,987,973,958,944,929,915,901,
	886,872,858,843,829,814,800,786,771,757,742,728,713,699,684,670,655,640,626,611,597,582,568,553,
	538  // índice 200 = 150ºC
};

// Variables locales utilizadas en Temperatura_get(). No declaradas en la función
// para mejorar el consumo de memoria de pila.
double l_adcTempSamples[TEMP_ADC_SAMPLES], l_adcFilteredTempValue,
	   l_tempActual_mV, l_tempActualGrados, l_tempAnteriorGrados;

/*-----------------------------------------------------------*/

void Temperatura_init(void)
{
	// Esta función no utiliza semáforo propio porque es la que le inicia y
	// o bien solo la llamaremos una vez a lo largo del programa, que sería
	// lo normal, o en caso necesario la llamaremos más de una vez desde código
	// secuencial, nunca desde Tasks.

	if(l_temp_mutex == NULL) // En la primera llamada a la función...
	{
		// Se crea el semáforo
		l_temp_mutex = xSemaphoreCreateMutex();

		// Conversión a ticks del tiempo de espera para obtener el semáforo
		l_temp_semaphoreTake_Ticks = (portTickType) TEMP_SEMAPHORE_TAKE_WAIT_MS / portTICK_RATE_MS;

		// Inicialización de la salida del uP utilizada para alimentación del sensor de temperatura

		// No es necesario programar los pines del micro como entrada-salida de
		// proposito general (GPIO). Por defecto en el inicio ya son GPIO, entradas con pull-up.

		// Programar el correspondiente pin del micro como salida:

		// Pin de salida para alimentación del sensor de temperatura
		GPIO_setDir(TEMP_POWER_PIN, GPIO_OUTPUT);
		// Deshabilitamos la alimentación del sensor
		GPIO_setValue(TEMP_POWER_PIN, FALSE);
	}
}

/*-----------------------------------------------------------*/

void Temperatura_get(ACTUAL_DATA *temperatura)
{
	uint16_t i, alarmState = 0U, alarmFlange = 0U;
	bool_t alarmIsActive = FALSE;

	// En este proyecto, la llamada a esta función se hace siempre desde solo una 'task'
	// y por lo tanto no necesita semáforo.
	// No obstante se protege con semáforo para posibles usos posteriores desde múltiples 'tasks'.
	if(xSemaphoreTake(l_temp_mutex, l_temp_semaphoreTake_Ticks) == pdTRUE)
	{
		// Habilitamos la alimentación del sensor
		GPIO_setValue(TEMP_POWER_PIN, TRUE);

		// 2 milisegundos de espera después de alimentación (ver hoja de datos, pag.6).
		Utils_delayMs(FALSE, 2);

		// Se lee el canal analógico que contiene el valor actual de la temperatura.
		// Se hacen 'TEMP_ADC_SAMPLES' lecturas del canal.
		for(i=0; i<TEMP_ADC_SAMPLES; i++) l_adcTempSamples[i] = (double)Adc_read(TEMP_ANALOG_CHANNEL);

		// Deshabilitamos la alimentación del sensor
		GPIO_setValue(TEMP_POWER_PIN, FALSE);

		// Se filtran las lecturas
		l_adcFilteredTempValue = Utils_simpleFilter(&l_adcTempSamples[0], TEMP_ADC_SAMPLES);

		// Calculamos los mV que envía el sensor de temperatura
		l_tempActual_mV = l_adcFilteredTempValue * ADC_REF /  (1 << ADC_PREC_BITS);

		// Temperatura actual:

		// temperatura actual = temperatura máxima ?
		if(l_tempActual_mV <= l_tempLookupTable[TEMP_LOOKUP_TABLE_SIZE-1]) l_tempActualGrados = 150.0;
		// temperatura actual = temperatura mínima ?
		else if(l_tempActual_mV >= l_tempLookupTable[0]) l_tempActualGrados = -50.0;
		// calculamos la temperatura actual
		else
		{
			// recorremos la tabla comenzando por el primer elemeto hasta encontrar
			// un valor en mV inferior al obtenido del conversor A/D
			for(i=0; i<TEMP_LOOKUP_TABLE_SIZE; i++)
			{
				if(l_tempLookupTable[i] < l_tempActual_mV) break;
			}

			// obtenemos el valor inferior (el encontrado) y el superior (el anterior al encontrado) en temperatura y en mV,
			// teniendo en cuenta que la tabla tiene un offset de -50ºC.
			int32_t temp_inferior = i-50, mV_inferior = l_tempLookupTable[i], mV_superior = l_tempLookupTable[i-1];

			// hacemos una interpolación lineal para aproximar más la temperatura:

			// mV por cada grado en el rango actual (1 grado).
			double mV_grado = (double)mV_superior - (double)mV_inferior;
			// diferencia en mV entre el valor leido por el conversor A/D y el valor inferior (el encontrado)
			double diferencia_mV = l_tempActual_mV - (double) mV_inferior;

			// valor actual de la temperatura (Cuantos más mV, menos temperatura. De ahí el signo menos.)
			l_tempActualGrados = (double)temp_inferior - (diferencia_mV / mV_grado);
		}

		// Determinamos si hay alarma por temperatura:

		// no elaboramos la alarma en el primer ciclo ya que no tenemos
		// aún la temperatura del ciclo anterior como referencia.
		if(Control_getIsFirstCicle() == FALSE)
		{
			// obtenemos la temperatura en el ciclo anterior
			// almacenada como coma fija con signo: -50.0 a 150.0 ºC
			l_tempAnteriorGrados = (double)ROM_GET_TEMPERATURE_VALUE() / 10.0;

			// leemos el estado de la alarma del ciclo anterior para ver si en el ciclo actual ha cambiado de estado
			// en cuyo caso, se genera el flanco
			alarmState = ROM_GET_TEMPERATURE_ALARM();

			// Siempre que desde la anterior llamada el incremento de temperatura
			// sea superior al prefijado, activamos la alarma.
			if((l_tempActualGrados - l_tempAnteriorGrados) > ((double)TEMP_ALARM_THRESHOLD)) alarmIsActive = TRUE;
			// en otro caso, si la temperatura desciende de 50 ºC desactivamos la alarma.
			else if((alarmState == TRUE) && (l_tempActualGrados < ((double)TEMP_ALARM_DEACTIVATE))) alarmIsActive = FALSE;

			alarmFlange = ((alarmState & (uint16_t)TEMP_ALARM_MASK) != (uint16_t)alarmIsActive);
		}

		// memorizamos la temperatura de este ciclo
		//almacenado como coma fija con signo: -50.0 a 150.0 ºC
		ROM_SET_TEMPERATURE_VALUE((int16_t)(l_tempActualGrados * 10.0));

		// memorizamos el estado de la alarma en este ciclo
		alarmState &= (~ ((uint16_t)TEMP_ALARM_MASK));
		if(alarmIsActive == TRUE) alarmState |= ((uint16_t)TEMP_ALARM_MASK);
		ROM_SET_TEMPERATURE_ALARM(alarmState);

		if(TEMP_DEBUG)
		{
			// The temperature's actual value in mV, in ºC and the alarm
			Log_log("Milivoltios leidos: %f\r\nTemperatura actual: %f\r\nAlarma temperatura activa: %d\r\n\r\n",
					l_tempActual_mV, l_tempActualGrados, alarmIsActive);
		}

		// Retornamos los datos en el parámetro pasado por referencia a esta función
		temperatura->id = ID_TEMPERATURA;
		temperatura->value = (int16_t) (l_tempActualGrados * 10.0);  // coma fija con signo: -50.0 a 150.0 ºC
		temperatura->alarm = alarmIsActive;
		temperatura->alarmFlange = (bool_t) alarmFlange;

		xSemaphoreGive(l_temp_mutex);
	}

}
/*-----------------------------------------------------------*/

