/*
 * verticalidad.c
 *
 *  Created on: 23/11/2015
 *      Author: Miguel A. Fdez.
 */

#include "FreeRTOS.h"
#include "semphr.h"
#include "LPC17xx.h"
#include "AppConfig.h"
#include "utils.h"
#include <math.h>
#include "gpio.h"
#include "log.h"
#include "rtc.h"
#include "adc.h"
#include "verticalidad.h"

#define VERTICALIDAD_DEBUG	0 //  debug mode ?

/*-----------------------------------------------------------*/

// Debido a la amplia tolerancia de la señal de salida (ver hoja de datos del acelerómetro)
// ajustamos un offset en cada eje para mejorar la precisión de la señal de salida
// del acelerómetro con el que estamos trabajando
const double l_acelOffset[3] = {-0.13, -0.18, 0.04}; // offset(g) in axis x, y, z.

// The time to wait for the semaphore in ticks.
// Defined in Verticalidad.h in milliseconds.
volatile portTickType l_acel_semaphoreTake_Ticks = 0;

// The verticalidad semaphore
volatile xSemaphoreHandle l_acel_mutex = NULL;

// Variables locales utilizadas en Verticalidad_get(). No declaradas en la función
// para mejorar el consumo de pila (memoria).
double l_adcAxisValues[3][ACEL_ADC_SAMPLES], l_adcFilteredAxisValues[3], l_gAxis[3];

/*-----------------------------------------------------------*/

void Verticalidad_init(void)
{
	// Esta función no utiliza semáforo propio porque es la que le inicia y
	// o bien solo la llamaremos una vez a lo largo del programa, que sería
	// lo normal, o en caso necesario la llamaremos más de una vez desde código
	// secuencial, nunca desde Tasks.

	if(l_acel_mutex == NULL) // En la primera llamada a la función...
	{
		// Se crea el semáforo
		l_acel_mutex = xSemaphoreCreateMutex();

		// Conversión a ticks del tiempo de espera para obtener el semáforo
		l_acel_semaphoreTake_Ticks = (portTickType) ACEL_SEMAPHORE_TAKE_WAIT_MS / portTICK_RATE_MS;

		// No es necesario programar los pines del micro como entrada-salida de
		// proposito general (GPIO). Por defecto en el inicio ya son GPIO, entradas con pull-up.

		// Programar los pines del micro como salidas:

		// Sleep pin
		GPIO_setDir(ACEL_SLEEP_PIN, GPIO_OUTPUT);
		// Sleep
		GPIO_setValue(ACEL_SLEEP_PIN, FALSE);

		// Self-test pin
		GPIO_setDir(ACEL_SELFTEST_PIN, GPIO_OUTPUT);
		// No self-test
		GPIO_setValue(ACEL_SELFTEST_PIN, FALSE);

		// 2 milisegundos de espera después de alimentación (ver hoja de datos, pag.3).
		Utils_delayMs(FALSE, 2);
	}
}

/*-----------------------------------------------------------*/

void Verticalidad_get(ACTUAL_DATA *verticalidad)
{
	uint16_t i, j, alarmState = 0U, alarmFlange = 0U, alarmasAcumuladas = 0U;
	bool_t alarmIsActive = FALSE;

	// En este proyecto, la llamada a esta función se hace siempre desde solo una 'task'
	// y por lo tanto no necesita semáforo.
	// No obstante se protege con semáforo para posibles usos posteriores desde múltiples 'tasks'.
	if(xSemaphoreTake(l_acel_mutex, l_acel_semaphoreTake_Ticks) == pdTRUE)
	{
		// Habilitar acelerómetro
		GPIO_setValue(ACEL_SLEEP_PIN, TRUE);

		// 2 milisegundos de espera después de habilitación (ver hoja de datos, pag.3).
		Utils_delayMs(FALSE, 2);

		// Se leen los canales analógicos 0, 1 y 2 que contienen respectivamente
		// los valores de los ejes x, y, z.
		// Se leen los 3 canales con propósitos didacticos. En realidad solamente es
		// necesario leer el canal correspondiente al eje Z (ver memória del proyecto).
		for(i=0; i<3; i++) // 3 canales: x, y, z
		{
			// Se hacen 'ACEL_ADC_SAMPLES' lecturas de cada canal
			for(j=0; j<ACEL_ADC_SAMPLES; j++) l_adcAxisValues[i][j] = (double)Adc_read((uint8_t)i);

			// Se filtran las lecturas de cada canal
			l_adcFilteredAxisValues[i] = Utils_simpleFilter(l_adcAxisValues[i], ACEL_ADC_SAMPLES);

			// Se calcula el valor de 'g' para cada canal:
			//
			//            (filteredAxisValues[canal] * mV Ref conversor A-D / resolución del conversor A-D) - mV(0g) acelerómet.
			// g[canal] = ------------------------------------------------------------------------------------------------------ + offsetAjuste[canal]
			//                                             sensibilidad del acelerómetro (mV/g)

			l_gAxis[i] = (((l_adcFilteredAxisValues[i] * (double) ADC_REF / (double) (1 << ADC_PREC_BITS)) -
					      (double) ACEL_0G) / (double) ACEL_SENSIVITY) + l_acelOffset[i];
		}

		// Deshabilitar acelerómetro
		GPIO_setValue(ACEL_SLEEP_PIN, FALSE);

		// Determinamos si hay alarma de verticalidad. Para ello utilizamos
		// solamente el eje 'z' (ver justificación en la memoria)
		// Además validamos la alarma solamente después de un número de ciclos de
		// lectura determinados para evitar generarla durante la descarga
		// del contenedor (ver memoria).

		// no elaboramos la alarma en el primer ciclo ya que no tenemos aún un
		// valor válido de número de alarmas consecutivas anteriores como referencia.
		if(Control_getIsFirstCicle() == FALSE)
		{
			// número de alarmas consecutivas de verticalidad acumuladas ?
			alarmasAcumuladas = ROM_GET_VERT_ALARM_ACCU_VALUE();

			// si hay alarma, incrementamos en uno el contador de ciclos con alarma activa
			// la alarma se calcula solo sobre el eje Z (ver justificación en la memoaria del PFC)
			if(l_gAxis[2] < ((double) VERT_SEN_ANGULO_ALARMA)) alarmasAcumuladas ++;
			else alarmasAcumuladas = 0; // en la primera lectura sin alarma, reseteamos el contador de ciclos con alarma activa

			// Validamos la alarma si alcanzamos el numero de ciclos con alarma activa necesarios.
			if(alarmasAcumuladas >= VERT_CICLOS_ALARMA_ACTIVA)
			{
				alarmasAcumuladas = VERT_CICLOS_ALARMA_ACTIVA; // evitar desbordamiento
				alarmIsActive = TRUE;
			}

			// leemos el estado de la alarma en el ciclo anterior para ver si en el ciclo actual ha cambiado de estado
			// y generar el flanco
			uint16_t alarmState = ROM_GET_VERT_ALARM();
			alarmFlange = ((alarmState & VERT_ALARM_MASK) != alarmIsActive);
		}

		// memorizamos las alarmas acumuladas en este ciclo
		// (en el ciclo inicial, se ponen a 0).
		// Ver inicialización de 'alarmasAcumuladas' al principio de esta función
		ROM_SET_VERT_ALARM_ACCU_VALUE(alarmasAcumuladas);

		// memorizamos el estado de la alarma en este ciclo
		alarmState &= (~VERT_ALARM_MASK);
		if(alarmIsActive) alarmState |= VERT_ALARM_MASK;
		ROM_SET_VERT_ALARM(alarmState);

		if(VERTICALIDAD_DEBUG)
		{
			// The 'g' value in the three axis and the alarm state.
			Log_log("Eje X: %f\r\nEje Y: %f\r\nEje Z: %f\r\nAlarma verticalidad activa: %d\r\n\r\n",
					l_gAxis[0], l_gAxis[1], l_gAxis[2], alarmIsActive);
		}

		// Retornamos los datos en el parámetro pasado por referencia a esta función
		// (retornamos solo la g del eje z. Ver justificación en la memoria)
		verticalidad->id = ID_VERTICALIDAD;
		// paso de radianes a grados x 10 (coma fija con signo: -180.0 a 180.0 º)
		// utilizamos el arcos -> contenedor vertical = 0º (visualizamos ángulo de desvío)
		verticalidad->value = (int16_t) (acos(l_gAxis[2]) * 1800.0 / (double) M_PI);
		verticalidad->alarm = alarmIsActive;
		verticalidad->alarmFlange = (bool_t) alarmFlange;

		xSemaphoreGive(l_acel_mutex);
	}

}

/*-----------------------------------------------------------*/
